#=================================================
#
#  admin.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package admin;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
ADMIN_main
);
use strict;
use integer;

use glo::env;
use glo::list;
use glo::ask;
use glo::scm;
use gbsenv;
use gbsscm;
use gbssfile;
use scmsmaint;
use system;
use validate;
use plugin;
use upgrade;




sub ADMIN_main($$$);

sub do_user_templates($$$);
sub check_type($$$);

sub fix_system_gbs($$$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SCRIPTS_PATH = ENV_getenv_perl_path( 'GBS_SCRIPTS_PATH');




sub ADMIN_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

my @admin_menu_items = (
[ "Plugins (Build/Audit/Tools): Install/Replace/Remove",
\&PLUGIN_dialog ],
[ "Create User Templates",
\&do_user_templates ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "SCMS Tools",
\&SCMSMAINT_main ],
[ "Fix system.gbs",
\&fix_system_gbs ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "Not used",
sub { ENV_say( 1, "Not used!") } ],
[ "Upgrade from previous release",
\&UPGRADE_main ],
);

ENV_say( 1, "GBS Administrator Tools");

if (VALIDATE_administrator())
{
ASK_menu( 'Select function to perform', \@admin_menu_items, $entries_ref);
}
}




sub do_user_templates($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Create User Templates");
ENV_say( 1, "You can find and edit the Templates in GBS_SYSGEN_PATH/templates");

GBSSCM_preset( 0);

my $gbs_templates_dir = "$GBS_SCRIPTS_PATH/template";
my $gbs_templates_dir_l = length( $gbs_templates_dir);
my $user_templates_dir = ENV_getenv_perl_path( 'GBS_SYSGEN_PATH') . '/templates';
my $user_templates_dir_l = length( $user_templates_dir);
my @gbs_templates = map { substr( $_, $gbs_templates_dir_l + 4) } glob( "$gbs_templates_dir/any.*");
do
{
my @user_templates = map { substr( $_, $user_templates_dir_l + 4) } glob( "$user_templates_dir/any.*");
ENV_say( 1, "Current User Templates:",
@user_templates);
my $from_type = ASK_value_from_menu( 'Select template type to copy', 0, ('', @gbs_templates));
ENV_say( 1, "From = $from_type");
if ($from_type ne '')
{
my $to_type = ASK_text( "Enter new type (must start with '.')", $from_type, 0, [ \&check_type, undef ]);
if ($to_type ne '')
{
ENV_say( 1, "  To = $to_type");
my $to_spec = "$user_templates_dir/any$to_type";
my $must_copy = 'Y';
if (-e $to_spec)
{
$must_copy = ASK_YN( "File already exists. Copy anyway?", 'Y');
}
if ($must_copy eq 'Y')
{
SCM_assert_co_file( $to_spec);
my $from_spec = "$gbs_templates_dir/any$from_type";
my $copied = ENV_copy_file( $from_spec, $to_spec, 0);    # 0 = no exit on error
if ($copied)
{
ENV_chmod( 'u+w', $to_spec);
ENV_say( 1, "File copied");

}
}
} else
{
ENV_say( 1, "- Copy cancelled");
}
} else
{
ENV_say( 1, "- Copy cancelled");
}
} while (ASK_YN( 'More?', 'N') eq 'Y');
}





sub check_type($$$)
{
my ($values_ref,
$default_ref,
$app_data_ref,
) = @_;
my $error_txt = '';	# non empty == error

my $value = $values_ref->[0];
if ($value ne '')
{
$error_txt = "Type '$value' must start with a '.' (dot)"
if (substr( $value, 0, 1) ne '.');
}

return $error_txt;
}




sub fix_system_gbs($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Fix system.gbs");
ENV_say( 1, "Check system.gbs...");

GBSSCM_preset( 0);

if (SYSTEM_fix( $GBS_ROOT_PATH))
{
ENV_say( 1, "  system.gbs: Internals Fixed");
} else
{
ENV_say( 1, "  system.gbs: No Internals Fix needed");
}




foreach my $name (qw( audit tool))
{
my $name_uc = uc $name;
my $env_name = "GBS_ALL_$name_uc" . 'S';
my @all_values = ENV_getenv( $env_name);
if (@all_values != 0)
{
my @all_dirs = GBSSFILE_get_all_structs( $GBS_ROOT_PATH, $name);
if (!LIST_is_eq_str( \@all_values, \@all_dirs))
{
ENV_say( 1, "  system.gbs: $name mismatch:");
ENV_say( 1, "'@all_dirs' <> '@all_values'");
my @pair = ($name . 's' => [ @all_dirs ]);
SYSTEM_write( $GBS_ROOT_PATH, @pair);
ENV_setenv( $env_name => @all_dirs);
ENV_say( 1, "  system.gbs: $name repaired");
}
}
}

ENV_say( 1, "Check system.gbs done");
}

return 1;
