#===============================================================
#
#   audithist.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================























package audithist;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
AUDITHIST_select_subsys
AUDITHIST_get_file
AUDITHIST_commit
);

use strict;
use integer;

use glo::env;
use glo::slurp;
use glo::scm;
use glo::time;
use gbsenv;
use gbsglo;
use files;




sub AUDITHIST_select_subsys($);
sub AUDITHIST_get_file($$$);
sub AUDITHIST_commit($$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_TARGET;
my $GBS_AUDIT;

my $CUR_SUBSYS = '';
my $CUR_HIST_DATE = 'None';
my %HISTORIES;





sub AUDITHIST_select_subsys($)
{
my ($subsys,
) = @_;


if (!defined $GBS_TARGET)
{
$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
}

if ($subsys ne $CUR_SUBSYS)
{
%HISTORIES = ();
$CUR_SUBSYS = $subsys;
my $gbs_audit_path = "$GBS_ROOT_PATH/dev/$CUR_SUBSYS/audit";
my $this_audit_path = "$gbs_audit_path/$GBS_AUDIT";
my @hist_files = grep /^hist_${GBS_TARGET}_\d+\.gbs$/, SLURP_dir_files( $this_audit_path, 0);

if (@hist_files)
{




my $cur_hist_file = $hist_files[-1];	# files are sorted
my ($hist_date) = $cur_hist_file =~ /hist_.*?_(\d{8})\.gbs/;
$CUR_HIST_DATE = TIME_unpack( $hist_date);




ENV_say( 1, "Reading hist $CUR_SUBSYS: $cur_hist_file...");
my @in_lines = SLURP_file( "$this_audit_path/$cur_hist_file");
my $comp_root = "$GBS_ROOT_PATH/dev/$CUR_SUBSYS/comp";
while (@in_lines)
{
my $line = shift @in_lines;
ENV_fatal( "ERROR in hist-file format ($cur_hist_file)")
if (substr( $line, 0, 2) ne '::');
my $component = substr( $line, 2);
my @comp_lines;
while (@in_lines && $in_lines[0] ne '::EOC')
{
push @comp_lines, shift @in_lines;
}
shift @in_lines;    # ::EOC


if (-d "$comp_root/$component")
{
while (@comp_lines)
{
my $line = shift @comp_lines;
my $file = substr( $line, 1);

my @out_lines;
while (@comp_lines && $comp_lines[0] ne ':EOF')
{
push @out_lines, shift @comp_lines;
}
shift @comp_lines;  # :EOF
$HISTORIES{$component}->{$file} = [ @out_lines ];
}
} else
{
ENV_say( 1, "Component '$component' does not exist anymore - skipped");
}
}
} else
{
$CUR_HIST_DATE = 'None';
ENV_say( 1, "No hist for SubSystem $CUR_SUBSYS");
}
}

return $CUR_HIST_DATE;
}




sub AUDITHIST_get_file($$$)
{
my ($subsys,
$component,
$file,
) = @_;




ENV_fatal( "Subsys $subsys not selected ($CUR_SUBSYS)")
if ($subsys ne $CUR_SUBSYS);

my $lines_ref = $HISTORIES{$component}->{$file};
if (defined $lines_ref)
{
return @{$lines_ref};
} else
{
return ();
}
}




sub AUDITHIST_commit($$)
{
my ($audits_ref,
$targets_ref,
) = @_;

my $date = TIME_date2num_packed();

ENV_say( 1, "Hist-id = $date");





foreach my $subsys (GBSGLO_subsystems_full_gbs())
{
my $subsys_root = "$GBS_ROOT_PATH/dev/$subsys";
ENV_say( 1, "$subsys...");
foreach my $audit (@{$audits_ref})
{
next if (! -d "$subsys_root/audit/$audit");

foreach my $target (@{$targets_ref})
{
next if (! -d "$subsys_root/gen/$target");





my $count = 0;
my @lines;
foreach my $component (GBSGLO_components( $subsys))
{
my $component_root = "$subsys_root/comp/$component";
next if (! -d "$component_root/bld/$target");
next if (! -d "$component_root/aud/$audit/$target");

my @audit_hist_files = glob( "$component_root/aud/$audit/$target/*.gmet");

if (@audit_hist_files)
{
push @lines, "::$component\n";
foreach my $audit_hist_file (@audit_hist_files)
{
my $file_name = ENV_split_spec_f( $audit_hist_file);

push @lines, ":$file_name\n";
push @lines, SLURP_file_raw( $audit_hist_file);
push @lines, ":EOF\n";
$count++;
}
push @lines, "::EOC\n";
}
}




if (@lines)
{
ENV_say( 1, "Consolidating: $subsys / $audit / $target ($count files)...");
my $hist_dir = "$subsys_root/audit/$audit";
my $hist_file_name = "hist_${target}_$date.gbs";
my $hist_file = "$hist_dir/$hist_file_name";

SCM_add_dirs( $hist_dir, \&FILES_get_ignore_spec, 1, 0)	    # $pre_validated, $verbose
if (SCM_get_states( $hist_dir) == -1); # not SCM

SCM_store_file( $hist_file, \@lines );
} else
{
ENV_say( 1, "No $audit files for $subsys / $audit / $target");
}

}
}
}
}

return 1;

