#===============================================================
#
#   files.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package files;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
FILES_makefile_spec
FILES_split_spec
FILES_attrib
FILES_mandatory_dirs
FILES_get_ignore_spec
FILES_all
FILES_find_root_path
FILES_check_system
FILES_check_system_other_platform
FILES_check_gbs_subsys
);

use strict;
use integer;

use glo::env;
use glo::slurp;
use gbsglo;
use gbsenv;




sub FILES_makefile_spec($$);
sub FILES_split_spec($);
sub FILES_mandatory_dirs($$);
sub FILES_attrib($);
sub FILES_get_ignore_spec($);
sub FILES_all($);
sub FILES_find_root_path($);
sub FILES_check_system($);
sub FILES_check_system_other_platform($);
sub FILES_check_gbs_subsys($);

sub node_init();
sub attrib($);
sub top_level_split($);
sub mandatory_dirs($$);
sub all_files($$$);
sub dummy_check_mandatory($$);
sub get_ignore($);

sub node_list($);
sub node_is_dir($);
sub node_data($);
sub node_recurse($$$);
sub node_next($$);




my %GBS_ALL_TARGETS;
my %GBS_ALL_AUDITS;
my %GBS_ALL_TOOLS;

my %PLUGIN_ITEMS;





my $ATTRIB_INIT_IS_DONE = 0;

my $THIS_CMD_EXT;
my $OTHR_CMD_EXT;
if (ENV_is_win32())
{
$THIS_CMD_EXT = '.bat';
$OTHR_CMD_EXT = '.sh';
} else
{
$THIS_CMD_EXT = '.sh';
$OTHR_CMD_EXT = '.bat';
}

my $HASH_LEVEL;
my @HASH_KEYS;






my %DIRS = (







SYSTEM =>
{
'.'		 => [ 'M', 1, 1, -1, 1, -1 ],
'system.gbs'		    => [ 'M', 1 ],
"switch.gbs$THIS_CMD_EXT"   => [ 'M', 1 ],
"switch.gbs$OTHR_CMD_EXT"   => [ 'O', 1 ],
'switch.usr.*'		    => [ 'O', 0 ],
dev =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.gbsrc' => [ 'O', 0 ],
'*'      =>			    # SubSystems
{
'.'	=> [ 'O', 1, 1, 1, 1, 1 ],
},
},
ext =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
res =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.*.lck' => [ 'O', 0 ],
'*'      =>			    # SubSystems
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
},
},
silo =>
{
'.'	    => [ 'M', 1, 1, 0, 1, 0 ],
'.gbs'  =>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
cache   =>
{
'.'		=> [ 'M', 1, 1, 0, 1, 0 ],
html  =>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
},
},
gbssum	=>
{
'.'	=> [ 'M', 1, 1, 0, 1, 0 ],
},
},
},
sys =>
{
'.'		  => [ 'M', 1, 1, 1, 1, 1 ],
'owners.gbs'  => [ 'O', 1 ],
'steps.gbs'   => [ 'O', 1 ],
'sys.gbs'     => [ 'M', 1 ],
'*.usr'	  => [ 'O', 0 ],
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
sysaudit =>
{
'.'		  => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'  => [ 'O', 1 ],		    # in audits?
'flags_*.gbs' => [ 'O', 1 ],		    # in audits?
'*.usr'	  => [ 'O', 0 ],		    # in audits?
'<A>' =>  # Audits
{
'.'		 => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'     => [ 'O', 1 ],		    # in audits?
'flags_*.gbs'	 => [ 'O', 1 ],		    # in audits?
'bld_incs_*.gbs' => [ 'O', 1 ],		    # in audits?
'bld_flags_*.gbs'=> [ 'O', 1 ],		    # in audits?
'audit.gbs'      => [ 'O', 1 ],
'*.usr'	         => [ 'O', 0 ],
},
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
sysgen =>
{
'.'		    => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'    => [ 'O', 1 ],
'flags_*.gbs'   => [ 'O', 1 ],
'*.usr'	    => [ 'O', 0 ],
templates	    =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
makemake_stubs  =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
'ALL'	=>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
'<G>' =>  # Targets
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
},
'<G>'	    =>  # Targets
{
'.'		 => [ 'M', 1, 1, 1, 1, 1 ],
'incs_*.gbs'     => [ 'O', 1 ],
'flags_*.gbs'	 => [ 'O', 1 ],
'bld_incs_*.gbs' => [ 'O', 1 ],
'bld_flags_*.gbs'=> [ 'O', 1 ],
'target.gbs'     => [ 'O', 1 ],
'*.usr'	         => [ 'O', 0 ],
},
'*'		    =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
systools =>
{
'.'	  => [ 'M', 1, 1, 1, 1, 1 ],
'<T>' => # Tools
{
'.'		=> [ 'M', 1, 1, 1, 1, 1 ],
'tool.gbs'      => [ 'M', 1 ],
'*.usr'		=> [ 'O', 0 ],
},
'*' =>
{
'.' => [ 'O', 1, 1, 1, 1, 1 ],
}
},
tmp =>
{
'.'	  => [ 'M', 1, 1, 0, 1, 0 ],
'<A>' => # Audits
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'<G>' => # Targets
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
'<T>' => # Tools
{
'.' => [ 'M', 1, 1, 0, 1, 0 ],
},
},
},








SUBSYS_GBS =>
{
'.'        => [ 'M', 1, 1, 1, 1, 1 ],
'full.gbs' => [ 'M', 1 ],
'.*.lck'   => [ 'O', 0 ],
audit =>
{
'.'	  => [ 'M', 1, 0, 1, 0, 0 ],
'<A>' => # Audits
{
'.'   => [ 'O', 1, 0, 0, 1, 1 ],
'<G>' => # Targets
{
'.' => [ 'O', 1, 0, 0, 1, 1 ],
},
},
},
comp =>
{
'.'	     => [ 'M', 1, 1, 1, 0, 0 ],
'.gbsrc' => [ 'O', 0 ],
},
export =>
{
'.'	=> [ 'O', 1, 1, 0, 1, 0 ],
},
gen =>
{
'.'	          => [ 'M', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'*.mk'        => [ 'O', 0 ],
'export.gbs'  => [ 'O', 1 ],
'*.usr'	  => [ 'O', 0 ],
'<G>'	  => #Targets
{
'.' => [ 'O', 1, 0, 0, 1, 1 ],
}
},
import =>
{
'.'	=> [ 'O', 1, 1, 1, 1, 1 ],
},
tools =>
{
'.'	=> [ 'M', 1, 1, 1, 0, 0 ],
'*' =>  # Tools
{
'.' => [ 'O', 1, 1, -1, 1, -1 ],
},
},
},








SUBSYS_NON_GBS =>
{
'.'        => [ 'M', 1, 1, -1, 1, -1 ],
"gbssubaudit$THIS_CMD_EXT"	  => [ 'M', 1 ],
"gbssubaudit$OTHR_CMD_EXT"	  => [ 'O', 1 ],
"gbssubcleanup_aud$THIS_CMD_EXT"  => [ 'M', 1 ],
"gbssubcleanup_aud$OTHR_CMD_EXT"  => [ 'O', 1 ],
"gbssubcleanup_bld$THIS_CMD_EXT"  => [ 'M', 1 ],
"gbssubcleanup_bld$OTHR_CMD_EXT"  => [ 'O', 1 ],
"gbssubgen$THIS_CMD_EXT"	  => [ 'M', 1 ],
"gbssubgen$OTHR_CMD_EXT"	  => [ 'O', 1 ],
"gbssubmake$THIS_CMD_EXT"	  => [ 'M', 1 ],
"gbssubmake$OTHR_CMD_EXT"	  => [ 'O', 1 ],
'export.gbs'			  => [ 'M', 1 ],
'.*.lck'			  => [ 'O', 0 ],
app =>
{
'.'	=> [ 'M', 1, 1, -1, 1, -1 ],
},
export =>
{
'.'	=> [ 'O', 1, 1, 0, 1, 0 ],
},
},








COMPONENT =>
{
'.'          => [ 'M',  1, 0, 0, 0, 0 ],
'export.gbs' => [ 'O',  1, ],
'export.usr' => [ 'O',  0, ],
'scope.gbs'  => [ 'O',  1, ],
'scope.usr'  => [ 'O',  0, ],
'*.dsp'      => [ 'O', -1, ],	# Microsoft Visual Studio 6
'*.dsw'      => [ 'O', -1, ],	# Microsoft Visual Studio 6
'*.sln'      => [ 'O', -1, ],	# Microsoft Visual Studio 8
'*.mcp'      => [ 'O', -1, ],	# Code Warrior
aud =>
{
'.'	=> [ 'M', 1, 0, 0, 0, 0 ],
'<A>' => #audits
{
'.'   => [ 'O', 1, 1, -1, 1, -1 ],
'<G>' => #Targets
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
},
'ALL' => #All-Targets
{
'.' => [ 'O', 1, 1, 0, 1, 0 ],
}
}
},
bld =>
{
'.'	    => [ 'M', 1, 0, 0, 0, 0 ],
'*.mk'  => [ 'O', 0 ],
'*.ref' => [ 'O', 0 ],
'<G>' => #Targets
{
'.' => [ 'O', 1, 0, 0, 1, 0 ],
}
},
dat =>
{
'.'	=> [ 'M', 1, 1, 1, 1, 1 ],
},
inc =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
loc =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
opt =>
{
'.'	=> [ 'M', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'*.usr'	  => [ 'O', 0 ],
'<G>' =>  # Targets
{
'.'	=> [ 'O', 1, 0, 0, 0, 0 ],
'incs_*.gbs'  => [ 'O', 1 ],
'flags_*.gbs' => [ 'O', 1 ],
'*.usr'	      => [ 'O', 0 ],
}
},
sav =>
{
'.'	=> [ 'M', 1, 1, -1, 1, -1 ],
},
src =>
{
'.'	=> [ 'M', 1, 0, 0, 1, 1 ],
},
},
);




sub node_init()
{
my @gbs_all_targets = ENV_getenv( 'GBS_ALL_TARGETS');
my @gbs_all_audits = ENV_getenv( 'GBS_ALL_AUDITS');
my @gbs_all_tools = ENV_getenv( 'GBS_ALL_TOOLS');

%PLUGIN_ITEMS = (

'<A>'    => [ @gbs_all_audits ],    # Audit
'<G>'    => [ @gbs_all_targets ],   # Gen
'<T>'    => [ @gbs_all_tools ],	    # Tool
);

%GBS_ALL_TARGETS = map { $_ => 1 } @gbs_all_targets;
%GBS_ALL_AUDITS = map { $_ => 1 } @gbs_all_audits;
%GBS_ALL_TOOLS = map { $_ => 1 } @gbs_all_tools;
}




sub FILES_makefile_spec($$)
{
my ($gbsgenroot,
$gbstarget,
) = @_;

return "$gbsgenroot/$gbstarget.mk";
}




sub FILES_find_root_path($)
{
my ($file_spec) = @_;	# Abs dir or file spec
my $root_path = '';

my @dirs = split( '/', $file_spec);
pop @dirs if (-f $file_spec);
while (@dirs)
{
my $possible_root = join( '/', @dirs);
if (-f "$possible_root/system.gbs" && FILES_check_system( $possible_root))
{
$root_path = $possible_root;
last;
}
pop @dirs;
}

return $root_path;
}




sub FILES_split_spec($)
{
my ($file_spec) = @_;
my ($root_path, $subsys, $component, $remain) = ( '', '', '', '' );

my $gbs_root = GBSENV_perl_gbs_root();
if ($gbs_root ne '' && ENV_is_in_path( $file_spec, $gbs_root))
{
$root_path = $gbs_root;
} else
{
$root_path = FILES_find_root_path( $file_spec);
}

if ($root_path ne '')
{
if ($file_spec =~ m!$root_path/dev/([^/]+)!)
{
$subsys = $1;
if ($file_spec =~ m!$root_path/dev/$subsys/comp/([^/]+)!)
{
$component = $1;
$remain = substr( $file_spec, length( "$root_path/dev/$subsys/comp/$component"));
} else
{
$remain = substr( $file_spec, length( "$root_path/dev/$subsys"));
}
} else
{
$remain = substr( $file_spec, length( $root_path));
}
$remain = substr( $remain, 1) if ($remain =~ m!^/!);
}

return ($root_path, $subsys, $component, $remain);
}




sub FILES_attrib($)
{
my ($spec,		# file or dir
) = @_;


if (!$ATTRIB_INIT_IS_DONE)
{
node_init();
$ATTRIB_INIT_IS_DONE = 1;
}

return attrib( top_level_split( $spec));
}




sub top_level_split($)
{
my ($spec) = @_;
my ($top_level, $path);

( my $root_path, my $subsys, my $component, $path) = FILES_split_spec( $spec);

if ($component)
{
$top_level = 'COMPONENT';
} elsif ($subsys)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
$top_level = 'SUBSYS_GBS';
} else
{
$top_level = 'SUBSYS_NON_GBS';
}
} else
{
$top_level = 'SYSTEM';
}

return [ $top_level, $path ];
}




sub attrib($)
{
my ($tp_ref) = @_;	    # [ $top_level, $path ]
my ($ignores_ref, $scm, $avail, $type);

my ($top_level, $path) = @{$tp_ref};
my $node_ref = [ \%DIRS, $top_level, $DIRS{$top_level} ];
foreach my $name (split( '/', $path))
{
$node_ref = node_next( $node_ref, $name);
last if (!defined $node_ref || $node_ref->[1] eq '.');
}


if (defined $node_ref)
{
my $name = $node_ref->[1];
if ($name eq '.')
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
$type = 'U';
$avail = 'O';
if ($scm_extra_dirs == 1 && $scm_extra_files == 1)
{
$scm = 1;
} elsif ($scm_extra_dirs == 0 && $scm_extra_files == 0)
{
$scm = 0;
} else
{
$scm = -1;
}
} else
{
($type, $avail, $scm) = node_data( $node_ref);
$ignores_ref = get_ignore( $node_ref);
}
}

return (wantarray) ? ($ignores_ref, $scm, $avail, $type) : $ignores_ref;
}




sub FILES_get_ignore_spec($)
{
my ($dir) = @_;
my $ignores_ref;		   #	[ $ignores ]	(wildcards)

$ignores_ref = attrib( top_level_split( $dir));

return $ignores_ref;
}




sub FILES_mandatory_dirs($$)
{
my ($level_root,
$top_level) = @_;   # SYSTEM, SUBSYS_GBS, SUBSYS_NON_GBS or COMPONENT
my @dirs;

$HASH_LEVEL = 0;
@HASH_KEYS = ($level_root);
my $top_ref = [ \%DIRS, $top_level , $DIRS{$top_level} ];
node_init();
node_recurse( $top_ref, \&mandatory_dirs, \@dirs);

return @dirs;
}




sub mandatory_dirs($$)
{
my ($dirs_ref,  # $data
$node_ref,  # [ $hash_ref, $name, $value_ref ]
) = @_;

if (node_is_dir( $node_ref))
{
my $dir_name = $node_ref->[1];
my (undef, $avail) = node_data( $node_ref);


if ($avail eq 'M')
{
if ($dir_name =~ /<.>/)
{
my @plugin_items = @{$PLUGIN_ITEMS{$dir_name}};
push @{$dirs_ref}, map { join( '/', @HASH_KEYS, $_) } @plugin_items;
} else
{
push @{$dirs_ref}, join( '/', @HASH_KEYS, $dir_name);
}
}
}
}











sub FILES_all($)
{
my ($root_path) = @_;
my @line_refs;

node_init();
my $node_ref = [ \%DIRS, undef, undef ];

$node_ref->[1] = 'SYSTEM';
$node_ref->[2] = $DIRS{'SYSTEM'};
@line_refs = all_files( $node_ref, $root_path, '');

foreach my $subsys (GBSGLO_subsystems())
{
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
$node_ref->[1] = 'SUBSYS_GBS';
$node_ref->[2] = $DIRS{'SUBSYS_GBS'};
push @line_refs, all_files( $node_ref, $root_path, "dev/$subsys");

foreach my $component (GBSGLO_components( $subsys))
{
$node_ref->[1] = 'COMPONENT';
$node_ref->[2] = $DIRS{'COMPONENT'};
push @line_refs, all_files( $node_ref, $root_path, "dev/$subsys/comp/$component");
}
} else
{
$node_ref->[1] = 'SUBSYS_NON_GBS';
$node_ref->[2] = $DIRS{'SUBSYS_NON_GBS'};
push @line_refs, all_files( $node_ref, $root_path, "dev/$subsys");
}
}

return @line_refs;
}












sub all_files($$$)
{
my ($node_ref,    # [ $hash_ref, $name, $value_ref ]
$root_path,
$mid,
) = @_;
my @line_refs;

my $this_root_path = ($mid eq '') ? $root_path : "$root_path/$mid";





my %this_dir;
my @this_dir_keys;
if (-d $this_root_path)
{
%this_dir = map { $_ => 1 } GBSGLO_select_direntries( SLURP_dir_all( $this_root_path, 0));
$this_dir{'.gbsrc'} = 1
if (-e "$this_root_path/.gbsrc");
$this_dir{'.gbs'} = 1
if (-e "$this_root_path/.gbs");
@this_dir_keys = keys %this_dir;

}






my @next_refs;

foreach my $member_ref (node_list( $node_ref))
{
my $name = $member_ref->[1];

my $member_is_dir = node_is_dir( $member_ref);

if ($name eq '*')
{
my $spec = ($mid eq '') ? '*' : "$mid/*";
push @next_refs, [ $spec, $member_ref ];
} else
{
my ($type, $avail, $scm) = node_data( $member_ref);

my @found;
my @missing;
if ($name =~ /<.>/)
{
foreach my $target_or_audit (@{$PLUGIN_ITEMS{$name}})
{
if (exists $this_dir{$target_or_audit})
{
push @found, $target_or_audit;
} else
{
push @missing, $target_or_audit;
}
}
} else
{
@found = ENV_wildcard( $name, \@this_dir_keys);
@missing = ($name) if (!@found);
}
if (@found)
{

foreach my $this_entry (@found)
{
my $spec = ($mid eq '') ? $this_entry : "$mid/$this_entry";

if ( $type eq 'D')
{
if (-d "$root_path/$spec")
{
push @line_refs, [ $type, $avail, $scm, $spec ];
push @next_refs, [ $spec, $member_ref ]
if ($member_is_dir);
} else
{
push @line_refs, [ $type, 'i', $scm, $spec ];
}
delete $this_dir{$this_entry};
} else
{
if (-d "$root_path/$spec")
{
push @line_refs, [ $type, 'i', $scm, $spec ];
} else
{
push @line_refs, [ $type, $avail, $scm, $spec ];
delete $this_dir{$this_entry};
}
}
}
}
if (@missing)	# actual file/dir not found
{
foreach my $name (@missing)
{
my $spec = ($mid eq '') ? $name : "$mid/$name";
if ($avail eq 'M')
{
push @line_refs, [ $type, 'm', $scm, $spec ];
}
push @next_refs, [ $spec, $member_ref ]
if ($member_is_dir);
}
}
}
}





if (%this_dir)
{

my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
foreach my $entry (keys %this_dir)
{
my ($type, $avail, $scm, $spec) = ( 0, 'I', 0, ($mid eq '') ? $entry : "$mid/$entry" );
if (-d "$root_path/$spec")
{
$type = 'D';
if ($extra_dirs)
{
$avail = 'O';
$scm = $scm_extra_dirs;


}
} else
{
$type = 'F';
if ($extra_files)
{
$avail = 'O';
$scm = $scm_extra_files;
}
}
push @line_refs, [ $type, $avail, $scm, $spec ];

}
}





push @line_refs, map { all_files( $_->[1], $root_path, $_->[0]) } @next_refs;

return @line_refs;
}




sub FILES_check_system($)
{
my ($root_path) = @_;


return ( -d "$root_path/dev" &&
-d "$root_path/ext" &&
-d "$root_path/res" &&
-f "$root_path/system.gbs" &&
-f "$root_path/switch.gbs$THIS_CMD_EXT");
}




sub FILES_check_system_other_platform($)
{
my ($root_path) = @_;


return ( -d "$root_path/dev" &&
-d "$root_path/ext" &&
-d "$root_path/res" &&
-f "$root_path/system.gbs" &&
-f "$root_path/switch.gbs$OTHR_CMD_EXT");
}





sub FILES_check_gbs_subsys($)
{
my ($subsys_root) = @_;

return ( -d "$subsys_root/comp" &&
-d "$subsys_root/gen" &&
-d "$subsys_root/audit");
}





sub dummy_check_mandatory($$)
{
my ($base_root,
$parent,
) = @_;
my $ok = 1;


my $top_ref = [ \%DIRS, $parent, $DIRS{$parent} ];
foreach my $member_ref (node_list( $top_ref ))
{
my (undef, $avail) = node_data( $member_ref);
if ($avail eq 'M')
{

if (!-e "$base_root/$member_ref->[1]")
{
$ok = 0;
last;
}
}
}

return $ok;
}




sub get_ignore($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $value_ref ]
my $ignores_ref;

if (node_is_dir( $node_ref))
{
my @ignores;
foreach my $member_ref (node_list( $node_ref))
{
my (undef, $avail, $scm) = node_data( $member_ref);

push @ignores, $member_ref->[1]
if ($scm == 0);
}
if (!@ignores)
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
@ignores = ('*')
if ($scm_extra_dirs == 0 && $scm_extra_files == 0);
}
$ignores_ref = [ @ignores ]
if (@ignores);
}
return $ignores_ref;
}




sub node_list($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $value_ref ]
my (@node_refs);

my $members_ref = $node_ref->[2];
foreach my $key (sort( keys(%{$members_ref})))
{
next if ($key eq '.');
push @node_refs, [ $members_ref, $key, $members_ref->{$key} ];
}

return @node_refs;
}




sub node_is_dir($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $value_ref ]


return (ref $node_ref->[2] eq 'HASH') ? 1 : 0;
}




sub node_data($)
{
my ($node_ref) = @_;    # [ $hash_ref, $name, $value_ref ]

my ($type);

my $attrs_ref;
if (node_is_dir( $node_ref))
{
$type = 'D';
$attrs_ref = $node_ref->[2]->{'.'};
} else
{
$type = 'F';
$attrs_ref = $node_ref->[2];
}

return ($type, @{$attrs_ref});
}




sub node_recurse($$$)
{
my ($node_ref,  # [ $hash_ref, $name, $value_ref ]
$func_ref,
$data
) = @_;

$HASH_LEVEL++;
foreach my $member_ref (node_list( $node_ref))
{
my ($hash_ref, $name, $value_ref) = @{$member_ref};

&$func_ref( $data, $member_ref);
if (node_is_dir( $member_ref))
{
push @HASH_KEYS, $name;
node_recurse( $member_ref, $func_ref, $data);
pop @HASH_KEYS;
}
}
$HASH_LEVEL--;
}




sub node_next($$)
{
my ($node_ref,      # [ $hash_ref, $name, $value_ref ]
$next_name,
) = @_;
my $next_node_ref;	# undef == not_found


my $value_ref = $node_ref->[2];
if (exists $value_ref->{$next_name})
{
$next_node_ref = [ $value_ref, $next_name ];
} else
{




if (exists $value_ref->{'*'})
{

$next_node_ref = [ $value_ref, '*' ];
} elsif (exists $value_ref->{'<G>'} && exists( $GBS_ALL_TARGETS{$next_name}))
{

$next_node_ref = [ $value_ref, '<G>' ];
} elsif (exists( $value_ref->{'<A>'}) && exists( $GBS_ALL_AUDITS{$next_name}))
{

$next_node_ref = [ $value_ref, '<A>' ];
} elsif (exists $value_ref->{'<T>'} && exists( $GBS_ALL_TOOLS{$next_name}))
{

$next_node_ref = [ $value_ref, '<T>' ];
} else
{

foreach my $member_ref (node_list( $node_ref))
{
my $member_name = $member_ref->[1];
next if (! ENV_is_wildcard( $member_name));
my $member_name_re = ENV_wild_2_re( $member_name);
if ($next_name =~ /$member_name_re/)
{
$next_node_ref = [ $value_ref, $member_name ];

last;
}
}
if (!defined $next_node_ref)
{
my (undef, undef, undef, $extra_dirs, $scm_extra_dirs, $extra_files, $scm_extra_files) =
node_data( $node_ref);
if ($extra_dirs == 1 && $extra_files == 1)
{
$next_node_ref = [ $value_ref, '.' ];
}
}
}
}



$next_node_ref->[2] = $next_node_ref->[0]->{$next_node_ref->[1]}
if (defined $next_node_ref);
return $next_node_ref;
}
return 1;
