#===============================================
#
#   flags.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package flags;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
FLAGS_get_flags
);

use strict;
use integer;

use glo::env;
use gbsfile;




sub FLAGS_get_flags($);

sub read_flags($);




my %FLAGS_CACHE;






sub FLAGS_get_flags($)
{
my ($file_spec) = @_;

return read_flags( $file_spec);
}




sub read_flags($)
{
my ($file_spec) = @_;
my @lines;

if (exists( $FLAGS_CACHE{$file_spec}))
{

@lines = @{$FLAGS_CACHE{$file_spec}};
} else
{
GBSFILE_open( $file_spec, 1, undef, undef);
my $line = GBSFILE_get_line();
while ($line)
{
$line = ENV_expand_envs( $line);
if (ENV_is_option( $line))
{
my $opt = substr( $line, 0, 2);
GBSFILE_error_exit( "'$opt' flags not allowed in GBS FLAG Files")
if (substr( $opt , 1, 1) eq 'I' || substr( $opt , 1, 1) eq 'L')
}
push @lines, $line;
$line = GBSFILE_get_line();
}
$FLAGS_CACHE{$file_spec} = [ @lines ];
}

return @lines;
}

return 1;

