#===============================================================
#
#   gbsbgsca.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsbgsca;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSBGSCA_init
GBSBGSCA_do
);

sub BEGIN
{
$ENV{GBS_SCA_PLUGIN_NAME} = ''
if (!defined $ENV{GBS_SCA_PLUGIN_NAME});
}
use strict;
use integer;

use glo::env;
use glo::slurp;
use glo::depend;
use gbsenv;
use gbsglo;
use flincs;
use scadef;
use scaglo;
use scafile;
use sca;
use gbsbgglo;
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/pclint/bin";
use lib "$ENV{GBS_SCRIPTS_PATH}/plugins/audit/qac/bin";

use if $ENV{GBS_SCA_PLUGIN_NAME} eq 'pclint',	'pclint';
use if $ENV{GBS_SCA_PLUGIN_NAME} eq 'qac',	'prqa';
use if $ENV{GBS_SCA_PLUGIN_NAME} eq 'qacpp',	'prqa';




sub GBSBGSCA_init($$);
sub GBSBGSCA_do($$$@);

sub do_file_pre($$);
sub do_file_run($);
sub do_file_post($);

sub find_hash_include_line($$);
sub parse_source($);
sub path_is_not_quieted($);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');

my $PLUGIN_NAME = $ENV{GBS_SCA_PLUGIN_NAME};	    # pclint, qac, qacpp
my $PLUGIN_NAME_UC = uc $PLUGIN_NAME;

my ($MUST_SHOW_STDOUT) = ENV_getenv( 'GBS_AUDIT_OPTIONS');	# from gbsaudit-option: --stdout (show_stdout)




my $TARGET_PLUGIN_NAME;

my $PLUGIN_NICE_NAME;

my $GBSEXT_PLUGIN_PATH;
my $GBSEXT_PLUGIN_REL;

my $STDOUT_FORMAT_NAME;	    # from sca_<audit>.gbs (PCLINT, PRQA, VISUAL_C, ECLIPSE, GNU_C, etc)

my $P_PACKAGE;		    # (not used)
my $P_INIT_FUNC;
my $P_PRE_FUNC;
my $P_RUN_FUNC;
my $P_POST_FUNC;

my $UNKN_MSG_FUNC;	    # returned by $P_INIT_FUNC





my %SPECWS;






my %LEVELS_BY_LEVEL;


my %MSG_REFS;


my %SPECIAL_MSG_NRS;	    # Fatal and SPECWS


my @QUIET_PATHS;
my %QUIET_FILE_SPECS;





my $SRC_PATH;
my $GBS_COMPONENT;
my $SOURCE_FILE;
my $SOURCE_FILE_SPEC;
my $SRC_TYPE;

my @PRINTABLE_OUT_FILES;

my %INCLUDE_SRC_LINE;





my %PLUGIN_REFS = (

pclint  => [ 'pclint',    \&PCLINT_init,    \&PCLINT_pre,    undef,		\&PCLINT_post ],

qac	    => [ 'prqa',      \&PRQA_init,      \&PRQA_pre,      undef,		\&PRQA_post ],

qacpp   => [ 'prqa',      \&PRQA_init,      \&PRQA_pre,      undef,		\&PRQA_post ],
);




sub GBSBGSCA_init($$)
{
($TARGET_PLUGIN_NAME,
$GBS_COMPONENT,
) = @_;

ENV_say( 1, "$PLUGIN_NAME_UC: $GBS_AUDIT ($TARGET_PLUGIN_NAME) Initialising...");




%SPECWS = map { @{$_} } @{SCADEF_get_section_items( $GBS_AUDIT, MESSAGES => 'SPECIAL')};
%LEVELS_BY_LEVEL = SCADEF_get_levels_by_level( $GBS_AUDIT);

$PLUGIN_NICE_NAME = SCADEF_get_section_items( $GBS_AUDIT, SETUP => 'NICE_NAME');

$GBSEXT_PLUGIN_PATH = ENV_getenv( "GBSEXT_${PLUGIN_NAME_UC}_PATH");
ENV_error_exit( "EnvVar 'GBSEXT_${PLUGIN_NAME_UC}_PATH' not set")
if ($GBSEXT_PLUGIN_PATH eq '');
ENV_error_exit( "Directory GBSEXT_PLUGIN_PATH (GBSEXT_${PLUGIN_NAME_UC}_PATH) does not exist")
if (! -e $GBSEXT_PLUGIN_PATH);

$GBSEXT_PLUGIN_REL = ENV_getenv( "GBSEXT_${PLUGIN_NAME}_REL");
ENV_error_exit( "EnvVar 'GBSEXT_${PLUGIN_NAME}_REL' not set")
if ($GBSEXT_PLUGIN_REL eq '');




($P_PACKAGE, $P_INIT_FUNC, $P_PRE_FUNC, $P_RUN_FUNC, $P_POST_FUNC) = @{$PLUGIN_REFS{$PLUGIN_NAME}};




SCAGLO_init( $PLUGIN_NAME, $GBSEXT_PLUGIN_REL, $PLUGIN_NICE_NAME);




%MSG_REFS = map { $_->[0] => $_ } SCAGLO_read_msg_file();

{
my @all_msg_nrs;
my @fatal_msg_nrs;
foreach my $ref (values %MSG_REFS)
{
my ($msg_nr, $level) = @{$ref};
push @all_msg_nrs, $msg_nr;
push @fatal_msg_nrs, $msg_nr
if ($LEVELS_BY_LEVEL{$level}->[3] == 2);	    #$severity_index == Fatal Error
}
SCA_init( $GBS_AUDIT, $GBS_TARGET, \@all_msg_nrs, \@fatal_msg_nrs, [ keys %SPECWS ]);
%SPECIAL_MSG_NRS = map { $_ => 1 } (@fatal_msg_nrs,  keys %SPECWS);
}





$STDOUT_FORMAT_NAME = SCA_get_format_name();
$STDOUT_FORMAT_NAME = $PLUGIN_NAME_UC
if ($STDOUT_FORMAT_NAME eq 'DEFAULT');

my @gbs_defaults;



{






@QUIET_PATHS = (
"$GBS_ROOT_PATH/res",
"$GBS_ROOT_PATH/ext",
SCA_get_quiet_paths(),
);





{
my $quiet_format = SCADEF_get_section_items( $GBS_AUDIT, OPTION_FORMATS => 'QUIET_FORMAT');
if (defined $quiet_format)
{
foreach my $quiet_path (@QUIET_PATHS)
{
push @gbs_defaults, sprintf( $quiet_format, ENV_os_path( $quiet_path));
}
}
}
}




($UNKN_MSG_FUNC) =
$P_INIT_FUNC->( $PLUGIN_NAME, $GBSEXT_PLUGIN_PATH, $TARGET_PLUGIN_NAME, \@gbs_defaults);	# e.g.: PCLINT_init, PRQA_init
}





sub GBSBGSCA_do($$$@)
{
my ($file,
$unix_style_search,	# bool
$out_files,		# List
@command_data_lines
) = @_;
my $rc;

ENV_say( 1, "$PLUGIN_NAME_UC: SCA $file");






my $file_path = ENV_cwd();
do_file_pre( $file_path, $file);




$rc = do_file_run( \@command_data_lines );




$rc = do_file_post( $unix_style_search)
if ($rc == 0 || $rc == 1);
if ($rc >= 2)
{
GBSBGGLO_handle_rc( $rc, $file, 'SCA-Post', $out_files);
ENV_say( 1, "$PLUGIN_NAME_UC: Could not proceed because of severe internal errors (rc=$rc)");
}


return $rc;
}




sub do_file_pre($$)
{
($SRC_PATH,			# CWD
$SOURCE_FILE,
) = @_;

$SOURCE_FILE_SPEC = "$SRC_PATH/$SOURCE_FILE";
$SRC_TYPE = ENV_split_spec_t( $SOURCE_FILE);
%INCLUDE_SRC_LINE = ();




ENV_say( 1, "$PLUGIN_NAME_UC: $PLUGIN_NICE_NAME($GBSEXT_PLUGIN_REL) Analysing $SOURCE_FILE...");




my $gout_file_spec = SCAFILE_init( $GBS_SUBSYS, $GBS_COMPONENT, $SOURCE_FILE);




SCAFILE_delete_files();	    # .garg .gerr .gmet .gout .html .p




@PRINTABLE_OUT_FILES = $P_PRE_FUNC->( $SRC_PATH, $GBS_COMPONENT, $SOURCE_FILE, $gout_file_spec);	# E.g.: PCLINT_pre, PRQA_pre
}




sub do_file_run($)
{
my ($command_data_lines_ref) = @_;
my $rc;




if (defined $P_RUN_FUNC)
{
$rc = $P_RUN_FUNC->();   # E.g.: PCLINT_run, PRQA_run
} else
{
$rc = GBSBGGLO_exec( $SOURCE_FILE, 1, '', @{$command_data_lines_ref});
}
if ($rc != 0)
{
print "***rc=$rc\n";
foreach my $file_spec (@PRINTABLE_OUT_FILES)
{
if (-e $file_spec)
{
print "***OutFile: $file_spec ***\n";
map { print "$_\n" } SLURP_file( $file_spec);
print "***End OutFile***\n";
} else
{
print "***OutFile: $file_spec Not Found***\n";
}
}
}

return $rc;
}




sub do_file_post($)
{
my ($unix_style_search,	# bool
) = @_;
my $rc = 0;	    # 1 = contains FATAL warnings, 2 = Execution error

my $worst_severity_index = -1;	#   'IWF' (-1 = none, 0 = Info, 1 = Warnings, 2 = (Fatal) Error




my $warning_refs_ref;





my $total_refs_ref;


my $metric_refs_ref;

($warning_refs_ref, $total_refs_ref, $metric_refs_ref) = $P_POST_FUNC->();	    # E.g.: PCLINT_post, PRQA_post

if (defined $warning_refs_ref)
{



{



my $msg_file_changed = 0;
my @warning_refs;
my @selected_warning_refs;
my @l_counts;			    # used for reporting only
my %m_counts;

foreach my $ref (@{$warning_refs_ref})
{

my ($src_line_nr, $col_nr, $os_file_spec, $line_nr, $msg_level, $level_text, $msg_nr, $msg_text,
$help_file_url, @extra_src_lines ) = @{$ref};

my $msg_ref = $MSG_REFS{$msg_nr};


if (defined $msg_ref)
{
$msg_level = $msg_ref->[1]		# $level
if ($msg_level eq '');
$level_text = $msg_ref->[2]		# $level_text
if ($level_text eq '');
$help_file_url = $msg_ref->[4];	# $help_file_url
} else
{



($msg_level, $level_text, $help_file_url) = $UNKN_MSG_FUNC->( $msg_nr, $msg_level, $level_text);
$MSG_REFS{$msg_nr} = [ $msg_nr, $msg_level, $level_text, "*$msg_text", $help_file_url ];
$msg_file_changed = 1;
ENV_say( 1, "$PLUGIN_NAME_UC: *****");
ENV_say( 1, "$PLUGIN_NAME_UC: Message nr: $msg_nr ('*$msg_text') added");
ENV_say( 1, "$PLUGIN_NAME_UC: *****");
}


if (path_is_not_quieted( $os_file_spec))
{
my $severity_index = $LEVELS_BY_LEVEL{$msg_level}->[3];   # 'IWF' (-1 = none, 0 = Info, 1 = Warnings, 2 = (Fatal) Error

$worst_severity_index = $severity_index
if ($severity_index > $worst_severity_index);

$src_line_nr = find_hash_include_line( $os_file_spec, $unix_style_search)
if ($src_line_nr == -1);

my @warning_items = ( $src_line_nr, $col_nr, $os_file_spec, $line_nr, $msg_level, $level_text, $msg_nr, $msg_text,
$help_file_url, @extra_src_lines );
push @warning_refs, \@warning_items;
if (SCA_msg_is_enabled( $msg_nr) || exists $SPECIAL_MSG_NRS{$msg_nr})
{
push @selected_warning_refs, \@warning_items;
$l_counts[$msg_level]++;
}




$m_counts{$msg_nr} = [ $msg_level, $msg_nr, 0, 0 ]
if (!exists $m_counts{$msg_nr});
if ($os_file_spec eq '-')	    # in source
{
$m_counts{$msg_nr}->[2]++;	# $s_msg_count
}
$m_counts{$msg_nr}->[3]++;	    # $t_msg_count




my $specw_text = $SPECWS{$msg_nr};
if (defined $specw_text)
{
my ($arg) = $msg_text =~ /'(.*)'/;	    # e.g.: include file name
$arg = ''
if (!defined $arg);
SCAFILE_gmet_put_specw( $specw_text, $arg);
}
} else
{
ENV_say( 1, "$PLUGIN_NAME_UC: Msg $msg_nr ignored because of Quiet file:",
"$PLUGIN_NAME_UC: $os_file_spec");
}
}




SCAGLO_write_msg_file( [ values %MSG_REFS ])
if ($msg_file_changed);




SCAFILE_gerr_write( \@warning_refs);




foreach my $msg_nr (sort( { $a <=> $b} keys( %m_counts)))
{
my ($level, $msg_nr, $s_msg_count, $t_msg_count) = @{$m_counts{$msg_nr}};
SCAFILE_gmet_put_count( $level, $msg_nr, $s_msg_count, $t_msg_count);
}




if ($MUST_SHOW_STDOUT || $worst_severity_index == 2)
{
if ($STDOUT_FORMAT_NAME eq 'NONE')
{
if ($worst_severity_index == 2)	    # 2 == Fatal
{
my @fatal_warning_refs = grep $LEVELS_BY_LEVEL{$_->[4]}->[1] eq 'F', @selected_warning_refs;	    # $msg_level
SCAFILE_write_stdout( $SOURCE_FILE_SPEC, $PLUGIN_NAME_UC, \@fatal_warning_refs);
} else
{

}
} else
{
SCAFILE_write_stdout( $SOURCE_FILE_SPEC, $STDOUT_FORMAT_NAME, \@selected_warning_refs);
}
}




ENV_say( 1, "$PLUGIN_NAME_UC: $SOURCE_FILE");

my $nr_warnings = @warning_refs;
my $nr_selected_warnings = @selected_warning_refs;
ENV_say( 1, "$PLUGIN_NAME_UC: $nr_selected_warnings ($nr_warnings) messages(s) found");

if ($nr_warnings > 0)
{
my @count_texts;
for (my $i = 0; $i < @l_counts; $i++)
{
push @count_texts, "$LEVELS_BY_LEVEL{$i}->[2]=$l_counts[$i]"
if (defined $l_counts[$i]);
}
ENV_say( 1, "$PLUGIN_NAME_UC: Nr Msgs: " . join( ', ', @count_texts) . "");

ENV_say( 1, "$PLUGIN_NAME_UC: Worst Severity: " . uc( SCADEF_get_severity_text( $worst_severity_index)) . "");
}
}
$rc = 1
if ($worst_severity_index >= 2);	    # FATALs




{



foreach my $ref (@{$total_refs_ref})
{
my ($id, $source_count, $total_count) = @{$ref};
SCAFILE_gmet_put_total( $id => $source_count, $total_count);
}




foreach my $ref (@{$metric_refs_ref})
{
my ($metric_name, $min, $max, $total) = @{$ref};
SCAFILE_gmet_put_stat( $metric_name => $min, $max, $total);
}




SCAFILE_gmet_write( $worst_severity_index);
}
} else
{




$rc = 2;
}

return $rc;
}




sub find_hash_include_line($$)
{
my ($os_inc_file_spec,
$unix_style_search,	# bool
) = @_;
my $src_line_nr;

if (!%INCLUDE_SRC_LINE)
{
parse_source( $unix_style_search);
}
my @file_elements = split( '/', ENV_perl_paths( $os_inc_file_spec));
my $file = pop @file_elements;
do
{
$src_line_nr = $INCLUDE_SRC_LINE{$file};
$file = pop( @file_elements) . "/$file";
} while ( !defined $src_line_nr && @file_elements);

if (!defined $src_line_nr)
{
$src_line_nr = 0;
ENV_say( 1, "$PLUGIN_NAME_UC: =======NOT FOUND: $file in $os_inc_file_spec");
}

return $src_line_nr;
}




sub parse_source($)
{
my ($unix_style_search,	# bool
) = @_;



my @inc_paths = ENV_abs_path( $SRC_PATH, [ FLINCS_get_incs( $SRC_TYPE, $GBS_COMPONENT),   # inc
FLINCS_get_bld_incs( $SRC_TYPE)]);	      # sysinc
my @stubs_paths = FLINCS_get_stubs_incs( $GBS_TARGET);

my $cur_src_line_nr = 0;
my ($nr_errors, $nr_warnings, @depend_refs) = DEPEND_c( $SOURCE_FILE_SPEC, $unix_style_search,
\@inc_paths, \@stubs_paths, undef);
ENV_error( "$nr_errors error(s) found while parsing $SOURCE_FILE_SPEC")
if ($nr_errors > 0);
ENV_warn( "$nr_warnings warning(s) found while parsing $SOURCE_FILE_SPEC")
if ($nr_warnings > 0);

foreach my $ref (@depend_refs)
{
my ($level, $found, $line_nr, $inc_type, $inc_name, $inc_file_spec) = @{$ref};

if ($found eq 'F')
{
$cur_src_line_nr = $line_nr
if ($level == 0);
$INCLUDE_SRC_LINE{$inc_name} = $cur_src_line_nr;
my $short_inc_file_spec = GBSGLO_short_filespecs( $inc_file_spec);

} elsif ($found eq 'S')
{
my $short_inc_file_spec = GBSGLO_short_filespecs( $inc_file_spec);

} else  # ($found eq 'N')
{

my $calling_file_spec = $inc_file_spec;
my $short_calling_file_spec = GBSGLO_short_filespecs( $calling_file_spec);
ENV_say( 1, "$PLUGIN_NAME_UC: ** $inc_name not found - ignored - $short_calling_file_spec($line_nr)");
}
}
}




sub path_is_not_quieted($)
{
my ($os_file_spec) = @_;
my $is_ok = 1;

if ($QUIET_FILE_SPECS{$os_file_spec})
{

$is_ok = 0;
} else
{
my $file_spec = $os_file_spec;
$file_spec =~ tr!\\!/!;	    # perl path

foreach my $ignore_path (@QUIET_PATHS)
{
if (ENV_is_in_path( $file_spec, $ignore_path))
{
$is_ok = 0;
$QUIET_FILE_SPECS{$os_file_spec} = 1;

last;
}
}
}

return $is_ok;
}

return 1;

