#==============================================================
#
#   gbsfile.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsfile;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSFILE_open
GBSFILE_get_line
GBSFILE_info
GBSFILE_warning
GBSFILE_error
GBSFILE_error_exit
);

use strict;
use integer;

use glo::env;
use gbsfileglo;




sub GBSFILE_open($$$$);
sub GBSFILE_get_line();
sub GBSFILE_info(@);
sub GBSFILE_warning(@);
sub GBSFILE_error(@);
sub GBSFILE_error_exit(@);















sub GBSFILE_open($$$$)
{
my ($file_spec,	    # if specific .type: not generic and existence was checked
$must_exist,
$inc_path_ref,	    # in application
$target,	    # OPT: Invokes target-line-selection. undef == no selection
) = @_;
my $full_file_spec;	    # undef if does not exist




my ($dir, $file_name, $file_type) = ENV_split_spec_dnt( $file_spec);
if ($file_type ne '')
{
$full_file_spec = (-e $file_spec) ? $file_spec : undef;
} else
{
$full_file_spec = GBSFILEGLO_select_usr_gbs( $file_spec);
}




if (defined $full_file_spec)
{
GBSFILEGLO_open_file( $full_file_spec, $inc_path_ref, undef, $target, 0);
} else
{
ENV_error_exit( "File '$file_spec.gbs' does not exist")
if ($must_exist);
}

return $full_file_spec;
}




sub GBSFILE_get_line()
{
my $line;

$line = GBSFILEGLO_get_line();
GBSFILEGLO_close_file()
if (!defined $line);

return $line;
}




sub GBSFILE_info(@)
{
my ($first_line,
@lines,
) = @_;

GBSFILEGLO_info( $first_line, @lines);
}




sub GBSFILE_warning(@)
{
my ($first_line,
@lines,
) = @_;

GBSFILEGLO_warning( $first_line, @lines);
}




sub GBSFILE_error(@)
{
my ($first_line,
@lines,
) = @_;

GBSFILEGLO_error( $first_line, @lines);
}




sub GBSFILE_error_exit(@)
{
my ($first_line,
@lines,
) = @_;

GBSFILEGLO_error_exit( $first_line, @lines);
}

return 1;


