#==============================================================
#
#   gbsglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsglo;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSGLO_reset
GBSGLO_subsystems
GBSGLO_is_subsystem
GBSGLO_subsystems_full_gbs
GBSGLO_subsystems_non_gbs
GBSGLO_subsystem_is_full_gbs
GBSGLO_subsystems_for_target
GBSGLO_subsystems_for_audit
GBSGLO_subsystems_for_tool
GBSGLO_subsystem_targets
GBSGLO_subsystem_audits
GBSGLO_subsystem_tools
GBSGLO_components
GBSGLO_components_for_target
GBSGLO_glob
GBSGLO_validate_direntry
GBSGLO_select_direntries
GBSGLO_exclude_direntries
GBSGLO_short_filespecs
GBSGLO_subsystem_exists
GBSGLO_component_exists
);

use strict;
use integer;

use glo::env;
use glo::slurp;
use gbsenv;




sub GBSGLO_reset();
sub GBSGLO_subsystems();
sub GBSGLO_is_subsystem($);
sub GBSGLO_subsystems_full_gbs();
sub GBSGLO_subsystems_non_gbs();
sub GBSGLO_subsystem_is_full_gbs($;$);
sub GBSGLO_subsystems_for_target($);
sub GBSGLO_subsystems_for_audit($$);
sub GBSGLO_subsystems_for_tool($);
sub GBSGLO_subsystem_targets($);
sub GBSGLO_subsystem_audits($);
sub GBSGLO_subsystem_tools($);
sub GBSGLO_components($);
sub GBSGLO_components_for_target($$);
sub GBSGLO_glob($);
sub GBSGLO_validate_direntry($);
sub GBSGLO_select_direntries(@);
sub GBSGLO_exclude_direntries(@);
sub GBSGLO_short_filespecs(@);
sub GBSGLO_subsystem_exists($$);
sub GBSGLO_component_exists($$$);

sub is_full_gbs($);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_ROOT_QM = quotemeta( $GBS_ROOT_PATH);
my $GBS_SCRIPTS_PATH = GBSENV_perl_gbs_scripts();
my $GBS_SCRIPTS_QM = quotemeta( $GBS_SCRIPTS_PATH);

my @GBS_SKIPTYPES = ENV_getenv( 'GBS_SKIPTYPES');


my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my %GBS_TARGETS = map { $_ => 1 } @GBS_TARGETS;

my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
my %GBS_AUDITS = map { $_ => 1 } @GBS_AUDITS;

my @GBS_TOOLS = ENV_getenv( 'GBS_TOOLS');
my %GBS_TOOLS = map { $_ => 1 } @GBS_TOOLS;




my @ALL_SUBSYSTEMS;
my %LOCAL_FULL_GBS_SUBSYSTEMS;




sub GBSGLO_reset()
{
@ALL_SUBSYSTEMS = ();
%LOCAL_FULL_GBS_SUBSYSTEMS = ();
}




sub GBSGLO_subsystems()
{
if (@ALL_SUBSYSTEMS == 0)
{
@ALL_SUBSYSTEMS = GBSGLO_select_direntries( SLURP_dir_dirs( "$GBS_ROOT_PATH/dev", 0))
if ($GBS_ROOT_PATH ne '');
} else
{

}

return @ALL_SUBSYSTEMS;
}




sub GBSGLO_is_subsystem($)
{
my ($subsys) = @_;

return (-d "$GBS_ROOT_PATH/dev/$subsys") ? 1 : 0;
}




sub GBSGLO_subsystems_full_gbs()
{
return grep( is_full_gbs( $_), GBSGLO_subsystems());
}




sub GBSGLO_subsystems_non_gbs()
{
return grep( ! is_full_gbs( $_), GBSGLO_subsystems());
}




sub GBSGLO_subsystem_is_full_gbs($;$)
{
my ($subsys,
$root_path,	# optional. Use ONLY if $root_path ne $GBS_ROOT_PATH
) = @_;
my $is_full_gbs;

if (defined $root_path)
{
$is_full_gbs = (-f "$root_path/dev/$subsys/full.gbs") ? 1 : 0;
} else
{
$is_full_gbs = is_full_gbs( $subsys);
}

return $is_full_gbs;
}




sub GBSGLO_subsystems_for_target($)
{
my ($target) = @_;

return grep( is_full_gbs( $_) && -d "$GBS_ROOT_PATH/dev/$_/gen/$target", GBSGLO_subsystems());
}




sub GBSGLO_subsystems_for_audit($$)
{
my ($audit,
$target,
) = @_;

return grep( is_full_gbs( $_) &&
-d "$GBS_ROOT_PATH/dev/$_/audit/$audit" &&
-d "$GBS_ROOT_PATH/dev/$_/gen/$target",
GBSGLO_subsystems());
}




sub GBSGLO_subsystems_for_tool($)
{
my ($tool) = @_;

return grep( is_full_gbs( $_) && -d "$GBS_ROOT_PATH/dev/$_/tools/$tool", GBSGLO_subsystems());
}





sub GBSGLO_subsystem_targets($)
{
my ($subsys) = @_;
my @targets;

if (is_full_gbs( $subsys))
{
@targets = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/gen", 0);
@targets = grep { exists $GBS_TARGETS{$_} } @targets;
} else
{
@targets = @GBS_TARGETS;
}

return @targets;
}




sub GBSGLO_subsystem_audits($)
{
my ($subsys) = @_;
my @audits;

if (is_full_gbs( $subsys))
{
@audits = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/audit", 0);
@audits = grep { exists $GBS_AUDITS{$_} } @audits;
} else
{
@audits = @GBS_AUDITS;
}

return @audits;
}




sub GBSGLO_subsystem_tools($)
{
my ($subsys) = @_;
my @tools;

if (is_full_gbs( $subsys))
{
@tools = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/tools", 0);
@tools = grep { exists $GBS_TOOLS{$_} } @tools;
} else
{
@tools = @GBS_TOOLS;
}

return @tools;
}




sub GBSGLO_components($)
{
my ($subsys) = @_;

return GBSGLO_select_direntries( SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/comp", 0));
}




sub GBSGLO_components_for_target($$)
{
my ($subsys,
$target,
) = @_;

return grep( -d "$GBS_ROOT_PATH/dev/$subsys/comp/$_/bld/$target",
GBSGLO_components( $subsys));
}




sub GBSGLO_glob($)
{
my ($glob_string) = @_;

return GBSGLO_select_direntries( glob( $glob_string));
}




sub GBSGLO_validate_direntry($)
{
my ($direntry) = @_;

return (grep( $direntry =~ $_, @GBS_SKIPTYPES)) ? 0 : 1;
}




sub GBSGLO_select_direntries(@)
{
my @out;


foreach my $direntry (@_)
{
my $name = ENV_split_spec_f( $direntry);

push @out, $direntry unless (grep( $name =~ $_, @GBS_SKIPTYPES));
}

return @out;
}




sub GBSGLO_exclude_direntries(@)
{
my @out;


foreach my $direntry (@_)
{
my $name = ENV_split_spec_f( $direntry);

push @out, $direntry if (grep( $name =~ $_, @GBS_SKIPTYPES));
}

return @out;
}




sub GBSGLO_short_filespecs(@)
{

my @short_filespecs;

foreach my $filespec (@_)
{
if ($filespec)
{
my $short_filespec = $filespec;
if (!($short_filespec =~ s/^$GBS_ROOT_QM/\$GBS_ROOT_PATH/))
{
$short_filespec =~ s/^$GBS_SCRIPTS_QM/\$GBS_SCRIPTS_PATH/
}
push @short_filespecs, $short_filespec;
} else
{
push @short_filespecs, $filespec
}
}

return (wantarray) ? @short_filespecs : $short_filespecs[0];
}





sub GBSGLO_subsystem_exists($$)
{
my ($root_path,
$subsys,  #case sensitive!
) = @_;
my $exists;

$exists = -d "$root_path/dev/$subsys";
if (ENV_is_win32() && $exists)
{
$exists = 0;
foreach my $entry (SLURP_dir_all( "$root_path/dev", 0))
{
if ($entry eq $subsys)
{
$exists = 1;
last;
}
}
}

return $exists;
}





sub GBSGLO_component_exists($$$)
{
my ($root_path,
$subsys,    #case sensitive!
$component, #case sensitive!
) = @_;
my $exists;

$exists = -d "$root_path/dev/$subsys/comp/$component";
if (ENV_is_win32() && $exists)
{
$exists = 0;
foreach my $entry (SLURP_dir_all( "$root_path/dev/$subsys/comp", 0))
{
if ($entry eq $component)
{
$exists = 1;
last;
}
}
}

return $exists;
}




sub is_full_gbs($)
{
my ($subsys) = @_;
my $is_full_gbs;

$is_full_gbs = $LOCAL_FULL_GBS_SUBSYSTEMS{$subsys};
if (!defined $is_full_gbs)
{
$is_full_gbs = (-f "$GBS_ROOT_PATH/dev/$subsys/full.gbs") ? 1 : 0;
$LOCAL_FULL_GBS_SUBSYSTEMS{$subsys} = $is_full_gbs;
} else
{

}

return $is_full_gbs;
}

return 1;


