#==============================================================
#
#   gbsinit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
print "*GBSINIT*\n" if ($ENV{GBSDEBUG_FILE});

use 5.016_003;
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::time;
use glo::genopt;
use glo::build;
use glo::banner;
use glo::ask;
use gbsenv;
use gbsres;
use gbscmd;
use roots;
use setup;
use profile;
use gbs;














$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'rel',    'rel',    'sso', '.' , 'Specific GBS release' ],
[ 'noroot', 'noroot', 'bso', 0   , 'Do not implicitly swr to current root' ],
);
GENOPT_set_optdefs( [ 'gbsinit', 'gbs' ], \@genopts,
'Startup GBS',
"Alias for this command is 'gbs'");
GENOPT_parse();
my $REL = GENOPT_get( 'rel');
my $NOROOT = GENOPT_get( 'noroot');

$REL = ''
if ($REL eq 'cur' || $REL eq 'current');




{



my $gbs_log_dir = ENV_getenv( 'GBS_LOG_PATH');
my $gbs_site = ENV_getenv( 'GBS_SITE');
my @LINES = GBS_init( $REL);


if ($gbs_log_dir eq '' || $gbs_site eq '')
{



SETUP_show_licence();

ENV_say( 1, "Some essential data is missing. You need to supply that first:",
'');
PROFILE_open( undef);
if ($gbs_log_dir eq '')
{
$gbs_log_dir = SETUP_ask_log_dir( '');
ENV_setenv( GBS_LOG_PATH => $gbs_log_dir);
PROFILE_set( GBS_LOG_PATH => $gbs_log_dir);
}
if ($gbs_site eq '')
{
$gbs_site = SETUP_ask_site( '');
ENV_setenv( GBS_SITE => $gbs_site);
PROFILE_set( GBS_SITE => $gbs_site);
}
PROFILE_close( 1);	# $must write
}
GBS_validate();




{
my $TITLE = GBSRES_title();
my $gbs_show_version = BUILD_get_show_version();
my ($LICS, $LICN, $LICE, $LICD) = BUILD_get_lic();
my $TIME_START = TIME_time2num( time);
my $UNAME;
{
my ($os, $host, @rest) = ENV_uname();
$UNAME = "$host ($os [@rest])";
}



my @lines;

push @lines,  [ '',      $TITLE ];
push @lines,  undef;						# prints a separator line
push @lines,  [ 'VERSION:', "$gbs_show_version" ];
push @lines,  [ 'LICENSE:', "$LICN ($LICS)" ];
push @lines,  [ '',         "Expires:  $LICE. Number of days left: $LICD" ];
push @lines,  [ '',	        "*** Expired ***" ]
if ($LICD < 0);
push @lines,  [ 'START:',   "$TIME_START" ];
push @lines,  [ 'HOST:',    $UNAME ];

BANNER_print( 'JOB', [ @lines ]);

if ($LICD < 0)
{
ENV_warn( "This license of GBS has expired.",
"Please contact Randy Marques Consultancy for a new license.",
"You may keep using it free of change for MAINTENANCE purposes for old, finished projects.",
"- Some functionality will be disabled.");
my $rand = time % 10000;
if (ASK_int( "Please enter '$rand'", '0', undef, 0) != $rand)
{
ENV_error_exit( "Invalid", "Cannot continue - GBS not started");
}
}
}




if (!$NOROOT)
{
if (ROOTS_get_nr() > 0)
{



push @LINES, GBSCMD_command( 'gbsswr', '.');
} else
{
ENV_say( 1, "No Roots/Systems defined for this user",
"- Type 'swr --add' to use an existing Root/System",
"- or   'swr --new' to define a new Root/System",
"- or   'gbshelp'   to get extensive help on how to proceed");
}
}




GBSENV_write_result_script( \@LINES, 0);
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);

}


