#===============================================================
#
#   gbslocate.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================


use strict;
use integer;




sub usage();




my $PREFIX = 'GBSLOCATE';






$| = 1;
my $RC = 0;

=optdoc
use glo::genopt;

my @genopts = (
[ '<1>', 'env_name',  'ssm', "", "Env Variable to contain the filespec" ],
[ '<2>', 'file',	  'ssm', "", "The file to search" ],
[ '<3>', 'prefix',	  'ssm', "", "prefix string preceding all path-list elements. e.g: -I" ],
[ '<*>', 'path-list', 'sam', "", "The directories to search preceded by <prefix>" ],
);
GENOPT_set_optdefs( 'gbslocate',\@genopts,
'Locate a file according to include-settings and assigns the location to a specified Environment Variable',
undef);

=cut




if (@ARGV == 1 && $ARGV[0] =~ /^--?(h|help)$/)
{
usage();
exit 0;
} else
{
if (@ARGV < 4)
{
print "**SYNTAX-EROR**\n";
usage();
exit 9;
}
}
my $env_name = shift @ARGV;
my $file = shift @ARGV;
my $prefix = shift @ARGV;
my @paths = @ARGV;

if ($env_name !~ /^\w+$/)
{
print STDERR "$PREFIX: <env_name> must consist of 'word' characters\n";
usage();
exit 1;
}






my $prefix_l = length $prefix;
my $file_spec;
my $found = 0;
my $sep = ($^O eq 'MSWin32') ? '\\' : '/';
foreach my $dir_spec (@paths)
{
my $dir = substr( $dir_spec, $prefix_l);
$file_spec = "${dir}${sep}$file";
if (-e $file_spec)
{
$found = 1;
last;
}
}
if ($found)
{
print "$file_spec\n";
} else
{
print STDERR "$PREFIX: File '$file' not found in @paths\a\n";
print "**$file-NOTFOUND**\n";
}

exit $RC;




sub END
{
print "$PREFIX: Failed ($?)\n" if ($?);
}




sub usage()
{



print "$PREFIX: usage is: gbslocate <env_name> <file> <prefix> <prefix><dir> [ <prefix><dir> ]...\n";
}


