#===============================================================
#
#   gbsrc.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package gbsrc;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
GBSRC_get_base
GBSRC_write_base
GBSRC_get_root
GBSRC_write_root
GBSRC_get_subsys
GBSRC_write_subsys
);

use strict;
use integer;

use glo::env;




sub GBSRC_get_base($);
sub GBSRC_write_base(;@);
sub GBSRC_get_root($$);
sub GBSRC_write_root($;@);
sub GBSRC_get_subsys($$$);
sub GBSRC_write_subsys($$;@);

sub setup($);
sub do_read($$);
sub do_write($$);
sub read_file($);
sub get_item($);
sub put_items($);
sub write_file();




my %BASE = (
SPEC => ''
);
my %BASE_DEF = (
DATA    => \%BASE,
ID	    => 'root',
ITEMS   => [ qw( id root ) ],
OSITEMS => [ qw( root) ],
HIDE    => 0,
);

my %ROOT = (
SPEC => ''
);
my %ROOT_DEF = (
DATA    => \%ROOT,
ID	    => 'subsys',
ITEMS   => [ qw( id subsys target audit ) ],
OSITEMS => [ qw( target audit) ],
HIDE    => 1,
);

my %SUBSYS = (
SPEC => ''
);
my %SUBSYS_DEF = (
DATA    => \%SUBSYS,
ID	    => 'component',
ITEMS   => [ qw( id component ) ],
OSITEMS => [],
HIDE    => 1,
);

my $CUR_DATA_REF;
my $CUR_ID;
my @CUR_ITEMS;
my %CUR_ITEMS;
my %CUR_OSITEMS;
my $CUR_HIDE;




sub GBSRC_get_base($)
{
my ($wanted_key) = @_;

my $gbs_base_dir = ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $file_spec = "$gbs_base_dir/.gbsrc";

setup( \%BASE_DEF);
return do_read( $file_spec, $wanted_key);
}




sub GBSRC_write_base(;@)
{
my (@key_value_pairs) = @_;     # ($wanted_key => $value, ...)

my $gbs_base_dir = ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $file_spec = "$gbs_base_dir/.gbsrc";

setup( \%BASE_DEF);
do_write( $file_spec, \@key_value_pairs);
}




sub GBSRC_get_root($$)
{
my ($cur_root,
$wanted_key,
) = @_;

my $file_spec = "$cur_root/dev/.gbsrc";

setup( \%ROOT_DEF);
return do_read( $file_spec, $wanted_key);
}




sub GBSRC_write_root($;@)
{
my ($cur_root,
@key_value_pairs    # ($wanted_key => $value, ...)
) = @_;

my $file_spec = "$cur_root/dev/.gbsrc";

setup( \%ROOT_DEF);
do_write( $file_spec, \@key_value_pairs);
}




sub GBSRC_get_subsys($$$)
{
my ($cur_root,
$cur_subsys,
$wanted_key,
) = @_;

my $file_spec = "$cur_root/dev/$cur_subsys/comp/.gbsrc";

setup( \%SUBSYS_DEF);
return do_read( $file_spec, $wanted_key);
}




sub GBSRC_write_subsys($$;@)
{
my ($cur_root,
$cur_subsys,
@key_value_pairs    # ($wanted_key => $value, ...)
) = @_;

my $file_spec = "$cur_root/dev/$cur_subsys/comp/.gbsrc";

setup( \%SUBSYS_DEF);
do_write( $file_spec, \@key_value_pairs);
}




sub setup($)
{
my ($def_ref) = @_;

$CUR_DATA_REF = $def_ref->{DATA};
$CUR_ID = $def_ref->{ID};
@CUR_ITEMS = @{$def_ref->{ITEMS}};
%CUR_ITEMS = map { $_ => 1 } @CUR_ITEMS;
%CUR_OSITEMS = map { $_ => 1 } @{$def_ref->{OSITEMS}};
$CUR_HIDE = $def_ref->{HIDE};
}




sub do_read($$)
{
my ($file_spec,
$wanted_key,
) = @_;

read_file( $file_spec);

return get_item( $wanted_key);
}




sub do_write($$)
{
my ($file_spec,
$key_value_pairs_ref,    # ($wanted_key => $value, ...)
) = @_;

read_file( $file_spec);
if (put_items( $key_value_pairs_ref))
{
ENV_debug( "Writing GBSRC ($CUR_ID)...");		    # TBR
write_file();
}
}




sub read_file($)
{
my ($file_spec) = @_;

my $current_file = $CUR_DATA_REF->{SPEC};
if ($file_spec ne $current_file)
{



%{$CUR_DATA_REF} = ();
foreach my $item (@CUR_ITEMS)
{
if ($CUR_OSITEMS{$item})
{
$CUR_DATA_REF->{$item}->{$^O} = '';
} else
{
$CUR_DATA_REF->{$item} = '';
}
}


if (open( IN, $file_spec))
{
my @lines = <IN>;
close IN;
foreach my $line (@lines)
{
$line =~ s/\s+$//;			# remove trailing white space
next if ($line eq "");			# skip empty lines
next if (substr( $line, 0, 1) eq "#");	# skip comments
my ($name, $value) = split( /=/, $line, 2);
my ($key, $osname) = split( /-/, $name, 2);
$value = '' if (! defined $value);
if ($CUR_ITEMS{$key})
{
if (defined $osname)
{
$CUR_DATA_REF->{$key}->{$osname} = $value;
} else
{
$CUR_DATA_REF->{$key} = $value;
}
} else
{
ENV_warn( "Invalid key '$name' in file $file_spec");
}
}
} else
{

}

$CUR_DATA_REF->{SPEC} = $file_spec;
$CUR_DATA_REF->{id} = $CUR_ID;
}
}




sub get_item($)
{
my ($wanted_key) = @_;
my $value;

my ($key, $osname) = split( /-/, $wanted_key, 2);
ENV_fatal( "$CUR_ID: No such key '$key'")
if (!$CUR_ITEMS{$key});
if ($CUR_OSITEMS{$key})
{
$osname = $^O
if (!defined $osname);
$value = $CUR_DATA_REF->{$key}->{$osname};
} else
{
$value = $CUR_DATA_REF->{$key};
}

return $value;
}





sub put_items($)
{
my ($key_value_pairs_ref) = @_;    # ($wanted_key => $value, ...)
my $file_modified = 0;

my @key_value_pairs = @{$key_value_pairs_ref};
while (@key_value_pairs)
{
my $wanted_key = shift @key_value_pairs;
my $value = shift @key_value_pairs;

my ($key, $osname) = split( /-/, $wanted_key, 2);
ENV_fatal( "$CUR_ID: No such key '$key'")
if (!$CUR_ITEMS{$key});
if ($CUR_OSITEMS{$key})
{
$osname = $^O
if (!defined $osname);
my $old_value = $CUR_DATA_REF->{$key}->{$osname};
if ($value ne $old_value)
{
$CUR_DATA_REF->{$key}->{$osname} = $value;
$file_modified = 1;
}
} else
{
my $old_value = $CUR_DATA_REF->{$key};
if ($value ne $old_value)
{
$CUR_DATA_REF->{$key} = $value;
$file_modified = 1;
}
}
}

return $file_modified;
}





sub write_file()
{
my @lines;
foreach my $key (@CUR_ITEMS)
{
if ($CUR_OSITEMS{$key})
{
foreach my $os_name (sort keys %{$CUR_DATA_REF->{$key}})
{
push @lines, "$key-$os_name=$CUR_DATA_REF->{$key}->{$os_name}\n";
}
} else
{
push @lines, "$key=$CUR_DATA_REF->{$key}\n";
}
}
my $current_file = $CUR_DATA_REF->{SPEC};


ENV_hide_file( $current_file, 0)
if (-e $current_file);	# unhide
unlink $current_file;					# On linux we HAVE to delete the DOT file first!
if (open( OUT, ">$current_file"))				# SPIT_file not used because we want to trap eror
{
print OUT @lines;
close OUT;
ENV_hide_file( $current_file, 1)			# hide
if ($CUR_HIDE);
} else
{
ENV_warn( "$CUR_ID: Unable to create '$current_file'",
"- Values will not be remembered!",
"- $!");
}
}

return 1;
