#==============================================================
#
#   gbsshow.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#print "\n*GBSSHOW*\n";
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use glo::format;
use system;
use gbsenv;
use glo::build;




sub show_currencies();




my $BRIEF;

my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'brief', 'brief',	   'bso',   0, "Brief (short) listing" ],
);
GENOPT_set_optdefs( [ 'gbsshow', 'gbs' ], \@genopts,
'Show GBS currencies',
undef);
GENOPT_parse();
$BRIEF = GENOPT_get( 'brief');




ENV_say( 1, '');
ENV_say( 0, show_currencies());




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub show_currencies()
{
my @lines;




if (!$BRIEF)
{
push @lines, "  Settings:";
my @row_refs;




my $gbs_show_version = BUILD_get_show_version();
my $gbs_platform = ENV_getenv( 'GBS_PLATFORM');
push @row_refs, [ 'GBS        ', ': ', "Version: $gbs_show_version on $gbs_platform" ];




my ($lics, $licn, $lice, $licd) = BUILD_get_lic();
push @row_refs, [ 'License', ': ', "$licn ($lics)" ];
push @row_refs, [ '',        ': ', "Expires: $lice. Remaining days: $licd" ];
if ($licd < 0)
{
push @row_refs, [ '',        ': ', "** Expired **" ];
ENV_warn( 'License expired!');
}




my $GBS_SITE = ENV_getenv( 'GBS_SITE');
push @row_refs, [ 'Site', ': ', $GBS_SITE ];




my @roles;
push @roles, 'Integrator' if (GBSENV_is_integrator());
push @roles, 'Administrator' if (GBSENV_is_administrator());
push @row_refs, [ 'Role(s)', ': ', "@roles" ];




if ($GBS_ROOT_PATH ne '')
{
my $gbs_scms = ENV_getenv( 'GBS_SCMS');
my $gbs_scms_repository = ENV_getenv("GBS_SCMS_REPOSITORY");
my $gbs_scms_data = ENV_getenv("GBS_SCMS_DATA");
my $text = $gbs_scms;
$text .= ": $gbs_scms_repository" if ($gbs_scms_repository ne '');
$text .= "($gbs_scms_data)" if ($gbs_scms_data ne '');
push @row_refs, [ 'SCMS', ': ', $text ];
}

push @lines, FORMAT_table( 0, 4, '', undef, @row_refs);
}




my @row_refs;
push @lines, "  Currencies:";

if ($GBS_ROOT_PATH ne '')
{



my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
if ($BRIEF)
{
push @row_refs, [ 'System Name', ': ', "$gbs_system_name" ];
} else
{
my $build_start = SYSTEM_get( $GBS_ROOT_PATH, 'build_start');
my $build_end = SYSTEM_get( $GBS_ROOT_PATH, 'build_end');
push @row_refs, [ 'System Name', ': ', "$gbs_system_name [$build_start - $build_end]" ];
}




if ($BRIEF)
{
push @row_refs, [ 'Root', ': ', "$GBS_ROOT_PATH" ];
} else
{
my $gbs_root_version = ENV_getenv( 'GBS_ROOT_VERSION');
push @row_refs, [ 'Root', ': ', "$GBS_ROOT_PATH [$gbs_root_version]" ];
}




my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');
my $gbs_full = ENV_getenv( 'GBS_FULL');
if ($gbs_subsys ne "")
{
if ($gbs_full)
{
push @row_refs, [ 'SubSystem', ': ', "$gbs_subsys [Full GBS]" ];
} else
{
push @row_refs, [ 'SubSystem', ': ', "$gbs_subsys [Non GBS]" ];
}
} else
{
push @row_refs, [ 'SubSystem', ': ', '' ];
}

if ($gbs_full)
{
my $gbs_component = ENV_getenv( 'GBS_COMPONENT');
push @row_refs, [ 'Component', ': ', $gbs_component];
}

my $gbs_target = ENV_getenv( 'GBS_TARGET');
push @row_refs, [ 'Target', ': ', $gbs_target];
my $gbs_audit = ENV_getenv( 'GBS_AUDIT');
push @row_refs, [ 'Audit', ': ', $gbs_audit];
} else
{



push @row_refs, [ 'System Name', ': ', '*** No Current Root ***' ];
}




push @row_refs, [ 'Currenty in', ': ', ENV_cwd() ];

push @lines, FORMAT_table( 0, 4, '', undef, @row_refs);

return @lines;
}


