#==============================================================
#
#   gbsstart.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSSTART*\n" if ($ENV{GBSDEBUG_FILE});
use 5.016_003;
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::spit;
use glo::genopt;
use gbsenv;
use gbscmd;
use gbs;














$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'root', 	   'ssm',  '!', "System / Root: '-' == none, '.' == current, '!' == CWD, '!<abs_file/dir_spec>', '!!' == CWD = GBS_ROOT_PATH" ],
[ '<2>',	'subsystem',	   'sso',  '', "SubSystem: '-' == none, '.' or '' == current" ],
[ '<3>',	'component',	   'sso',  '', "Component: '-' == none, '.' or '' == current" ],
[ 'target',	'target',	   'sso',  '.', "Target: '-' == none, '.' or '' == current" ],
[ 'exec',	'execute_command', 'ssm',  '', "(GBS-)Command to execute. 'none' = OK" ],
[ 'rel',	'rel',		   'sso',  '.', 'Specific GBS release'],
);
my @genconflicts = (
[ [ '<1>' => '-' ], '=', '<2>' ],
[ [ '<1>' => '-' ], '=', '<3>' ],
[ [ '<2>' => '-' ], '=', '<3>' ],
);
GENOPT_set_optdefs( 'gbsstart', \@genopts,
'Startup GBS and execute a (GBS) command. To be used for integration of GBS in other tools',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $ROOT = GENOPT_get( 'root');
my $SUBSYS = GENOPT_get( 'subsystem');
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $EXEC = GENOPT_get( 'execute_command');
my $REL = GENOPT_get( 'rel');




{
my @LINES = GBS_init( $REL);
GBS_validate();




if (ENV_is_win32())
{
$EXEC =~ s/ /\$_/g;
} else
{
$EXEC =~ s/ /%20/g;
}

{
my @args;
push @args, $ROOT;
push @args, $SUBSYS		if ($SUBSYS ne '');
push @args, $COMPONENT		if ($COMPONENT ne '');
push @args, "--target=$TARGET"  if ($TARGET ne '');
push @args, "--exec=$EXEC";
push @LINES, GBSCMD_command( 'gbsswr', @args);
}




my $file_spec = 'gbsstart_exec';

map $_ .= "\n", @LINES;

$file_spec = SPIT_tmp_script_file( $file_spec, [ @LINES ]);
$RC = ENV_system( ENV_command_ref( $file_spec, undef), undef, undef);
unlink $file_spec;
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}


