#===============================================================
#
#   gbssub.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

print "*GBSSUB*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use gbsenv;
use gbscmd;
use fail;
use target;
use flincs;
use exec;
use run;




sub get_command_args();








my ($JOBTYPE,	    # gen, make, etc
$IGNORE_ERRORS, # bool
);

my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT  = ENV_getenv( 'GBS_AUDIT');

my %JOBTYPES = (
audit	=> [ \&RUN_gbssubaudit ],
gen		=> [ \&RUN_gbssubgen ],
make	=> [ \&RUN_gbssubmake ],
cleanup_aud	=> [ \&RUN_gbssubcleanup_aud ],
cleanup_bld	=> [ \&RUN_gbssubcleanup_bld ],
);

my $FOR_GEN = 0;
my $FOR_MAKE = 0;
my $FOR_AUDIT = 0;
my $FOR_CLEANUP_BLD = 0;
my $FOR_CLEANUP_AUD = 0;
my $FOR_BUILD = 0;	    # gen | make
my $FOR_CLEANUP = 0;	    # cleaup_aud | cleaup_build
















$| = 1;
my $RC = 0;




GBSENV_init();

get_command_args();

if ($JOBTYPE eq 'gen')
{
$FOR_GEN = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'make')
{
$FOR_MAKE = 1;
$FOR_BUILD = 1;
} elsif ($JOBTYPE eq 'audit')
{
$FOR_AUDIT = 1;
} elsif ($JOBTYPE eq 'cleanup_bld')
{
$FOR_CLEANUP_BLD = 1;
$FOR_CLEANUP = 1;
} elsif ($JOBTYPE eq 'cleanup_aud')
{
$FOR_CLEANUP_AUD = 1;
$FOR_CLEANUP = 1;
} else
{
ENV_fatal( "Invalid JOBTYPE '$JOBTYPE'");
}

ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);
ENV_setenv( GBS_MAKE => GBSCMD_make());




{
TARGET_set_envs( $GBS_TARGET);	    # set GBS_BLD_* and GBS_options

my %options;

foreach my $src_type (TARGET_get_src_types( $GBS_TARGET))
{
if ($FOR_BUILD || $FOR_AUDIT)	    # exclude FOR_MAKE and FOR_CLEANUPs
{



my $key = 'GBS_FLAGS_' . uc substr( $src_type, 1);
my @flags = FLINCS_get_flags( $src_type, '');
my $gbs_flag = ENV_getenv( $key);
push @flags, $gbs_flag
if ($gbs_flag ne '');

foreach my $opt_ref (TARGET_get_default_options( $GBS_TARGET, $src_type))    #  MODE OPT DEBUGGER MAP
{
my ($option, $default_choice) = @{$opt_ref};
my $choice = ENV_getenv( "GBS_$option");
my @values = TARGET_get_src_subitems( $GBS_TARGET, $src_type, $option, $choice);
push @flags, @values;
$options{$option} = [ $default_choice, $choice ];
}




@flags = grep( /^"?-D/, @flags)
if ($FOR_AUDIT);

ENV_setenv( $key => "@flags");
}
}




my @banner;
foreach my $option (TARGET_get_options_order())	# MODE OPT DEBUGGER MAP
{
my $opt_ref = $options{$option};
if (defined $opt_ref)
{
my ($default_choice, $choice) = @{$opt_ref};

my $banner = "$option=$choice";
$banner .= " ($default_choice)" if ($choice ne $default_choice);
push @banner, $banner;
}
}
my $banner = join( ', ', @banner);
ENV_say( 1, "** $banner");




my $func = $JOBTYPES{$JOBTYPE}->[0];
$RC = &$func();
ENV_say( 0, '-');
FAIL_log( "gbssub$JOBTYPE", $RC, $GBS_SUBSYS, '', '')
if ($RC != 0);

if ($FOR_BUILD)
{



if (-d "$GBS_ROOT_PATH/res/$GBS_SUBSYS" || -d "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/export")
{
if ($IGNORE_ERRORS || $RC == 0)
{
ENV_say( 1, "Implicit gbsexport of $GBS_SUBSYS");
my @gbsexport_opts;
push @gbsexport_opts, '--i' if ($IGNORE_ERRORS);
my $this_rc = EXEC_run( 'gbsexport.pl', \@gbsexport_opts);
$RC = $this_rc if ($this_rc > $RC);
FAIL_log( 'gbsexport', $this_rc, $GBS_SUBSYS, '', '')
if ($this_rc != 0);
} else
{
ENV_say( 1, "SKIPPED: gbsexport of $GBS_SUBSYS");
}
}
}
}




ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 1);
}




sub get_command_args()
{


($JOBTYPE,	     # gen, make or audit
$IGNORE_ERRORS, # bool
) = ARGS_get( 2, 'perl gbssub.pl audit|gen|make|cleanp_bld|cleanup_aud <ignore_errors>', undef);
}



