#==============================================================
#
#   gbsswc.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
print "*GBSSWC*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swt;
use swc;
use swglo;




sub ask_component();




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');

my $OLD_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $NEW_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $OLD_TARGET = ENV_getenv( 'GBS_TARGET');
my $NEW_TARGET = ENV_getenv( 'GBS_TARGET');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'component',		'sso',	'', "Component: '' == ASK, '-' == none, '.' == current" ],
[ 'target',	'target',		'sso',	'', "Target: '-' == none, '.' == current" ],
[ 'new',	'create_new_component', 'bso',	 0, "Create a new Component" ],
[ 'nocd',	'no_cd',		'bso',   0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
[ [ new => 1], '=', '<1>', '=', 'target' ],
);
GENOPT_set_optdefs( [ 'gbsswc', 'swc' ], \@genopts,
'Set and/or Create new Current Component',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $NEW = GENOPT_get( 'create_new_component');
my $NO_CD = GENOPT_get( 'no_cd');







VALIDATE_subsys_full_gbs();

my $GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";




if ($NEW)
{
$COMPONENT = RUN_gbsnew( 'new_component', $COMPONENT);
if ($COMPONENT eq '')
{
$COMPONENT = '.';
$CANCELLED = 1;
}
}

if (!$CANCELLED)
{



if ($COMPONENT eq '')
{
$COMPONENT = ask_component();
}
($NEW_TARGET, $NEW_COMPONENT) = SWGLO_validate( $TARGET, $COMPONENT);






if ($NEW_COMPONENT ne ENV_getenv( 'GBS_COMPONENT'))
{
my $must_write = $NEW_COMPONENT ne '' || $COMPONENT eq '-';
SWC_set( $GBS_ROOT_PATH, $GBS_SUBSYS, $NEW_COMPONENT, $must_write);
}
if ($NEW_TARGET ne ENV_getenv( 'GBS_TARGET'))
{
SWT_set( $GBS_ROOT_PATH, $NEW_TARGET, 1);
}
ENV_warn( "No current Component") if ($NEW_COMPONENT eq '' && $COMPONENT ne '-');
ENV_warn( "No current Target") if ($NEW_TARGET eq '' && $TARGET ne '-');










{
my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $GBS_SUBSYS, $NEW_COMPONENT);

my @lines;
if (ENV_get_changed_envs() || $new_cwd ne $CWD)
{
push @lines, SWC_exit( $GBS_ROOT_PATH, $GBS_SUBSYS, $OLD_COMPONENT) if ($OLD_COMPONENT ne '');

push @lines, GBSENV_changed_setenv_commands( 0);

push @lines, SWC_entry( $GBS_ROOT_PATH, $GBS_SUBSYS, $NEW_COMPONENT) if ($NEW_COMPONENT ne '');
push @lines, SHELL_cd( $new_cwd);
}

push @lines, GBSCMD_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub ask_component()
{
my $component;




my @all_components = GBSGLO_components( $GBS_SUBSYS);

if (scalar @all_components == 0)
{
ENV_error_exit( "No Component(s) defined - use 'swc --new' to define Components");
} elsif (scalar @all_components == 1)
{
$component = $all_components[0];
ENV_say( 1, "Single Component '$component' taken by default");
} else
{



my $default_value = ENV_getenv( 'GBS_COMPONENT');
$component = ASK_value_from_menu( "Select Component",
\$default_value, ( '', @all_components ));
$component = '-'
if ($component eq '');
}

return $component;
}


