#==============================================================
#
#   gbssws.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
print "*GBSSWS*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swt;
use swc;
use sws;
use swglo;




sub ask_subsys();




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();

my $OLD_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $NEW_SUBSYS = ENV_getenv( 'GBS_SUBSYS');

my $OLD_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $NEW_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $NEW_TARGET = ENV_getenv( 'GBS_TARGET');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'subsystem',	     'sso',    '', "SubSystem. '' == ASK, '-' == none, '.' == current" ],
[ '<2>',	'component',	     'sso',    '', "Component. '-' == none, '.' == current" ],
[ 'target',	'target',	     'sso',    '', "Target. '-' == none, '.' == current" ],
[ 'new',	'create_new_subsys', 'bso',     0, "Create a new SubSystem" ],
[ 'nocd',	'no_cd',	     'bso',     0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
[ [ new => 1],	'=', '<1>', '=', '<2>', '=', 'target' ],
[ [ '<1>' => '-' ], '=', '<2>' ],
);
GENOPT_set_optdefs( [ 'gbssws', 'sws' ], \@genopts,
'Set and/or Create new Current SubSystem',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $SUBSYS = GENOPT_get( 'subsystem');
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $NEW = GENOPT_get( 'create_new_subsys');
my $NO_CD = GENOPT_get( 'no_cd');




VALIDATE_root();

my $GBS_DEV_PATH = "$GBS_ROOT_PATH/dev";




if ($NEW)
{
$SUBSYS = RUN_gbsnew( 'new_subsys', $SUBSYS);
if ($SUBSYS eq '')
{
$SUBSYS = '.';
$CANCELLED = 1;
} else
{
$COMPONENT = '-';
}
}

if (!$CANCELLED)
{



if ($SUBSYS eq '')
{
$SUBSYS = ask_subsys();
}
($NEW_TARGET, $NEW_COMPONENT, $NEW_SUBSYS) = SWGLO_validate( $TARGET, $COMPONENT, $SUBSYS);







if ($NEW_SUBSYS ne ENV_getenv( 'GBS_SUBSYS'))
{
SWC_set( $GBS_ROOT_PATH, $NEW_SUBSYS, '', 0);	# Clear Current Component

my $must_write = $NEW_SUBSYS ne '' || $SUBSYS eq '-';
SWS_set( $GBS_ROOT_PATH, $NEW_SUBSYS, $must_write);
}

if ($NEW_COMPONENT ne ENV_getenv( 'GBS_COMPONENT'))
{
my $must_write = $NEW_COMPONENT ne '' || $COMPONENT eq '-';
SWC_set( $GBS_ROOT_PATH, $NEW_SUBSYS, $NEW_COMPONENT, $must_write);
}

if ($NEW_TARGET ne ENV_getenv( 'GBS_TARGET'))
{
SWT_set( $GBS_ROOT_PATH, $NEW_TARGET, 1);
}

ENV_warn( "No current SubSystem") if ($NEW_SUBSYS eq '' && $SUBSYS ne '-');
ENV_warn( "No current Component") if ($NEW_COMPONENT eq '' &&
$COMPONENT ne '-' &&
GBSGLO_subsystem_is_full_gbs( $ENV{GBS_SUBSYS}));
ENV_warn( "No current Target") if ($NEW_TARGET eq '' && $TARGET ne '-');










{
my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $NEW_SUBSYS, $NEW_COMPONENT);

my @lines;
if (ENV_get_changed_envs() || $new_cwd ne $CWD)
{
push @lines, SWC_exit( $GBS_ROOT_PATH, $OLD_SUBSYS, $OLD_COMPONENT) if ($OLD_COMPONENT ne '');
push @lines, SWS_exit( $GBS_ROOT_PATH, $OLD_SUBSYS) if ($OLD_SUBSYS ne '');

push @lines, GBSENV_changed_setenv_commands( 0);

push @lines, SWS_entry( $GBS_ROOT_PATH, $NEW_SUBSYS) if ($NEW_SUBSYS ne '');
push @lines, SWC_entry( $GBS_ROOT_PATH, $NEW_SUBSYS, $NEW_COMPONENT) if ($NEW_COMPONENT ne '');
push @lines, SHELL_cd( $new_cwd);
}

push @lines, GBSCMD_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub ask_subsys()
{
my $subsys;




my @all_subsys = GBSGLO_subsystems();
if (scalar @all_subsys == 0)
{
ENV_error_exit( "No SubSystem(s) defined - use 'sws --new' to define a SubSystem");
} elsif (scalar @all_subsys == 1)
{
$subsys = $all_subsys[0];
ENV_say( 1, "Single SubSystem '$subsys' taken by default");
} else
{



my @subsys_refs = ( [ '', '-' ] );
my $longest_name = 0;

map { $longest_name = length $_ if (length $_ > $longest_name) } @all_subsys;
foreach my $subsys (@all_subsys)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
push @subsys_refs, [ sprintf( "%-*s   [full GBS]", $longest_name, $subsys), $subsys ];
} else
{
push @subsys_refs, [ $subsys, $subsys ];
}
}
my $default_subsys = ENV_getenv( 'GBS_SUBSYS');
$subsys = ASK_value_from_menu( "Select SubSystem",
\$default_subsys, @subsys_refs);
}

return $subsys;
}


