#==============================================================
#
#   gbsswt.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
print "*GBSSWT*\n" if ($ENV{GBSDEBUG_FILE});
use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbscmd;
use run;
use validate;
use swt;
use swglo;




sub ask_target();




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');	# commalist
my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
@GBS_TARGETS = sort( @GBS_TARGETS);

my $OLD_TARGET = ENV_getenv( 'GBS_TARGET');
my $NEW_TARGET = ENV_getenv( 'GBS_TARGET');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'target',	     'sso',  '', "Target: '' == ASK, '-' == none, '.' == current" ],
[ 'new',	'create_new_target', 'bso',   0, "Create a new Target" ],
[ 'remove', 'remove_target',     'bso',   0, "Remove a Target from the Targets List" ],
);
my @genconflicts = (
[ [ new => 1 ], '=', [ remove => 1] ],
);
GENOPT_set_optdefs( [ 'gbsswt', 'swt' ], \@genopts,
'Set and/or Create new Current Target',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $TARGET = GENOPT_get( 'target');
my $NEW = GENOPT_get( 'create_new_target');
my $REMOVE = GENOPT_get( 'remove_target');




VALIDATE_root();




if ($NEW)
{
$TARGET = RUN_gbsnew( 'new_target', $TARGET);
@GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
if ($TARGET eq '')
{
$TARGET = '.';
$CANCELLED = 1;
} else
{
ENV_say( 1, "Current Target is $TARGET");
}
} elsif ($REMOVE)
{
my $removed_target = RUN_gbsnew( 'rem_target', $TARGET);
@GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
if ($OLD_TARGET eq $removed_target)
{
ENV_say( 1, "Current Target removed");
$TARGET = '';	# No current
} else
{
ENV_say( 1, "Current Target is $OLD_TARGET");
$TARGET = '.';	# Keep current
}
}

if (!$CANCELLED)
{



if ($TARGET eq '')
{
$TARGET = ask_target();
}
($NEW_TARGET) = SWGLO_validate( $TARGET);




if ($NEW_TARGET ne ENV_getenv( 'GBS_TARGET'))
{
my $must_write = $NEW_TARGET ne '' || $TARGET eq '-';
SWT_set( $GBS_ROOT_PATH, $NEW_TARGET, $must_write);
}
ENV_warn( "No current Target") if ($NEW_TARGET eq '' && $TARGET ne '-');










{
my @lines;
if (ENV_get_changed_envs())
{
push @lines, GBSENV_changed_setenv_commands( 0);
}
push @lines, GBSCMD_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub ask_target()
{
my $target = '';

if (scalar @GBS_TARGETS == 0)
{
ENV_error_exit( "No Target(s) defined - use 'swt --new' to define Targets");
} elsif (scalar @GBS_TARGETS == 1)
{
$target = $GBS_TARGETS[0];
ENV_say( 1, "Single Target '$target' taken by default");
} else
{
my $default_target = ENV_getenv( 'GBS_TARGET');
$target = ASK_value_from_menu( "Select Target", \$default_target, @GBS_TARGETS);
}

return $target;
}


