#===============================================================
#
#   gbsxref_tkx.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use integer;

use Tkx;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::slurp;
use glo::args;
use glo::tkxglo;
use gbsenv;
use gbsglo;
use scope;
use validate;
use steps;




sub init();
sub exec_xref();
sub select_xref_none();
sub select_xref_scope();
sub select_xref_make();
sub create_tables($);
sub add_line($);
sub set_display_mode();
sub display_initial();
sub display($);
sub show($@);
sub select_center();
sub select_left();
sub select_right();
sub get_scope_lines();
sub get_make_lines();
sub select_target_dirs($$@);
sub about();
sub help();
sub message($);
sub warning($);
sub error($);
sub startup();
sub NONE;




my $PREFIX = 'GBSXREF_TKX';

my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();

my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');




my @SUBSYSTEMS_ORDER;
my $SUBSYS = $GBS_SUBSYS;

my $MESSAGE = '';

my $CURRENT_XREF_FUNC = \&select_xref_none;

my @SUBSYSTEMS_COMBO;
my @TARGETS_COMBO;
my @DISPLAY_MODES_COMBO = qw( Full Name.Type Name );
my $CURRENT_DISPLAY_MODE = 'Full';

my @ALL;

my %ALL;

my @CUR;
my %USES;

my %USED;


my @LEFT;
my @CENTER;
my %CENTER;

my @RIGHT;

my $CURRENT_CENTER;

my $MW;		    # Main Window
my $LLB;	    # Left List Box
my $RLB;;	    # Right List Box
my $CLB;;	    # Center List Box
my $MTXT;	    # Message Text







$| = 1;
my $RC = 0;
GBSENV_init();




ARGS_get( 0, 'perl gbsxref_tkx.pl', '- Must be called via perl gbsxref.pl');




VALIDATE_root();

@SUBSYSTEMS_ORDER = STEPS_get_gbs_subsystems();

@SUBSYSTEMS_COMBO = ( NONE, @SUBSYSTEMS_ORDER);
$SUBSYS = NONE if (! grep $SUBSYS eq $_, @SUBSYSTEMS_COMBO);
@TARGETS_COMBO = ( NONE, @GBS_ALL_TARGETS );
$GBS_TARGET = NONE if (! grep $GBS_TARGET eq $_, @TARGETS_COMBO);




{








$MW = Tkx::widget->new( ".");
$MW->g_wm_geometry( "800x500");	# Width, Height
$MW->g_wm_minsize( 500, 200);	# Width, Height
$MW->g_wm_title( "$PREFIX: Root/System: $GBS_ROOT_PATH");
$MW->g_wm_iconphoto( Tkx::image_create_photo( -file => "$ENV{GBS_SCRIPTS_PATH}/doc/images/gbs.gif"));




TKXGLO_add_menus( $MW, ( [ 'File', [ 'Exit', sub {$MW->g_destroy;} ] ],
[ 'Help', [ 'About', \&about],
[ undef, undef ],
[ 'Help', \&help ] ]
));




my $nb = $MW->new_ttk__notebook;
$nb->g_pack( -side => 'top', -fill => 'x');




my $np = $nb->new_ttk__frame( -relief => 'sunken');
$nb->add( $np, -text => "None");
$np->g_bind( '<Visibility>',  \&select_xref_none);




my $mp = $nb->new_ttk__frame( -relief => 'sunken');
$nb->add( $mp, -text => "Make");
$mp->g_bind( '<Visibility>',  \&select_xref_make);
$mp->new_ttk__label( -text => 'SubSystem: ')->g_pack( -side => 'left');
my $mps = $mp->new_ttk__combobox( -values =>  \@SUBSYSTEMS_COMBO,
-textvariable => \$SUBSYS,
-state => 'readonly');
$mps->g_bind( "<<ComboboxSelected>>", \&exec_xref);
$mps->g_pack( -side => 'left');

$mp->new_ttk__label( -text => 'Target: ')->g_pack( -side => 'left');
my $mpt = $mp->new_ttk__combobox( -values => [ @TARGETS_COMBO ],
-textvariable => \$GBS_TARGET,
-state => 'readonly');
$mpt->g_bind( "<<ComboboxSelected>>", \&exec_xref);
$mpt->g_pack( -side => 'left');

$mp->new_ttk__label( -text => 'Display Mode: ')->g_pack( -side => 'left');
my $mpd = $mp->new_ttk__combobox( -values => [@DISPLAY_MODES_COMBO ],
-textvariable => \$CURRENT_DISPLAY_MODE,
-state => 'readonly');
$mpd->g_bind( "<<ComboboxSelected>>", \&set_display_mode);
$mpd->g_pack( -side => 'left');




my $sp = $nb->new_ttk__frame( -relief => 'sunken');
$nb->add( $sp, -text => "Scope");
$sp->g_bind( '<Visibility>',  \&select_xref_scope);
$sp->new_ttk__label( -text => 'SubSystem: ')->g_pack( -side => 'left');
my $sps = $sp->new_ttk__combobox( -values =>  \@SUBSYSTEMS_COMBO,
-textvariable => \$SUBSYS,
-state => 'readonly');
$sps->g_bind( "<<ComboboxSelected>>", \&exec_xref);
$sps->g_pack( -side => 'left');

$sp->new_ttk__label( -text => 'Target: ')->g_pack( -side => 'left');
my $spt = $sp->new_ttk__combobox( -values => [ @TARGETS_COMBO ],
-textvariable => \$GBS_TARGET,
-state => 'readonly');
$spt->g_bind( "<<ComboboxSelected>>", \&exec_xref);
$spt->g_pack( -side => 'left');





my $lf = $MW->new_ttk__frame( -relief => 'groove', -borderwidth => 3);
$lf->g_pack( -side => 'top', -fill => 'both' , -expand => 1);

(my $llbf, $LLB) = TKXGLO_new_listbox( $lf, \&select_left);
$llbf->g_pack( -side => 'left', -fill => 'both' , -expand => 1);
$LLB->configure(-foreground => 'darkblue');

(my $clbf, $CLB) = TKXGLO_new_listbox( $lf, \&select_center);
$clbf->g_pack( -side => 'left', -fill => 'both' , -expand => 1);

(my $rlbf, $RLB) = TKXGLO_new_listbox( $lf, \&select_right);
$rlbf->g_pack( -side => 'right', -fill => 'both' , -expand => 1);
$RLB->configure(-foreground => 'darkblue');




my $mlf = $MW->new_ttk__frame( -relief => 'sunken',
-borderwidth => 3);
$mlf->g_pack( -side => 'bottom', -fill => 'x');

$MTXT = $mlf->new_ttk__label( -textvariable => \$MESSAGE,
-justify => 'left');
$MTXT->g_pack( -side => 'left', -fill => 'x', -anchor => 'w');
$MTXT->g_bind( '<Visibility>', [ \&startup ] );

$mlf->new_ttk__sizegrip->g_pack( -side => 'right');
}





Tkx::MainLoop();

ENV_exit( $RC);




sub END
{
ENV_print_end_msg(1);
}




sub init()
{
my $ok = 1;

return $ok;
}




sub exec_xref()
{
&$CURRENT_XREF_FUNC();
}




sub select_xref_none()
{
$CURRENT_XREF_FUNC = \&select_xref_none;
message( 'None');

$LLB->delete( 0, 'end');
$CLB->delete( 0, 'end');
$RLB->delete( 0, 'end');
message( '');
}




sub select_xref_make()
{
$CURRENT_XREF_FUNC = \&select_xref_make;
message( 'Make');

$LLB->delete( 0, 'end');
$CLB->delete( 0, 'end');
$RLB->delete( 0, 'end');

if ($SUBSYS eq NONE)
{
warning( 'No current SubSystem');
} elsif ($GBS_TARGET eq NONE)
{
warning( 'No current Target');
} elsif ( ! -d "$GBS_ROOT_PATH/dev/$SUBSYS/gen/$GBS_TARGET")
{
warning( 'This SubSystem does not generate for this Target');
} else
{
my $saved_cursor = $MW->cget( -cursor);
$MW->g_grab();
$MW->configure( -cursor => 'watch');
message( 'Reading...');
my @line_refs = get_make_lines();

message( 'Processing...');
create_tables( \@line_refs);
set_display_mode();

message( 'Displaying...');
display_initial();

$MW->configure( -cursor => $saved_cursor);
$MW->g_grab_release();
message( '');
}
}



sub select_xref_scope()
{
$CURRENT_XREF_FUNC = \&select_xref_scope;
message( 'Scope');

$LLB->delete( 0, 'end');
$CLB->delete( 0, 'end');
$RLB->delete( 0, 'end');

if ($SUBSYS eq NONE)
{
warning( 'No current SubSystem');
} elsif ($GBS_TARGET eq NONE)
{
warning( 'No current Target');
} elsif ( ! -d "$GBS_ROOT_PATH/dev/$SUBSYS/gen/$GBS_TARGET")
{
warning( 'This SubSystem does not generate for this Target');
} else
{
my $saved_cursor = $MW->cget( -cursor);
$MW->configure( -cursor => 'watch');
ENV_setenv( GBS_TARGET => $GBS_TARGET);
message( 'Reading...');
my @line_refs = get_scope_lines();

message( 'Processing...');
create_tables( \@line_refs);
$CURRENT_DISPLAY_MODE = 'Full';
set_display_mode();

message( 'Displaying...');
display_initial();
display( $ALL{$GBS_COMPONENT}) if (defined $ALL{$GBS_COMPONENT});

$MW->configure( -cursor => $saved_cursor);
message( '');
}
}





sub create_tables($)
{
my ($line_refs_ref
) =@_;

@ALL = ();
%ALL = ();
%USES = ();
%USED = ();
$CURRENT_CENTER = undef;

add_line( '<NO-REF>');

my $def_index;
foreach my $line_ref (@{$line_refs_ref})
{
my ($fc, $line) = @{$line_ref};

my $index;
if ($fc eq '-')
{
$index = add_line( $line);
push @{$USES{$def_index}}, $index if (! grep $index eq $_, @{$USES{$def_index}});
} elsif ($fc eq '+')
{
$index = add_line( $line);
$def_index = $index;
} else
{
$index = add_line( $line);
$index = undef;
}
}





while (my ($key, $valref) = each %USES)
{
foreach my $index (@{$valref})
{
push @{$USED{$index}}, $key;
}
}




my @norefs;
while (my ($key, $valref) = each %USES)
{
push @norefs, $key if (scalar @{$valref} == 0)
}
$USED{0} = [ @norefs ];

@norefs = ();
while (my ($key, $valref) = each %USED)
{
push @norefs, $key if (scalar @{$valref} == 0)
}
$USES{0} = [ @norefs ];
}




sub add_line($)
{
my ($name
) = @_;
my $index;

$index = $ALL{$name};
if (!defined $index)
{
push @ALL, $name;
$index = $#ALL;
$ALL{$name} = $index;
$USES{$index} = [];
$USED{$index} = [];
}

return $index;
}




sub set_display_mode()
{
if ($CURRENT_DISPLAY_MODE eq 'Full')
{
@CUR = @ALL;
} elsif ($CURRENT_DISPLAY_MODE eq 'Name.Type')
{
@CUR = ();
foreach my $line (@ALL)
{
my $cur = $line;
my $loc = rindex( $cur, '/');
$cur = substr( $cur, $loc + 1) if ($loc >= 0);
push @CUR, $cur;
}
} else #($CURRENT_DISPLAY_MODE eq 'Name')
{
@CUR = ();
foreach my $line (@ALL)
{
my $cur = $line;
my $loc = rindex( $cur, '/');
$cur = substr( $cur, $loc + 1) if ($loc >= 0);
$loc = rindex( $cur, '.');
$cur = substr( $cur, 0, $loc) if ($loc > 0);
push @CUR, $cur;
}
}
display_initial();
display( $CURRENT_CENTER) if (defined $CURRENT_CENTER);
}




sub display_initial()
{
$LLB->delete( 0, 'end');
$CLB->delete( 0, 'end');
$RLB->delete( 0, 'end');

@CENTER = show( $CLB, (0..$#ALL));
my $i = 0;
foreach my $index (@CENTER)
{
$CENTER{$index} = $i++;
}
}




sub display($)
{
my ($index
) = @_;

$CURRENT_CENTER = $index;

@LEFT = show( $LLB, @{$USED{$index}});

my $cindex = $CENTER{$index};
$CLB->see( $cindex);
$CLB->selection_set( $cindex);


@RIGHT = show( $RLB, @{$USES{$index}});





}




sub show($@)
{
my ($lb,
@indices
) =@_;


@indices = sort { $CUR[$a] cmp $CUR[$b] } @indices;
my @list = map { $CUR[$_] } @indices;
$lb->delete( 0, 'end');
$lb->insert( 'end', @list);

return @indices;
}




sub select_center()
{
my ($selection) = $CLB->curselection();
if ($selection ne '')
{
my $index = $CENTER[$selection];
my $file = $ALL[$index];

message( $file);
display( $index);
} else
{
Tkx::bell();
}
}




sub select_left()
{
my ($selection) = $LLB->curselection();
if ($selection ne '')
{
my $index = $LEFT[$selection];
my $file = $ALL[$index];

message( $file);
display( $index);
} else
{
Tkx::bell();
}
}




sub select_right()
{
my ($selection) = $RLB->curselection();
if ($selection ne '')
{
my $index = $RIGHT[$selection];
my $file = $ALL[$index];

message( $file);
display( $index);
} else
{
Tkx::bell();
}
}




sub get_scope_lines()
{
my @line_refs = ();

my $comproot = "$GBS_ROOT_PATH/dev/$SUBSYS/comp";
foreach my $component (GBSGLO_components( $SUBSYS))
{
push @line_refs, [ '+', $component ];
foreach my $comp (SCOPE_get_components( $component))
{
push @line_refs, [ '-', $comp ];
}
}
return @line_refs;
}




sub get_make_lines()
{
my @line_refs;

my $subsysroot = "$GBS_ROOT_PATH/dev/$SUBSYS";
my $devroot_ = "$GBS_ROOT_PATH/dev/";
my $extroot_ = "$GBS_ROOT_PATH/ext/";
my $resroot_ = "$GBS_ROOT_PATH/res/";
my $base_l = length $devroot_;




my $froms = "$subsysroot/comp/*/bld/$GBS_TARGET/*.* $subsysroot/comp/*/inc/*.* $subsysroot/comp/*/loc/*.*";
foreach my $file (glob $froms)
{
push @line_refs, [ '+', 'DEV:' . substr( $file, $base_l) ];
}





my @other_targets = grep $GBS_TARGET ne $_, @GBS_ALL_TARGETS;
foreach my $subsys (@SUBSYSTEMS_ORDER)
{
my $res_subsysroot = "$GBS_ROOT_PATH/res/$subsys";
if ( -d $res_subsysroot)
{
my @file_dir_refs = SLURP_dir_tree_path( $res_subsysroot, 0);
my @file_dirs = select_target_dirs( $GBS_TARGET, \@other_targets, @file_dir_refs);
foreach my $file (@file_dirs)
{
push @line_refs, [ '+', 'RES:' . substr( $file, $base_l) ];
}
}
last if ($subsys eq $SUBSYS);	    # stop after current (or before?????)
}




my @ref_files = glob "$subsysroot/comp/*/bld/$GBS_TARGET.ref";
if (@ref_files)
{
foreach my $ref_file (@ref_files)
{
foreach my $line (SLURP_file( $ref_file))
{
my ($level, $file) = split( / /, $line, 2);
my $fc = ($level eq '0') ? '+' : '-';
if (substr($file, 0, 1) eq '#')
{
my ($unk_file) = split ' ', substr( $file, 2);
$file = "UNK:$unk_file";
} else
{
my $base = substr( $file, 0, $base_l);
if ($base eq $devroot_)
{
$file = 'DEV:' . substr( $file, $base_l);
} elsif ($base eq $resroot_)
{
$file = 'RES:' . substr( $file, $base_l);
} elsif ($base eq $extroot_)
{
$file = 'EXT:' . substr( $file, $base_l);
} else
{

}
}
push @line_refs, [ $fc, $file ];
}
}
} else
{
@line_refs = ();
error( "No Makefiles found for target '$GBS_TARGET'");
sleep( 3);
}

return @line_refs;
}




sub select_target_dirs($$@)
{
my ($target,
$other_targets_ref,
@file_dir_refs
) = @_;
my @selection;

my @other_targets = @{$other_targets_ref};
foreach my $ref (@file_dir_refs)
{
my $file_dir = $ref->[1];
if ($file_dir =~ m!/res/.*/$target(/|$)!)
{
push @selection, $file_dir;
} else
{
push @selection, $file_dir if (!grep( $file_dir =~ m!/res/.*/$_(/|$)!, @other_targets));
}
}


return @selection;
}




sub about()
{
TKXGLO_aboutbox( "$PREFIX - About", <<ENDHERE

This program is a part of the GBS (Generic Build Support)
package created by Randy Marques of Randy Marques Consultancy.
For information: http://www.genericbuildsupport.com

Copyright  2010-2020 - Randy Marques - All rights reserved

ENDHERE
);
}




sub help()
{
TKXGLO_messagebox_ok( "$PREFIX - Help", 'question', <<ENDHERE
For CrossReference of Files select TAB 'Make'
For CrossReference of Components: select TAB 'Scope'

Click on an item in the center column:
- On the left the items referring TO the item in the center are displayed
- On the right the items referred BY the item in the center are displayed

ENDHERE
);
}




sub message($)
{
my ($text
) = @_;

$MTXT->configure( -foreground => 'black');
$MESSAGE = $text;
Tkx::update();
}




sub warning($)
{
my ($text
) = @_;

$MTXT->configure( -foreground => 'OrangeRed');
$MESSAGE = $text;
Tkx::update();
Tkx::bell();
}




sub error($)
{
my ($text
) = @_;

$MTXT->configure( -foreground => 'red');
$MESSAGE = $text;
Tkx::update();
}




sub startup()
{
message( "Starting...");
my $ok = init();
message( '') if ($ok);
$MTXT->g_bind( '<Visibility>', '' );
}




sub NONE
{
return '<None>';
}

