#=================================================
#
#   env.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Rules:
#	Do not use $&, $` or $'
#
#===============================================================

package glo::env;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
ENV_set_application
ENV_get_application_name
ENV_get_application_root_path
ENV_get_application_home_path
ENV_get_application_data_path
ENV_get_perl_cmd
ENV_get_main_name
ENV_is_development
ENV_is_lunix
ENV_is_linux
ENV_is_win32
ENV_is_interactive
ENV_enable_beeps
ENV_beeps_are_enabled
ENV_beep
ENV_reap_beeps
ENV_say
ENV_say_stderr
ENV_print
ENV_get_prefix
ENV_info
ENV_warn
ENV_error
ENV_warn_exit
ENV_error_exit
ENV_fatal
ENV_exit
ENV_stackdump
ENV_debug
ENV_is_debug
ENV_print_end_msg
ENV_printable
ENV_clone
ENV_dump
ENV_eval_empty
ENV_eval_undef
ENV_undef2empty
ENV_getenv
ENV_getenv_perl_path
ENV_setenv
ENV_get_changed_envs
ENV_detab
ENV_commandfile_spec
ENV_commandfile_ext
ENV_is_commandfile
ENV_chmod
ENV_get_hostname
ENV_uname
ENV_unique_id
ENV_get_userid
ENV_glob
ENV_wild_2_re
ENV_is_wildcard
ENV_wildcard
ENV_wildcards
ENV_not_wildcards
ENV_get_term_size
ENV_get_term_size_default
ENV_is_abs_path
ENV_is_abs_path_perl
ENV_is_option
ENV_is_in_path
ENV_parent_path
ENV_parent_dir
ENV_set_tmpdir_path
ENV_get_tmpdir_path
ENV_get_os_tmpdir_path
ENV_get_tmp_spec
ENV_os_path
ENV_perl_paths
ENV_perl_path_nospace
ENV_perl_path_space
ENV_canonicalize_path
ENV_expand_envs
ENV_expand_arg
ENV_delink_spec
ENV_rel_spec
ENV_abs_path
ENV_rel_path
ENV_shortest_path
ENV_short_path_name
ENV_split_spec_dnt
ENV_split_spec_df
ENV_split_spec_d
ENV_split_spec_f
ENV_split_spec_nt
ENV_split_spec_n
ENV_split_spec_t
ENV_split_path
ENV_join_path
ENV_chdir
ENV_cwd
ENV_file_is_newer
ENV_file_is_same
ENV_mkdir
ENV_rmdir
ENV_rmdir_f
ENV_readdir
ENV_rename
ENV_unlink
ENV_copy_file
ENV_touch_file
ENV_hide_file
ENV_max_command_line
ENV_command_split
ENV_new_command_ref
ENV_get_command_ref_data
ENV_command_ref
ENV_command_line
ENV_find_in_perl_inc
ENV_system
ENV_run3
ENV_exec
ENV_rc_status
ENV_detab
ENV_split_quoted_space
ENV_join_quoted_space
ENV_enquote
ENV_dequote
ENV_escape_os_text
ENV_redirect_stdouterr
ENV_restore_stdouterr
ENV_get_desktop_path
ENV_get_startmenu_path
ENV_get_programs_path
ENV_get_user_path
ENV_is_administrator
);

use strict;
use integer;

use File::Glob ':glob';      # Override glob built-in

BEGIN
{
if ($^O eq 'MSWin32')
{
require glo::os_win;
import  glo::os_win;
} else
{
require glo::os_linux;
import  glo::os_linux;
}
}




sub ENV_set_application($$$$$);
sub ENV_get_application_name();
sub ENV_get_application_root_path();
sub ENV_get_application_data_path();
sub ENV_get_perl_cmd();
sub ENV_get_main_name();
sub ENV_is_development();
sub ENV_is_lunix();
sub ENV_is_linux();
sub ENV_is_win32();
sub ENV_is_interactive();
sub ENV_enable_beeps($);
sub ENV_beeps_are_enabled();
sub ENV_beep();
sub ENV_reap_beeps();
sub ENV_say($@);
sub ENV_say_stderr($@);
sub ENV_print($@);
sub ENV_get_prefix($);
sub ENV_info(@);
sub ENV_warn(@);
sub ENV_error(@);
sub ENV_warn_exit(@);
sub ENV_error_exit(@);
sub ENV_fatal(@);
sub ENV_exit(;$);
sub ENV_stackdump();
sub ENV_debug($@);
sub ENV_is_debug(;$);
sub ENV_print_end_msg($);
sub ENV_printable($);
sub ENV_clone($);
sub ENV_dump($$;$$);
sub ENV_eval_empty($);
sub ENV_eval_undef($);
sub ENV_undef2empty(@);
sub ENV_getenv($);
sub ENV_getenv_perl_path($);
sub ENV_setenv($$);
sub ENV_get_changed_envs();
sub ENV_commandfile_spec($);
sub ENV_commandfile_ext();
sub ENV_is_commandfile($);
sub ENV_chmod($@);
sub ENV_get_hostname();
sub ENV_uname();
sub ENV_unique_id();
sub ENV_get_userid();
sub ENV_glob(@);
sub ENV_wild_2_re($);
sub ENV_is_wildcard($);
sub ENV_wildcard($$);
sub ENV_wildcards($$$);
sub ENV_not_wildcards($$);
sub ENV_get_term_size();
sub ENV_get_term_size_default();
sub ENV_is_abs_path($);
sub ENV_is_abs_path_perl($);
sub ENV_is_option($);
sub ENV_is_in_path($$);
sub ENV_parent_path($);
sub ENV_parent_dir($$);
sub ENV_get_application_home_path();
sub ENV_set_tmpdir_path($);
sub ENV_get_tmpdir_path();
sub ENV_get_os_tmpdir_path();
sub ENV_get_tmp_spec($);
sub ENV_os_path($);
sub ENV_perl_paths(@);
sub ENV_perl_path_nospace(@);
sub ENV_perl_path_space(@);
sub ENV_canonicalize_path($);
sub ENV_expand_envs($);
sub ENV_expand_arg($);
sub ENV_delink_spec($);
sub ENV_rel_spec($$);
sub ENV_abs_path($$);
sub ENV_rel_path($$);
sub ENV_shortest_path($$);
sub ENV_short_path_name(@);
sub ENV_split_spec_dnt($);
sub ENV_split_spec_df($);
sub ENV_split_spec_d($);
sub ENV_split_spec_f($);
sub ENV_split_spec_nt($);
sub ENV_split_spec_n($);
sub ENV_split_spec_t($);
sub ENV_split_path($);
sub ENV_join_path($);
sub ENV_chdir($);
sub ENV_cwd();
sub ENV_file_is_newer($$);
sub ENV_file_is_same($$);
sub ENV_mkdir($;$);
sub ENV_rmdir($);
sub ENV_rmdir_f($);
sub ENV_readdir($);
sub ENV_rename($$);
sub ENV_unlink(@);
sub ENV_copy_file($$$);
sub ENV_touch_file($);
sub ENV_hide_file($$);
sub ENV_max_command_line();
sub ENV_command_split($);
sub ENV_new_command_ref($$$$);
sub ENV_get_command_ref_data($);
sub ENV_command_ref($$);
sub ENV_command_line($);
sub ENV_find_in_perl_inc($$);
sub ENV_system($$$);
sub ENV_run3($$$$$);
sub ENV_exec($$);
sub ENV_rc_status($$);
sub ENV_detab($);
sub ENV_split_quoted_space($);
sub ENV_join_quoted_space(@);
sub ENV_enquote(@);
sub ENV_dequote(@);
sub ENV_escape_os_text($);
sub ENV_redirect_stdouterr($);
sub ENV_restore_stdouterr();
sub ENV_get_desktop_path($);
sub ENV_get_startmenu_path($);
sub ENV_get_programs_path($);
sub ENV_get_user_path($);
sub ENV_is_administrator();

sub deref_path($$;$);
sub deref_value($);
sub exec_beep();
sub print_info($@);
sub get_debug_name($);
sub dump_item($$$$$);
sub de_ref($$);
sub slurp_file($);




(my $MAIN_NAME = uc $0 ) =~ s/\.PL$//;	    # UC and remove .pl
$MAIN_NAME =~ s!.*[/\\]!!;		    # remove leading device/directories

my $MAIN_NAME_ = ' ' x ((length $MAIN_NAME) + 1);

my $PREFIX = $MAIN_NAME . '/ENV';

my $IS_WIN32 = ($^O eq 'MSWin32');
my $IS_LINUX = ($^O eq 'linux');
my $IS_LUNIX = !$IS_WIN32;

my $IS_INTERACTIVE = (-t STDIN && -t STDOUT);	    # connected to keyboard and window

my $XTERM_DEFAULT_WIDTH = 100;
my $XTERM_DEFAULT_HEIGHT = 60;

my $PATH_SEPARATOR = ($IS_WIN32) ? ';' : ':';

my $IS_DEVELOPMENT;	    # Do not initialize!

my $APPLICATION_NAME;	    # Do not initialize!
my $APPLICATION_ROOT_PATH;  # Do not initialize!
my $APPLICATION_HOME_PATH;  # Do not initialize!
my $APPLICATION_DATA_PATH;  # Do not initialize!
my $PERL_CMD = 'perl -w';

my $APPLICATION_DEBUG_PREFIX;			    # e.g.: GBSDEBUG_
my $DEBUG_ALL;					    # bool
my %DEBUG_NAMES;

my $DEBUG_ACTIVE = 0;

my $OS_TMPDIR_PATH;	    # Do not initialize!
my $TMPDIR_PATH;	    # Do not initialize!
my $LOG_PATH;		    # Do not initialize!

my $MAX_COMMAND_LINE;	    # Do not initialize!

my $BEEPS_ENABLED = 1;
my $NR_BEEPS = 0;
my $NR_WARNINGS = 0;
my $NR_ERRORS = 0;

my $UNIQUE_COUNT = 0;

my $RUNNING_IN_EXEC = 0;

my %CENVS;


my %QUOTED_PATHS;


my $SAVED_STDOUT;
my $SAVED_STDERR;







sub ENV_set_application($$$$$)
{
my ($app_name,		    # e.g.: GBS
$app_root_path,	    	    # must be canonicalised root_path!	    e.g.: GBS_SCRIPTS_PATH
$app_home_path,		    # must be verified existing directory   e.g.: GBS_HOME
$app_data_path,		    # must be verified existing directory   e.g.: GBS_BASE_PATH
$perl_cmd,		    # must exist			    e.g.: GBS_PERL_CMD
) = @_;

if (defined $app_name)
{
$app_name = deref_value( $app_name);
if (defined $app_name)
{
ENV_say( 1, "APPLICATION_NAME ($APPLICATION_NAME) changed to '$app_name'")
if (defined $APPLICATION_NAME && $app_name ne $APPLICATION_NAME);
$APPLICATION_NAME = $app_name;




my $app_debug_prefix = "${app_name}DEBUG_";
my @debug_env_names = grep /^$app_debug_prefix/, keys %ENV;
foreach my $env_name (grep /^$app_debug_prefix/, keys %ENV)
{
my $env_value = ENV_getenv( $env_name);
if ($env_value eq '1' || substr( $env_value, 0, 1) eq 'Y')
{
my $debug_name = substr( $env_name, length( $app_debug_prefix));
$DEBUG_NAMES{$debug_name} = 1;
ENV_say( 1, "DBG: $env_name=$env_value");
}
$DEBUG_ACTIVE = 1;
}
$DEBUG_ALL = (exists $DEBUG_NAMES{ALL}) ? 1 : 0;
}
}

if (defined $app_root_path)
{
$app_root_path = deref_path( $app_root_path, 1, "APPLICATION_ROOT_PATH");
if (defined $app_root_path)
{
ENV_say( 1, "APPLICATION_ROOT_PATH ($APPLICATION_ROOT_PATH) changed to '$app_root_path'")
if (defined $APPLICATION_ROOT_PATH && $app_root_path ne $APPLICATION_ROOT_PATH);
$APPLICATION_ROOT_PATH = $app_root_path;
}
}

if (defined $app_home_path)
{
$app_home_path = deref_path( $app_home_path, 1, "APPLICATION_HOME_PATH");
if (defined $app_home_path)
{
ENV_say( 1, "APPLICATION_HOME_PATH ($APPLICATION_HOME_PATH) changed to '$app_home_path'")
if (defined $APPLICATION_HOME_PATH && $app_home_path ne $APPLICATION_HOME_PATH);
$APPLICATION_HOME_PATH = $app_home_path;
}
}

if (defined $app_data_path)
{
$app_data_path = deref_path( $app_data_path, 1, "APPLICATION_DATA_PATH");
if (defined $app_data_path)
{
ENV_say( 1, "APPLICATION_DATA_PATH ($APPLICATION_DATA_PATH) changed to '$app_data_path'")
if (defined $APPLICATION_DATA_PATH && $app_data_path ne $APPLICATION_DATA_PATH);
$APPLICATION_DATA_PATH = $app_data_path;
}
}

if (defined $perl_cmd)
{
$perl_cmd = deref_value( $perl_cmd);
if (defined $perl_cmd)
{
$PERL_CMD = "$perl_cmd -w";
}
}
}




sub deref_path($$;$)
{
my ($path,		    # path or '<path_envvar>'
$must_exist,
$name,		    # optional. for error-message
) = @_;
my $new_path;	    # path or undef

if (substr( $path, 0, 1) eq '<' && substr( $path, -1, 1) eq '>')
{
my $envvar = substr( $path, 1, -1);
$new_path = ENV_eval_undef( $ENV{ $envvar});
if (defined $new_path)
{
$new_path = ENV_getenv_perl_path( $envvar);
$new_path = ENV_perl_path_nospace( $new_path)
if ($IS_WIN32);
ENV_fatal( "$name: $envvar => $new_path does not exist")
if ($must_exist && ! -d $new_path);
}
} else
{
$new_path = $path;
$new_path = ENV_perl_path_nospace( $path)
if ($IS_WIN32);
ENV_fatal( "$name: $path($new_path) does not exist")
if ($must_exist && ! -d $new_path);
}

return $new_path;
}




sub deref_value($)
{
my ($value,		    # value or '<value_envvar>'
) = @_;
my $new_value;	    # value or undef

if (substr( $value, 0, 1) eq '<' && substr( $value, -1, 1) eq '>')
{
$new_value = ENV_eval_undef( $ENV{ substr( $value, 1, -1)});
} else
{
$new_value = $value;
}

return $new_value;
}





sub ENV_get_application_name()
{
return $APPLICATION_NAME;
}





sub ENV_get_application_root_path()
{
return $APPLICATION_ROOT_PATH;
}





sub ENV_get_application_home_path()
{
if (!defined $APPLICATION_HOME_PATH)
{



$APPLICATION_HOME_PATH = ENV_get_user_path( 0);
}

return $APPLICATION_HOME_PATH;
}





sub ENV_get_application_data_path()
{
return $APPLICATION_DATA_PATH;
}





sub ENV_get_perl_cmd()
{
return $PERL_CMD;
}




sub ENV_get_main_name()
{
return $MAIN_NAME;
}




sub ENV_is_development()
{
$IS_DEVELOPMENT = -d "$APPLICATION_ROOT_PATH/Tools"
if (!defined $IS_DEVELOPMENT);
return $IS_DEVELOPMENT;
}




sub ENV_is_lunix()
{
return $IS_LUNIX;
}




sub ENV_is_linux()
{
return $IS_LINUX;
}




sub ENV_is_win32()
{
return $IS_WIN32;
}




sub ENV_is_interactive()
{
return $IS_INTERACTIVE;
}




sub ENV_enable_beeps($)
{
$BEEPS_ENABLED = $_[0];
}




sub ENV_beeps_are_enabled()
{
return $BEEPS_ENABLED;
}




sub ENV_beep()
{
$NR_BEEPS++;
}




sub ENV_reap_beeps()
{
exec_beep() if ($NR_BEEPS != 0);
$NR_BEEPS = 0;
}




sub exec_beep()
{
if ($BEEPS_ENABLED && $IS_INTERACTIVE)
{

OS_beep();
}
}




sub ENV_say($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;



ENV_fatal( "Invalid indent ($indent)")
if ($indent !~ /^-?[012]$/);			    # TBR

my @lines = map { (ref $_) ? @{$_} : $_ } @lines_and_or_refs;
@lines = ('')
if (@lines == 0);

if ($indent == 0)
{
map { CORE::say( $_) } @lines;
} else
{
my $level = 1;
if ($indent < 0)
{
$level = 2;
$indent *= -1;
}
my ($prefix, $prefix_) = ENV_get_prefix( $level);
if ($indent == 1)
{
my $first_line = shift @lines;
CORE::say( "$prefix: $first_line");
}
map { CORE::say( "$prefix_ $_") } @lines;
}
}






sub ENV_say_stderr($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;



ENV_fatal( "Invalid indent ($indent)")
if ($indent !~ /^-?[012]$/);			    # TBR

my @lines = map { (ref $_) ? @{$_} : $_ } @lines_and_or_refs;
if ($indent == 0)
{
map { CORE::say( STDERR $_) } @lines;
} else
{
my $level = 1;
if ($indent < 0)
{
$level = 2;
$indent *= -1;
}
my ($prefix, $prefix_) = ENV_get_prefix( $level);
if ($indent == 1)
{
my $first_line = shift @lines;
CORE::say( STDERR "$prefix: $first_line");
}
map { CORE::say( STDERR "$prefix_ $_") } @lines;
}
}




sub ENV_print($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;



ENV_fatal( "Invalid indent ($indent)")
if ($indent !~ /^-?[012]$/);			    # TBR

my @lines = map { (ref $_) ? @{$_} : $_ } @lines_and_or_refs;

if ($indent == 0)
{
map { print( $_) } @lines;
} else
{
my $level = 1;
if ($indent < 0)
{
$level = 2;
$indent *= -1;
}
my ($prefix, $prefix_) = ENV_get_prefix( $level);
if ($indent == 1)
{
my $first_line = shift @lines;
print( "$prefix: $first_line");
}
map { print( "$prefix_ $_") } @lines;
}
}





sub ENV_get_prefix($)
{
my ($top_level) = @_;	    # may not be 0
my ($prefix, $prefix_);

if ($DEBUG_ACTIVE)
{
my $prev_package = caller(0);	# Caller of ENV_print or ENV_say
my @packages;
my $at_end = 0;
for (my $i = 1; !$at_end; $i++)
{
my $this_package = caller( $i);
if (defined $this_package)
{
if ($this_package eq 'main')
{
unshift @packages, $MAIN_NAME;
$at_end = 1;
} else
{

if ($this_package ne $prev_package)
{
unshift @packages, $this_package;
$prev_package = $this_package;
}
}
} else
{
unshift @packages, "$MAIN_NAME---";	# BEGIN- or END-block
$at_end = 1;
}
}

for (my $i=1; $i < $top_level; $i++)
{
pop @packages;
}

map { $_ =~ s/.*::// } @packages;
$prefix = uc join( '/', @packages);

$prefix_ = ' ' x ((length $prefix) + 1);

} else
{
($prefix, $prefix_) = ($MAIN_NAME, $MAIN_NAME_);
}

return (wantarray) ? ($prefix, $prefix_) : $prefix;
}








sub ENV_info(@)
{


if (@_)
{
my @lines = print_info( '*INFO*', @_);
print( @lines);
}
}








sub ENV_warn(@)
{


$NR_WARNINGS++;
if (@_)
{
my @lines = print_info( '*WARNING*', @_);
print( @lines);
}
$NR_BEEPS++;
}








sub ENV_error(@)
{


$NR_ERRORS++;
if (@_)
{
my @lines = print_info( '** ERROR **', @_);
print( @lines);
}
$NR_BEEPS++;
}








sub ENV_warn_exit(@)
{


$NR_WARNINGS++;
if (@_)
{
my @lines = print_info( '*WARNING*', @_);
print( @lines);
}
$NR_BEEPS++;
ENV_exit( 1);
}








sub ENV_error_exit(@)
{


$NR_ERRORS++;
if (@_)
{
my @lines = print_info( '** ERROR **', @_);
print( @lines);
}
$NR_BEEPS++;
ENV_exit( 2);
}








sub ENV_fatal(@)
{


$NR_ERRORS++;
if (@_)
{
my @lines = print_info( '**INTERNAL ERROR**', @_);
print( @lines);
}
$NR_BEEPS++;
ENV_stackdump();

exit 9;
}




sub print_info($@)
{
my ($severity,
@lines_or_refs,
) = @_;
my @lines;

my $info_ref;
if (ref $lines_or_refs[0])
{
$info_ref = shift @lines_or_refs;
}
my @texts = map { (ref $_) ? @{$_} : $_ } @lines_or_refs;

if ($info_ref)
{
my $prefix;
my $src;
my $file;
my $line_nr;
my $line;
my $line2;
($prefix, $src, $file, $line_nr, $line, $line2) = @{$info_ref}
if (defined $info_ref);
$prefix = ENV_get_prefix( 1)
if (!$prefix);

push @lines, "$prefix: $severity\n";
if ($src)
{
if (defined $file && $file ne $src)
{
my $file_part = (defined $line_nr) ? "$file ($line_nr)" : $file;
push @lines, "  File: $src\n";
push @lines, "  Incl: $file_part\n";
} else
{
my $src_part = (defined $line_nr) ? "$src ($line_nr)" : $src;
push @lines, "  File: $src_part\n";
}
}

if ($line)
{
if (defined $line_nr)
{
push @lines, sprintf( "%6d: %s\n", $line_nr, $line);
push @lines, sprintf( "%6s: %s\n", ' ',      $line2)
if (defined $line2);
} else
{
push @lines, "  Line: $line\n";
push @lines, "      : $line2\n"
if (defined $line2);
}
}
push @lines, map { "  - $_\n" } @texts;
} else
{
my ($prefix, $prefix_) = ENV_get_prefix( 1);
push @lines, "$prefix: $severity\n";
push @lines, map { "    $_\n" } @texts;
}

return @lines;
}




sub ENV_exit(;$)
{
my ($rc) = @_;

$rc = 0 if (!defined $rc);
$rc = 2 if ($rc < 2 && $NR_ERRORS > 0);
exit $rc;
}




sub ENV_stackdump()
{
my @call_info = caller(1);
print( "** at: @call_info[1..2]\n");
for (my $i = 2; @call_info; $i++)
{
@call_info = caller( $i);
if (@call_info)
{
print( "** called by: @call_info[1..3]\n");
}
}
}




sub ENV_debug($@)
{
my ($debug_names_ref,	# optional. Default is caller's package name
@lines,
) = @_;
my $debug = 0;

if ($DEBUG_ACTIVE)
{
my $debug_name = get_debug_name( $debug_names_ref);
if (defined $debug_name)
{
my $prefix = ENV_get_prefix( 1);
my $first_line = (ref $debug_names_ref) ? shift @lines : $debug_names_ref;

print( "$prefix: DBG: [$debug_name]: $first_line\n");
map { print( "  $_\n"); } @lines;
$debug = 1;
}
}

return $debug;
}




sub ENV_is_debug(;$)
{
my ($debug_names_ref,	# optional. Default is caller's package name
) = @_;


if ($DEBUG_ACTIVE)
{
return (defined get_debug_name( $debug_names_ref)) ? 1 : 0;
} else
{
return 0;
}
}




sub get_debug_name($)
{
my ($debug_names_ref) = @_;	    # if undef, take the caller's package name
my $debug_name;		    # undef if debug not active for this context

if ($DEBUG_ALL)
{
$debug_name = 'ALL';
} else
{
my @debug_names;
if (ref $debug_names_ref)
{
@debug_names = @{$debug_names_ref}
} else
{
my ($caller_package, undef) = caller( 1);
if ($caller_package eq 'main')
{
@debug_names = ($MAIN_NAME);
} else
{
my @caller_package = split( '::', $caller_package);
@debug_names = (uc $caller_package[-1]);
}
}
foreach my $this_debug_name (@debug_names)
{
if (exists $DEBUG_NAMES{$this_debug_name})
{
$debug_name = $this_debug_name;
last;
}
}
}


return $debug_name;
}




sub ENV_print_end_msg($)
{
my ($allways) = @_;		# Bool

if (!$RUNNING_IN_EXEC)
{
my $rc = $?;

$rc = 1 if ($rc == 0 && $NR_ERRORS > 0);
$allways = 1 if ($NR_WARNINGS > 0);
ENV_beep() if ($rc != 0);

if ($rc != 0 || $allways)
{
my $text = "$MAIN_NAME: ";

if ($rc == 0)
{
$text .= 'Done';
} else
{
$text .= "Failed ($rc)";
}
if ($NR_ERRORS > 0 || $NR_WARNINGS > 0)
{
$text .= ' with ';
my @texts;
push @texts, "$NR_ERRORS errors(s)" if ($NR_ERRORS > 0);
push @texts, "$NR_WARNINGS warning(s)" if ($NR_WARNINGS > 0);
$text .= join( ' and ', @texts);
}

ENV_reap_beeps();
ENV_say( 0, $text);
}
}
}




sub ENV_printable($)
{
my ($var,
) = @_;
my $text;

if (!defined $var)
{
$text = 'undef';
} else
{
my $reftype = ref $var;
if (!defined $reftype)
{
$text = "\"$var\"";
} elsif ($reftype eq 'ARRAY')
{
my @items = @{$var};
foreach my $item (@items)
{
if (!defined $item)
{
$item = 'undef';
} elsif (ref $item)
{
$item = 'REF ' . ref $item;
} else
{
$item = $var;
}
}
$text = '[' . join( ',', @items) . ']';
} elsif ($reftype eq 'HASH')
{
my @keys = keys %{$var};
$text = '{' . join( ',', @keys) . '}';
} else
{
$text = $reftype;
}
}
return $text;
}




sub ENV_clone($)
{
my ($this) = @_;

if (not ref $this)
{
$this;
} elsif (ref $this eq 'ARRAY')
{
[ map ENV_clone($_), @$this ];
} elsif (ref $this eq 'HASH')
{
scalar { map { $_ => ENV_clone($this->{$_}) } keys %$this };
} else
{
ENV_fatal( "clone: What type is $this?");
}
}




sub ENV_dump($$;$$)
{
my ($name,
$item,		# undef, scalar or ref
$max_level,
$unsorted_hash
) = @_;

print( "---$name\n");
$max_level = 100
if (!defined $max_level);
dump_item( 0, $max_level, $item, 1, $unsorted_hash);
print( "---\n");
}




sub dump_item($$$$$)
{
my ($level,
$max_level,
$item,
$must_indent,
$unsorted_hash
) = @_;

my $indent = '  ' x $level;
$level++;
ENV_fatal( "maxlevel reached") if ($level > 1000);

print( $indent)
if ($must_indent);
if ($level > $max_level)
{
print( "***\n");
} else
{
if (defined $item)
{

if (ref $item)
{
if (ref $item eq 'SCALAR')
{
print( "->");
dump_item( $level, $max_level, $$item, 1, $unsorted_hash);

} elsif (ref $item eq "ARRAY")
{
if ( @{$item})
{
print( "->[\n");
foreach my $value (@{$item})
{
dump_item( $level + 1, $max_level, $value, 1, $unsorted_hash);
}
print( $indent . "  ]\n");
} else
{
print( "->[]\n");
}
} elsif (ref $item eq "HASH")
{
if (keys %{$item})
{
print( "->{\n");
if ($unsorted_hash)
{
while (my ($key, $value) = each %{$item})
{
print( $indent . "    {$key}");
dump_item( $level + 1, $max_level, $value, 0, $unsorted_hash);
}
} else
{
foreach my $key (sort(keys %{$item}))
{
print( $indent . "    {$key}");
dump_item( $level + 1, $max_level, $item->{$key}, 0, $unsorted_hash);
}
}
print( $indent . "  }\n");
} else
{
print( "->{}\n");
}
} else
{
print( "->" . ref( $item) . "\n");
}
} else
{
print( "=") if (!$must_indent);
print( "'$item'\n");
}
} else
{
print( "=") if (!$must_indent);
print( "undef\n");
}
}
$level--;
}






sub ENV_eval_empty($)
{

return (defined $_[0] && $_[0] ne '""' && $_[0] ne "''") ? $_[0] : '';
}






sub ENV_eval_undef($)
{

return (defined $_[0] && $_[0] ne '' && $_[0] ne '""' && $_[0] ne "''") ? $_[0] : undef;
}






sub ENV_undef2empty(@)
{



return (wantarray) ? map { (defined $_) ? $_ : '' } @_ : (defined $_[0]) ? $_[0] : '';
}







sub ENV_getenv($)
{

my $env = ENV_eval_empty( $ENV{$_[0]});

return (wantarray) ? split( ',', $env) : $env;
}







sub ENV_getenv_perl_path($)
{
my ($name) = @_;
my $env;

$env = ENV_eval_empty( $ENV{$name});
if ($env ne '')
{

$env = ENV_perl_path_nospace( $env)
if ($IS_WIN32);

}

return $env;
}




sub ENV_setenv($$)
{
my ($name,
$value_or_ref
) = @_;
my $value;

if (!exists $CENVS{$name})
{
$CENVS{$name} = $ENV{$name};
}
if (ref $value_or_ref)
{
$value = join ',', @{$value_or_ref};
} else
{
$value = $value_or_ref;
}




ENV_fatal( "ENV_setenv: Invalid item $name = $value")
if (!defined $name || !defined $value);
$value = '""' if ($value eq '');
$ENV{$name} = $value;
$value = '' if ($value eq '""');

return $value;
}




sub ENV_get_changed_envs()
{
my @changed_envs;

foreach my $name (keys %CENVS)
{
my $old_value = $CENVS{$name};
my $cur_value = $ENV{$name};
if (defined $old_value)
{

push @changed_envs, $name if ($cur_value ne $old_value);
} else
{
push @changed_envs, $name;
}
}
@changed_envs = sort @changed_envs;


return @changed_envs;
}





sub ENV_commandfile_spec($)
{
my ($spec) = @_;

$spec .= ENV_commandfile_ext();
$spec =~ tr!/!\\!
if ($IS_WIN32);

return $spec;	# os format
}





sub ENV_commandfile_ext()
{
return ($IS_WIN32) ? '.bat' : '.sh';
}




sub ENV_is_commandfile($)
{
my ($file) = @_;	    # file or filespec

my ($type) = $file =~ /(\.*)/;		# greedy!

return ($type eq '.bat' || $type eq '.sh') ? 1 : 0;
}





sub ENV_chmod($@)
{
my ($symbolic,
@files
) = @_;
my $count = 0;

my %bits = ( u => 0, g => 1, o => 2, r => 4, w => 2, x => 1 );

my ($who, $op, $perms) = $symbolic =~ /^([ugoa]+)([+-=])([rwx]+)$/;
ENV_fatal( "chmod: Invalid symbolic '$symbolic'") if (!defined $who);
$who = 'ugo' if ($who =~ /a/);

my @ugos;
map { push @ugos, $bits{$_} } split //, $who;

my $value = 0;
map { $value |= $bits{$_} } split //, $perms;

foreach my $file (@files)
{
my $org_mode = (stat $file)[2];
if (!defined $org_mode)
{
ENV_error( "chmod $symbolic: No such file/dir: $file");
next;
}
my @values = (($org_mode & 0700) >> 6, ($org_mode & 070) >> 3, $org_mode & 07);

if ($op eq '+')
{
map { $values[$_] |=  $value } @ugos;
} elsif ($op eq '-')
{
map { $values[$_] &= ~$value } @ugos;
} else	# ($op eq '=')
{
map { $values[$_]  = $value } @ugos;
}
my $mode = ($values[0] << 6) | ($values[1] << 3) | $values[2];
my $new_mode = $org_mode & ~0777 | $mode;
if ($new_mode != $org_mode)
{

$count += chmod $new_mode, $file;
} else
{
$count++;
}
}

return $count;
}





sub ENV_get_hostname()
{
return OS_get_hostname();
}





sub ENV_uname()
{
return OS_get_uname();
}





sub ENV_unique_id()
{
my $unique_id;

my $hostname = ENV_get_hostname();
my $time = time;

$unique_id = "${hostname}_${time}_${$}_${UNIQUE_COUNT}";
$UNIQUE_COUNT++;


return $unique_id;
}





sub ENV_get_userid()
{
return OS_get_userid();
}





sub ENV_is_wildcard($)
{
return ($_[0] =~ /\*|\?/) ? 1 : 0;
}





sub ENV_glob(@)
{



return (wantarray) ? map { glob $_ } @_ : glob $_[0];
}




sub ENV_wild_2_re($)
{
my ($wildcard) = @_;
my $re;

$re = quotemeta $wildcard;
$re =~ s/\\\*/.*/g;
$re =~ s/\\\?/./g;
$re = "^$re\$";


return $re;
}









sub ENV_wildcard($$)
{
my ($find,		# this is the wildcard
$list_ref,	# single or ref-list
) = @_;
my @found_items;

$list_ref = [$list_ref] unless( ref $list_ref);

if ($find eq '*')
{
@found_items = @{$list_ref};
} else
{
my $find_re = ENV_wild_2_re( $find);
@found_items = grep( /$find_re/, @{$list_ref});
}


return (wantarray) ? @found_items : $found_items[0];
}








sub ENV_wildcards($$$)
{
my ($finds_ref,	    # this is the wildcards list
$list_ref,	    # single or ref-list
$no_matches_ref,    # may be undef
) = @_;
my @found_items;

$finds_ref = [$finds_ref] unless( ref $finds_ref);
$list_ref = [$list_ref] unless( ref $list_ref);

my %dups;


foreach my $find (@{$finds_ref})
{
my $find_re = ENV_wild_2_re( $find);
my @these_found_items = grep( /$find_re/, @{$list_ref});
if (@these_found_items)
{
foreach my $found_item (@these_found_items)
{
if (!defined $dups{$found_item})
{
push @found_items, $found_item;
$dups{$found_item} = 1;
}
}
} else
{
push @{$no_matches_ref}, $find
if (defined $no_matches_ref);
}
}


return @found_items;
}








sub ENV_not_wildcards($$)
{
my ($finds_ref,	# this are the wildcards
$list_ref,	# single or ref
) = @_;
my @found_items;

$list_ref = [$list_ref] unless( ref $list_ref);
if (ref $finds_ref)
{
my @find_res = map { ENV_wild_2_re( $_) } @{$finds_ref};
foreach my $item (@{$list_ref})
{
my $found = 0;
foreach my $find_re (@find_res)
{
if ($item =~ $find_re)
{
$found = 1;
last;
}
}
push @found_items, $item
if (!$found);
}
} else
{
if ($finds_ref eq '*')
{
@found_items = ();
} else
{
my $find_re = ENV_wild_2_re( $finds_ref);
@found_items = grep( !/$find_re/, @$list_ref);
}
}

return @found_items;
}








sub ENV_get_term_size()
{
my ($width, $height) = OS_get_term_size();

$width = $XTERM_DEFAULT_WIDTH
if (!defined $width);
$height = $XTERM_DEFAULT_HEIGHT
if (!defined $height);

return (wantarray) ? ($width, $height) : $width;
}







sub ENV_get_term_size_default()
{
return (wantarray) ? ($XTERM_DEFAULT_WIDTH, $XTERM_DEFAULT_HEIGHT) : $XTERM_DEFAULT_WIDTH;
}




sub ENV_is_abs_path($)
{
my ($path) = @_;



if ($IS_WIN32)
{
return $path =~ m!^"?([A-Za-z]:(/|\\)|//.+|\\\\.+)!;
} else
{
return $path =~ m!^"?(/|~/)!;
}
}




sub ENV_is_abs_path_perl($)
{
my ($path) = @_;



if ($IS_WIN32)
{
return $path =~ m!^(//.+|[A-Za-z]:/)!;
} else
{
return $path =~ m!^(/|~/)!;
}
}




sub ENV_is_option($)
{
my ($spec) = @_;

my $fc = substr( $spec, 0, 1);
if ($IS_WIN32)
{
return ($fc eq '-' || $fc eq '/');
} else
{
return ($fc eq '-');
}
}




sub ENV_is_in_path($$)
{
my ($path,			# perl path
$starts_with_path,	# perl path
) = @_;

$starts_with_path = quotemeta $starts_with_path;

return ( $path =~ m!^$starts_with_path($|/)! ) ? 1 : 0;
}




sub ENV_parent_path($)
{
my ($path) = @_;
my $parent;

$parent = $path;
$parent =~ s!/[^/]+$!!;		    #! for syntax colouring
$parent = ''
if ($parent eq $path);


return $parent;
}




sub ENV_parent_dir($$)
{
my ($path,
$index	    # -1 == last, 0 == first
) = @_;
my $parent_name;

my @dirs = split( '/', $path);
$parent_name = $dirs[$index];
$parent_name = '' if (!defined $parent_name);


return $parent_name;
}




sub ENV_set_tmpdir_path($)
{
my ($tmpdir_path) = @_;	    # may be undef

$TMPDIR_PATH = $tmpdir_path;
if (defined $tmpdir_path)
{
ENV_setenv( ENV_TMPDIR_PATH => $tmpdir_path);		    # used only by this package
} else
{
ENV_setenv( ENV_TMPDIR_PATH => '');			    # used only by this package
}
}




sub ENV_get_tmpdir_path()
{
if (!defined $TMPDIR_PATH)
{
my $tmpdir_path = ENV_getenv_perl_path( 'ENV_TMPDIR_PATH'); # used only by this package
$tmpdir_path = ENV_get_os_tmpdir_path()
if ($tmpdir_path eq '');
$TMPDIR_PATH = $tmpdir_path;

}

return $TMPDIR_PATH;
}




sub ENV_get_os_tmpdir_path()
{
if (!defined $OS_TMPDIR_PATH)
{
my @tmp_envs = ($IS_WIN32) ? qw(TEMP TMP TMPDIR) : qw(TMPDIR);
my $tmpdir_path;
foreach my $tmp_env (@tmp_envs)
{
my $test_path = ENV_getenv_perl_path( $tmp_env);
if (-e $test_path)
{
$tmpdir_path = $test_path;
last;
}
}
if (!defined $tmpdir_path)
{
if ($IS_LUNIX)
{
$tmpdir_path = ENV_glob( '~/tmp');
$tmpdir_path = '/tmp'
if (! -e $tmpdir_path);
} else
{
ENV_error_exit( "No valid Temp. directory found in (@tmp_envs)");
}
}
$OS_TMPDIR_PATH = $tmpdir_path;
}

return $OS_TMPDIR_PATH;
}




sub ENV_get_tmp_spec($)
{
my ($file) = @_;
my $spec;

my $temp_path = ENV_get_tmpdir_path();
my $dotloc = rindex( $file, '.');
my $name;
my $type;
if ($dotloc == 0)	    # starts with '.'
{
$name = "";
$type = $file;
} elsif ($dotloc > 0)   # has '.'
{
$name = substr( $file, 0, $dotloc);
$type = substr( $file, $dotloc);
} else  	    	    # no '.'
{
$name = $file;
$type = '.tmp';
}

my $unique_id = ENV_unique_id();
$spec = "$temp_path/${name}_${unique_id}$type";

return $spec;
}




sub ENV_os_path($)
{
my ($file_spec) = @_;

if ($IS_WIN32)
{
$file_spec =~ tr!/!\\!;
return $file_spec;
} else  # Assume LUNIX
{
return $file_spec;
}
}





sub ENV_perl_paths(@)
{
my @paths = @_;
my @perl_paths;

foreach my $path (@paths)
{
$path =~ tr!\\!/!;	    #/ perl path
$path = ENV_canonicalize_path( $path);
push @perl_paths, $path;
}

return (wantarray) ? @perl_paths : $perl_paths[0];
}





sub ENV_perl_path_nospace(@)
{
my @paths = @_;
my @perl_paths;

foreach my $path (@paths)
{
if (substr( $path, 0, 1) eq '"' || index( $path, ' ') >= 0)
{
ENV_fatal( "DBG:  Path $path contains '$PATH_SEPARATOR'") if ($path =~ /$PATH_SEPARATOR/);




$path =~ s/"//g;
my $perl_path = $QUOTED_PATHS{$path};
if (defined $perl_path)
{
push @perl_paths, $perl_path;
} else
{
if ($IS_WIN32)
{
$perl_path = OS_short_path_name( $path);
$perl_path =~ tr!\\!/!;	    # Perl path
} else
{
$perl_path = $path;
}
$perl_path = ENV_canonicalize_path( $perl_path);
$QUOTED_PATHS{$path} = $perl_path;
push @perl_paths, $perl_path;
}
} else
{



my $perl_path;
if ($IS_WIN32)
{

$perl_path = $path
if (!defined $perl_path);
$perl_path =~ tr!\\!/!;	    # perl path
} else
{
$perl_path = $path;
}
$perl_path = ENV_canonicalize_path( $perl_path);
push @perl_paths, $perl_path;
}
}

return (wantarray) ? @perl_paths : $perl_paths[0];
}





sub ENV_perl_path_space(@)
{
my @paths = @_;
my @perl_paths;

foreach my $path (@paths)
{
push @perl_paths, OS_long_path_name( $path);
}

return (wantarray) ? @perl_paths : $perl_paths[0];
}





sub ENV_canonicalize_path($)
{
my ($file_spec) = @_;	# Perl path!






my $device;
my $dir_file;
if ($file_spec =~ m!^(/+)(.*)!)
{



$device = $1;
$dir_file = $2;
} elsif ($file_spec =~ m!^([a-zA-Z]:/?)(.*)!)
{



$device = ucfirst $1;
$dir_file = $2;
} else
{



$device = '';
$dir_file = $file_spec;
}



$dir_file =~ s!/+!/!g;				    # xx////yy	 -> xx/yy
$dir_file =~ s!(/\.)+/!/!g; 			    # xx/././yy  -> xx/yy
while ($dir_file =~ s!/[^/]+/\.\.(/|$)!/!) {};	    # /xx/yy/../ -> /xx
$dir_file =~ s!/\.$!!;				    # xx/.	 -> xx
$dir_file =~ s!/$!!;				    # xx/	 -> xx

$file_spec = "$device$dir_file";


return $file_spec;
}






sub ENV_expand_envs($)
{
my ($line) = @_;




$line =~ s/\$(\w+)/$ENV{$1} ? $ENV{$1} : "\$$1"/eg;




$line =~ s/\$\{([^\}]+)\}/$ENV{$1} ? $ENV{$1} : "\${$1}"/eg;

if ($IS_WIN32)
{



$line =~ s/\%(\w+)\%/$ENV{$1} ? $ENV{$1} : "%$1%"/eg;
} else
{




$line = ENV_glob( $line)
if (substr( $line, 0, 1) eq '~');
}

return $line;
}





sub ENV_expand_arg($)
{
my ($arg) = @_;
my $args;

$args = "\\\$$arg\(\\W|\$)|\\\$\\\{$arg\\\}";
$args .= "|\\\%$arg\\\%"
if ($IS_WIN32);

return qr($args);
}





sub ENV_delink_spec($)
{
my ($spec) = @_;
my $new_spec;


$new_spec = $spec;
if ($IS_LUNIX)
{
my $link_replaced;
my $count = 0;
my @spec = split '/', $spec;
do
{
$count++;
ENV_error_exit( "More than $count nested links in '$spec'") if ($count > 10);
$link_replaced = 0;
my @new_spec;
while (@spec)
{
my $part = shift @spec;

my $part_spec = join( '/', @new_spec, $part);

if (-l $part_spec)
{
my $link = readlink $part_spec;
if (substr( $link, 0, 1) eq '/')
{
@new_spec = ($link);
} else
{
push @new_spec, $link;
}
$link_replaced = 1;
} else
{
push @new_spec, $part
}
}
if ($link_replaced)
{
$new_spec = join( '/', @new_spec);

$new_spec = ENV_canonicalize_path( $new_spec);

@spec = split '/', $new_spec;
}
} while ($link_replaced);
}

return $new_spec;
}




sub ENV_rel_spec($$)
{
my ($from_path,   # Absolute
$to_file     # Absolute
) = @_;
my $result;



my @from_path = split '/', $from_path;
my @to_file = split '/', $to_file;
my $org_nr_from = @from_path;
$org_nr_from -= 1 if (substr( $from_path, 0, 1) eq '/');

if ($from_path[0] eq $to_file[0])
{
while (@from_path && ($from_path[0] eq $to_file[0]))
{
shift @from_path;
shift @to_file;
}
my $nr_from = scalar @from_path;

if ($nr_from < $org_nr_from)
{
$result = '../' x $nr_from . join( '/', @to_file);
} else
{
$result = $to_file;
}
} else
{
$result = $to_file;
}


return $result;
}




sub ENV_abs_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!defined $cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

if (ENV_is_abs_path_perl( $path))
{
push @paths, $path;
} else
{
push @paths, ENV_canonicalize_path( "$cwd/$path");
}
}


(wantarray) ? return @paths : $paths[0];
}




sub ENV_rel_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!$cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

if (ENV_is_abs_path_perl( $path))
{
push @paths, ENV_rel_spec( $cwd, $path);
} else
{
push @paths, $path;
}
}


(wantarray) ? return @paths : $paths[0];
}




sub ENV_shortest_path($$)
{
my ($cwd,
$paths_ref	    # ref or scalar
) = @_;
my @paths;

$paths_ref = [ $paths_ref ] if (!ref $paths_ref);

$cwd = ENV_cwd() if (!$cwd || $cwd eq '.');


foreach my $path (@{$paths_ref})
{

my $abs_path;
my $rel_path;
if (ENV_is_abs_path_perl( $path))
{
$abs_path = $path;
$rel_path = ENV_rel_spec( $cwd, $path);
} else
{
$abs_path = ENV_canonicalize_path( "$cwd/$path");
$rel_path = $path;
}
if (length( $abs_path) < length( $rel_path))
{
push @paths, $abs_path;
} else
{
push @paths, $rel_path;
}
}


(wantarray) ? return @paths : $paths[0];
}







sub ENV_short_path_name(@)
{
my (@paths,
) = @_;
my @short_os_paths;

foreach my $path (@paths)
{
push @short_os_paths, OS_short_path_name( $path);
}

return (wantarray) ? @short_os_paths : $short_os_paths[0];
}






sub ENV_split_spec_dnt($)
{
my ($file_spec) = @_;
my ($dir_path, $name, $type);

my $i = rindex( $file_spec, '/');
my $file;
if ($i >= 0)
{
$dir_path = substr( $file_spec, 0, $i);
$file = substr( $file_spec, $i + 1);
} else
{
$dir_path = '';
$file = $file_spec;
}

$i = rindex ($file, '.');
if ($i >= 0)
{
$name = substr( $file, 0, $i);
$type = substr( $file, $i);
} else
{
$name = $file;
$type = '';
}


return ($dir_path, $name, $type);
}






sub ENV_split_spec_df($)
{
my ($file_spec) = @_;
my ($dir_path, $file);

my $i = rindex( $file_spec, '/');
if ($i >= 0)
{
$dir_path = substr( $file_spec, 0, $i);
$file = substr( $file_spec, $i + 1);
} else
{
$dir_path = '';
$file = $file_spec;
}


return ($dir_path, $file);
}






sub ENV_split_spec_d($)
{
my ($file_spec) = @_;
my $dir_path = '';

my $i = rindex( $file_spec, '/');
$dir_path = substr( $file_spec, 0, $i) if ($i >= 0);


return $dir_path;
}






sub ENV_split_spec_f($)
{
my ($file_spec) = @_;
my $file;

my $i = rindex( $file_spec, '/');
if ($i >= 0)
{
$file = substr( $file_spec, $i + 1) ;
} else
{
$file = $file_spec;
}


return $file;
}






sub ENV_split_spec_nt($)
{
my ($file_spec) = @_;
my ($name, $type);

my $i = rindex( $file_spec, '/');
$file_spec = substr( $file_spec, $i + 1) if ($i >= 0);
$i = rindex( $file_spec, '.');
if ($i >= 0)
{
$name = substr( $file_spec, 0, $i);
$type = substr( $file_spec, $i);
} else
{
$name = $file_spec;
$type = '';
}

return ($name, $type);
}






sub ENV_split_spec_n($)
{
my ($file_spec) = @_;
my $name;

my $i = rindex( $file_spec, '/');
$file_spec = substr( $file_spec, $i + 1) if ($i >= 0);

$i = rindex( $file_spec, '.');
if ($i >= 0)
{
$name = substr( $file_spec, 0, $i);
} else
{
$name = $file_spec;
}
return $name;
}






sub ENV_split_spec_t($)
{
my ($file_spec) = @_;
my $type = '';

my $i = rindex( $file_spec, '/');
$file_spec = substr( $file_spec, $i + 1) if ($i >= 0);

$i = rindex( $file_spec, '.');
$type = substr( $file_spec, $i) if ($i >= 0);

return $type;
}





sub ENV_split_path($)
{
my ($path) = @_;


return split( $PATH_SEPARATOR, $path);
}





sub ENV_join_path($)
{
my ($path_ref) = @_;


return join( $PATH_SEPARATOR, @{$path_ref});
}





sub ENV_chdir($)
{
my ($path) = @_;

chdir( $path) || ENV_fatal( "Unable to chdir( $path)", "- $!");
$ENV{PWD} = $path;

return $path;
}





sub ENV_cwd()
{
my $pwd = OS_get_cwd();

$ENV{PWD} = $pwd;

return $pwd;
}







sub ENV_file_is_newer($$)
{
my ($file1,
$file2,
) = @_;
my $is_newer;

no integer;
my $dmtp1 = -M $file1;	    # date modified to present
if (defined $dmtp1)
{
my $dmtp2 = -M $file2;


if (defined $dmtp2)
{
$is_newer = ($dmtp1 < $dmtp2) ? 1 : 0;		# produce prinatble result
} else
{
$is_newer = 1;
}
} else
{
$is_newer = 0;
}
use integer;

return $is_newer;
}






sub ENV_file_is_same($$)
{
my ($file1,
$file2,
) = @_;
my $is_same = 0;

no integer;
my $dmtp1 = -M $file1;	    # date modified to present
if (defined $dmtp1)
{
my $size1 = -s _;
my $dmtp2 = -M $file2;

if (defined $dmtp2)
{
my $size2 = -s _;

$is_same = ($dmtp1 == $dmtp2 && $size1 == $size2) ? 1 : 0;	# produce prinatble result
}
}
use integer;

return $is_same;
}






sub ENV_mkdir($;$)
{
my ($path,
$mask
) = @_;

$mask = 0777 if (!$mask);	    # ugo=rwx

if (!mkdir( $path, $mask))
{




my $rt = $!;
if (!-d $path)
{
my $omask = sprintf( "0%o", $mask);
ENV_fatal( "Unable to mkdir '$path' ($omask)", "- $rt");
}
}
if (!-d $path)
{
ENV_fatal( "Failed to mkdir '$path' for unclear reason");
}
}





sub ENV_rmdir($)
{
my ($path) = @_;

rmdir( $path) || ENV_fatal( "Unable to rmdir( $path)", "- $!");
}





sub ENV_rmdir_f($)
{
my ($path) = @_;

if (!rmdir $path)
{
chmod 0777, $path;
ENV_fatal( "Unable to rmdir_f( $path)\n", " -$!")
if !(rmdir $path);
}
}






sub ENV_readdir($)
{
my ($path) = @_;
my @all_entries;

opendir( THISDIR, $path) || ENV_fatal( "Unable to opendir '$path'", "- $!");
@all_entries = grep( ! /^\.\.?$/, readdir THISDIR); # skip . and ..
closedir THISDIR;

return sort( @all_entries);
}






sub ENV_rename($$)
{
my ($oldname,
$newname) = @_;

rename( $oldname, $newname) || ENV_fatal( "Unable to rename '$oldname'",
"              -> '$newname'",
"- $!");
}





sub ENV_unlink(@)
{


foreach my $file (@_)
{
if (! unlink( $file))
{



sleep( 1);
if (unlink( $file))
{
ENV_warn( "Delete of '$file' succeeded after delay");
} else
{
ENV_fatal( "Cannot delete '$file'", "- $!");
}
}
}
}




sub ENV_copy_file($$$)
{
my ($in_file,
$out_file,
$exit_on_error,
) = @_;
my $replicated;    	    # 1 = replicated, -1 = error




my ($rc, $command, $fail_reason) = OS_copy_file( $in_file, $out_file);
if ($rc != 0)
{
if ($exit_on_error)
{
ENV_fatal( "Copy (replicate) command failed. rc = $rc",
"Command: $command",
"- $fail_reason");
} else
{
ENV_error( "Copy (replicate) command failed. rc = $rc",
"Command: $command",
"- $fail_reason");
}
$replicated = -1;
} else
{
$replicated = 1;
}
return $replicated;
}





sub ENV_touch_file($)
{
my ($file_spec) = @_;

open( TOUCH, ">>$file_spec") || ENV_fatal( "Unable to touch (append) file '$file_spec'", "- $!");
close( TOUCH);
}




sub ENV_hide_file($$)
{
my ($file_spec,
$must_hide	    # 0 = unhide, 1 = hide, undef = test-only
) = @_;


return OS_hide_file( $file_spec, $must_hide);
}




sub ENV_max_command_line()
{
if (!defined $MAX_COMMAND_LINE)
{

if ($IS_WIN32)
{
$MAX_COMMAND_LINE = 8191;

} else  # Unix
{
$MAX_COMMAND_LINE = 32000;
}

$MAX_COMMAND_LINE--;    # just to be sure!
}

return $MAX_COMMAND_LINE;
}






sub ENV_command_split($)
{
my ($command_line) = @_;
my ($command, @args);

($command, @args) = ENV_split_quoted_space( $command_line);

return ($command, [ @args ]);
}




sub ENV_new_command_ref($$$$)
{
my ($type,
$prefix,    # (perl | call | start | '')
$os_exec,
$args_ref,  # may be undef
) = @_;
my $command_ref;

my @args = (defined $args_ref) ? @{$args_ref} : ();
$command_ref = [ $type, $prefix, $os_exec, @args ];

return $command_ref;
}




sub ENV_get_command_ref_data($)
{
my ($command_ref
) = @_;


return @{$command_ref};
}




sub ENV_command_ref($$)
{
my ($os_command_file,
$args_ref,
) = @_;
my $command_ref;

my $type;
my $prefix = '';
my $os_exec;

$type = lc ENV_split_spec_t( $os_command_file);
if ($type eq '.pl')
{
$prefix = 'perl';
$os_command_file = ENV_find_in_perl_inc( $os_command_file, 1);
} else
{
if ($IS_WIN32)
{
if ($os_command_file eq 'start')
{
$prefix = 'start';
$os_command_file = undef;
} else
{
if ($type eq '.bat')
{
$prefix = 'call';
} elsif ($type eq '.exe' || $type eq '.com' || $type eq '')
{
;	# no change
} else
{
$prefix = 'start';
}
}
} else	#Unix
{
;	# no change
}
}
$os_exec = $os_command_file;

$command_ref = ENV_new_command_ref( $type, $prefix, $os_exec, $args_ref);

return $command_ref;
}






sub ENV_command_line($)
{
my ($command_ref) = @_;	# [ $type, $prefix, $os_exec, @args ]


my $command_line;

ENV_fatal( "Error Calling Env_command_ref - not a ref")
if (!ref $command_ref);
my (undef, $prefix, $os_exec, @args) = ENV_get_command_ref_data( $command_ref);

my @command;
if ($prefix ne '')
{
$prefix = $PERL_CMD
if ($prefix eq 'perl');
push @command, $prefix;
}
push @command, $os_exec
if ($os_exec);
push @command, ENV_join_quoted_space( @args)
if (@args && $args[0] ne '');
$command_line = "@command";



return $command_line;
}





sub ENV_find_in_perl_inc($$)
{
my ($file,
$must_find,
) = @_;
my $file_spec = '';

if (ENV_is_abs_path( $file))
{
$file_spec = $file;
} else
{
foreach my $path (@INC)
{
if (-f "$path/$file")
{
$file_spec = "$path/$file";
last;
}
}
ENV_fatal( "Could not find '$file' in (@INC)")
if ($file_spec eq '' && $must_find);
}

return $file_spec;
}






sub ENV_system($$$)
{
my ($command_or_ref,    # command or $command_ref (ADT)
$oks_ref,	    # allowed return-codes (undef == nocheck)
$logfile,	    # undef = no-redirection, >name = apend, 0 or '' = redirect to NULL
) = @_;
my $rc;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;

if (defined $logfile)
{
my $redirect = '>';
if ($logfile eq '' || $logfile eq '0')
{
$logfile = ($IS_WIN32) ? 'NUL' : '/dev/null';
} elsif (substr( $logfile, 0, 1) eq '>')
{
$logfile = substr( $logfile, 1);
$redirect = '>>';
}
$command = "$command $redirect $logfile 2>&1";
}



if (length( $command) > ENV_max_command_line())
{
my $command_length = length( $command);
ENV_fatal( "system(): commandline length ($command_length) > $MAX_COMMAND_LINE.",
"First part is:",
substr( $command, 0, 80),
"Last part is:",
substr( $command, -80));
}
my $full_rc = system( $command ); 	  # execute the command

if ($full_rc == 0)
{
$rc = 0;
} else
{
if ($full_rc == -1)
{
ENV_fatal( "system: Cannot execute '$command'",
"- $!");
} else
{
$rc = ENV_rc_status( $full_rc, "system $command");
}
}
if (defined $oks_ref)
{
my @oks = (ref $oks_ref) ? @{$oks_ref} : ($oks_ref);
my $rc_ok = grep( $_ == $rc, @oks);
if (!$rc_ok)
{
ENV_fatal( "Unable to execute '$command' (rc=$rc)");
}
}

return $rc;
}




sub ENV_run3($$$$$)
{
my ($command_or_ref,    # command or $command_ref (ADT)
$multipart_ref,	    # undef = ok. Entries with whitespace will be quoted
$oks_ref,	    # allowed return-codes (undef == nocheck)
$stdout_ref,	    # 1 (non-ref==discard)		 can be ARRAY or SCALAR
$stderr_ref,	    # 2 (undef==print, non-ref==discard) can be ARRAY or SCALAR
) = @_;
my $rc;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;


my $stdout_file = ENV_get_tmp_spec( 'run3_stdout');
my $stderr_file = ENV_get_tmp_spec( 'run3_stderr');
my $command_tail = "2> $stderr_file > $stdout_file";
my @commands;
if (defined $multipart_ref)
{
my @multiparts = ENV_enquote( @{$multipart_ref});

my $full_command = "$command @multiparts $command_tail";
if (length( $full_command) <= ENV_max_command_line())
{
@commands = ($full_command);
} else
{
my $command_tail_l = length( $command_tail);
my $full_command = $command;
foreach my $multipart (@multiparts)
{
if (length( $full_command) + 1 + length( $multipart) + 1 + $command_tail_l < $MAX_COMMAND_LINE)
{
$full_command .= ' ' . $multipart;
} else
{
push @commands, $full_command . ' '. $command_tail;
$full_command = $command;
}
}
push @commands, $full_command . ' '. $command_tail
if (length( $full_command) != length($command));
}
} else
{
@commands = ("$command $command_tail");
}




my $full_stdout;
my $full_stderr;
foreach my $command_line (@commands)
{

$rc = ENV_system( $command_line, undef, undef);
my $stdout = slurp_file( $stdout_file);
my $stderr = slurp_file( $stderr_file);
unlink( $stdout_file, $stderr_file);
if (defined $oks_ref)
{
my @oks = (ref $oks_ref) ? @{$oks_ref} : ($oks_ref);
my $rc_ok = grep( $_ == $rc, @oks);
if (!$rc_ok)
{
ENV_say(   1, "### ENV_run3 failed (rc=$rc) ###");
ENV_say(   0, "COMMAND:",
"  $command_line",
"STDOUT:");
ENV_print( 0, $stdout);
ENV_say(   0, "STDERR:");
ENV_print( 0, $stderr);
ENV_say(   0, "***");
ENV_fatal( "Unable to execute command (rc=$rc)");
} else
{
if (!defined $stderr_ref && $stderr ne '')
{
exec_beep();
ENV_say( 0, $command);
ENV_print( 0, $stderr);
}
}
}



$full_stdout .= $stdout;
$full_stderr .= $stderr;
}

de_ref( $stdout_ref, $full_stdout);
de_ref( $stderr_ref, $full_stderr);

return $rc;
}






sub ENV_exec($$)
{
my ($command_or_ref,	    # command or $command_ref (ADT)
$log_file,		    # undef = no redirection
) = @_;

my $command = (ref $command_or_ref) ? ENV_command_line( $command_or_ref): $command_or_ref;

$command = "$command > $log_file 2>&1"
if (defined $log_file);

$RUNNING_IN_EXEC = 1;
exec( $command);

die( "$PREFIX: exec '$command' failed\n - $!");	# perl will not accept anything other than 'die' after 'exec'
}




sub ENV_rc_status($$)
{
my ($full_rc,
$msg,
) = @_;
my $rc;

my $rc16 = $full_rc & 0xffff;
$rc = $rc16 >> 8;
my $sig = $rc16 & 127;
my $core = $rc16 & 128;



if ($sig != 0)
{
ENV_error( "Command '$msg' failed with signal $sig (rc=$rc)");
}
if ($core != 0)
{
ENV_error( "Command '$msg' failed with coredump (sig=$sig rc=$rc)");
}

return $rc;
}




sub de_ref($$)
{
my ($value_ref,	# undef: do nothing. ref to scalar or list
$string,
) = @_;

my $ref = ref $value_ref;
if ($ref eq 'ARRAY')
{
@{$value_ref} = split( /\s*\n/, $string);
} elsif ($ref eq 'SCALAR')
{
$$value_ref = $string;
}
}





sub ENV_detab($)
{
my ($line) = @_;

my $tab_pos = index( $line, "\t");
while ($tab_pos >= 0)
{

my $wanted_out_pos = (($tab_pos + 8) / 8) * 8;
my $nr_spaces = $wanted_out_pos - $tab_pos;
my $spaces = ' ' x $nr_spaces;
$line = substr( $line, 0, $tab_pos) .
$spaces .
substr( $line, $tab_pos + 1);

$tab_pos = index( $line, "\t");
}

return $line;
}





sub ENV_split_quoted_space($)
{
my ($line) = @_;
my @list;

if (index( $line, '"') >= 0)

{
my $i = 0;
my $last_i = length( $line) + 1;
$line .= '  ';				    # add 2 dummy trailing space
my $next_char = substr( $line, $i++, 1);
do
{




if ($i < $last_i)
{
while ($next_char eq ' ' || $next_char eq "\t")
{
$next_char = substr( $line, $i++, 1);
}
}
if ($i < $last_i)
{
my $word = '';
if ($next_char eq '"')
{



$next_char = substr( $line, $i++, 1);	# skip opening quote
while ($next_char ne '"' && $i < $last_i)
{
$word .= $next_char;
if ($next_char eq '\\')
{
$next_char = substr( $line, $i++, 1);
$word .= $next_char;
}
$next_char = substr( $line, $i++, 1);
}
$next_char = substr( $line, $i++, 1);	#skip closing quote
} else
{



while ($next_char ne ' ' && $next_char ne "\t")
{
$word .= $next_char;
$next_char = substr( $line, $i++, 1);
if ($next_char eq '"')			# internal '"'
{
$word .= $next_char;		# add opening quote
$next_char = substr( $line, $i++, 1);
while ($next_char ne '"' && $i < $last_i)
{
$word .= $next_char;
if ($next_char eq '\\')
{
$next_char = substr( $line, $i++, 1);
$word .= $next_char;
}
$next_char = substr( $line, $i++, 1);
}
$word .= $next_char; 		# add closing quote
$next_char = substr( $line, $i++, 1);
}
}
}
push @list, $word;
}
} while ($i < $last_i);

return @list;




my @items = split( / /, $line);	    #/ for syntax colouring
my @substring = ();
foreach my $item (@items)
{
if (@substring)
{
if (substr( $item, -1, 1) eq '"' && substr( $item, -2, 1) ne '\\')
{



push @substring, $item;
push @list, "@substring";
@substring = ();
} else
{
push @substring, $item;
}
} else
{



if ($item =~ /[^\\]?"/)
{
if (substr( $item, -1, 1) eq '"' && substr( $item, -2, 1) ne '\\')
{
push @list, $item;
} else
{
push @substring, $item;
}
} else
{
push @list, $item;
}
}
}
push @list, @substring;
} else
{
@list = split( ' ', $line);	    # split on any whitespaces
}

return @list;
}




sub ENV_join_quoted_space(@)
{
my @list = @_;


foreach my $word (@list)
{
ENV_stackdump() if (!defined $word);
if ($word eq '')
{
$word = '""';
} elsif (index( $word, ' ') >= 0 && index( $word, '"') < 0)
{



$word =~ s/"/\\"/g;
$word = "\"$word\"";
}
}

return "@list";
}




sub ENV_enquote(@)
{
my @list = (@_);


foreach my $word (@list)
{
if ($word eq '')
{
$word = '""'
} elsif (substr( $word, 0, 1) ne '"' && $word =~ /"|\s/)
{
$word =~ s/"/\\"/g;
$word = "\"$word\"";
}
}


return wantarray ? @list : $list[0];
}




sub ENV_dequote(@)
{
my @list = (@_);


foreach my $word (@list)
{
if (substr( $word, 0, 1) eq '"')
{
$word =~ s/\\"/"/g;
$word = substr( $word, 1, -1);
}
}


return wantarray ? @list : $list[0];
}




sub ENV_escape_os_text($)
{
my ($text) = @_;
my $new_text = $text;

if ($IS_WIN32)
{
my $dos_special_chars = '()<>|&^';
$new_text =~ s/([$dos_special_chars])/^$1/g;	# escape special characters
} else
{
my $unix_special_chars = '\\\\()<>|';
$new_text =~ s/([$unix_special_chars])/\\$1/g;	# escape special characters
}


return $new_text;
}




sub ENV_redirect_stdouterr($)
{
my ($log_file) = @_;

open( $SAVED_STDOUT, '>&', STDOUT) || die "$PREFIX: Cannot save STDOUT\n- $!";
open( $SAVED_STDERR, '>&', STDERR) || die "$PREFIX: Cannot save STDERR\n- $!";
open( STDOUT, '>>', $log_file) || die "$PREFIX: Cannot redirect STDOUT to $log_file\n- $!";
open( STDERR, '>&', STDOUT) || die "$PREFIX: Cannot duplicate STDERR to STDOUT\n- $!";

my $saved_handle = select STDERR;
$| = 1;
select STDOUT;
$| = 1;
select $saved_handle;
}




sub ENV_restore_stdouterr()
{
close( STDOUT) || die "$PREFIX: Cannot close STDOUT\n- $!";
close( STDERR) || die "$PREFIX: Cannot close STDERR\n- $!";
open( STDOUT, '>&', $SAVED_STDOUT) || die "$PREFIX: Cannot restore STDOUT\n- $!";
open( STDERR, '>&', $SAVED_STDERR) || die "$PREFIX: Cannot restore STDERR\n- $!";
}




sub ENV_get_desktop_path($)
{
my ($system_item) = (@_);

return OS_get_desktop_path( $system_item);
}




sub ENV_get_startmenu_path($)
{
my ($system_item) = (@_);

return OS_get_startmenu_path( $system_item);
}






sub ENV_get_programs_path($)
{
my ($system_item) = (@_);

return OS_get_programs_path( $system_item);
}






sub ENV_get_user_path($)
{
my ($system_item) = (@_);

return OS_get_user_path( $system_item);
}




sub ENV_is_administrator()
{
return OS_is_administrator();
}




sub slurp_file($)
{
my ($file) = @_;
my $lines;

open( IN, $file) || ENV_fatal( "Unable to open file '$file'", "- $!");
my $old_irs = $/;
$/ = undef;
$lines .= <IN>;
$/ = $old_irs;
close IN;

return $lines;
}

return 1;
