#===============================================================
#
#   spit.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package glo::spit;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SPIT_file
SPIT_file_nl
SPIT_tmp_file
SPIT_script_file
SPIT_tmp_script_file
);


use strict;
use integer;

use glo::env;
use glo::shell;




sub SPIT_file($@);
sub SPIT_file_nl($@);
sub SPIT_tmp_file($@);
sub SPIT_script_file($@);
sub SPIT_tmp_script_file($@);

sub unable_to_open($);









sub SPIT_file($@)
{
my ($file,
@lines,
) = @_;

if (open( OUT, ">$file"))
{
map { print( OUT (ref $_) ? @{$_} : $_) } @lines;
close( OUT) || ENV_fatal( "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file);
}
}





sub SPIT_file_nl($@)
{
my ($file,
@lines,
) = @_;

if (open( OUT, ">$file"))
{
foreach my $ref_or_line (@lines)
{
if (ref $ref_or_line)
{
map { CORE::say( OUT $_) } @{$ref_or_line};
} else
{
CORE::say( OUT $ref_or_line);
}
}
close( OUT) || ENV_fatal( "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file);
}
}





sub SPIT_tmp_file($@)
{
my ($tmp_file,
@lines,
) = @_;

my $file = ENV_get_tmp_spec( $tmp_file);

if (open( OUT, ">$file"))
{
map { print( OUT (ref $_) ? @{$_} : $_) } @lines;
close( OUT) || ENV_fatal( "Unable to close file '$file'", "- $!");
} else
{
unable_to_open( $file);
}

return $file;
}





sub SPIT_script_file($@)
{
my ($file,
@lines,
) = @_;

if (open( OUT, ">$file"))
{
map { print( OUT "$_\n") } SHELL_batch_header();
map { print( OUT (ref $_) ? @{$_} : $_) } @lines;
close( OUT) || ENV_fatal( "Unable to close file '$file'", "- $!");
chmod 0755, $file;	# set file to 'x'
} else
{
unable_to_open( $file);
}
}





sub SPIT_tmp_script_file($@)
{
my ($tmp_file,	# without .type
@lines,
) = @_;

my $file = ENV_get_tmp_spec( $tmp_file) . ENV_commandfile_ext();

if (open( OUT, ">$file"))
{
map { print( OUT "$_\n") } SHELL_batch_header();
map { print( OUT (ref $_) ? @{$_} : $_) } @lines;
map { print( OUT "$_\n") } SHELL_exit_rc();
close( OUT) || ENV_fatal( "Unable to close file '$file'", "- $!");
chmod 0755, $file;	# set file to 'x'
} else
{
unable_to_open( $file);
}

return $file;
}




sub unable_to_open($)
{
my ($file) = @_;

my $action = (substr( $file, 0, 1) eq '>') ? 'append' : 'create';
ENV_fatal( "Unable to $action file '$file'",
"- $!");
}

return 1;
