#=================================================
#
#  intgrtr.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package intgrtr;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
INTGTR_main
);
use strict;
use integer;

use glo::env;
use glo::ask;
use glo::build;
use gbsenv;
use gbsscm;
use validate;
use system;
use audithist;




sub INTGTR_main($$$);
sub consolidate_audits($$$);
sub version_limits($$$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');




sub INTGTR_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

my @menu_items = (
[ "Consolidate (base) Audits",
\&consolidate_audits ],
[ "Set GBS Version Limits",
\&version_limits ],
);

ENV_say( 1, "GBS Integrator Tools");

if (VALIDATE_integrator())
{
ASK_menu( 'Select function to perform', \@menu_items, $entries_ref);
}
}




sub consolidate_audits($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Consolidate (base) Audits");

my @gbs_all_audits = ENV_getenv( 'GBS_AUDITS');
if (@gbs_all_audits)
{
my @audits  = ASK_values_from_list( "Select Audit(s)",  '*', \@gbs_all_audits, 0);
if (@audits)
{
my @targets = ASK_values_from_list( "Select Target(s)", '*', \@GBS_TARGETS, 0);
if (@targets)
{




GBSSCM_preset( 0);
AUDITHIST_commit( \@audits, \@targets);
}
}
} else
{
ENV_say( 1, "No Audits defined");
}
}




sub version_limits($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Set GBS Version Limits for the current System/Root");

GBSSCM_preset( 0);




my $build_start_cur = SYSTEM_get( $GBS_ROOT_PATH, 'build_start');
my $build_end_cur   = SYSTEM_get( $GBS_ROOT_PATH, 'build_end');

my $build_now = ENV_getenv( 'GBS_BUILD');
my $build_start_new = $build_start_cur;
my $build_end_new   = $build_end_cur;




my $format1 = 'Build %-5.5s: %s';
my $format2 = 'Build %-5.5s: %-17.17s, was: %-17.17s';
ENV_say( 1, "Current Limits:");
ENV_say( 2, sprintf $format1, 'Start', scalar BUILD_build_2_version( $build_start_cur),
sprintf $format1, 'End',   scalar BUILD_build_2_version( $build_end_cur));




my @build_refs = ( [ '', '' ]);

my $default_value;
foreach my $spec (BUILD_get_all_spec_build_files( ENV_getenv_perl_path( 'GBS_SCRIPTS_PATH')))
{
my $dir = ENV_split_spec_d( $spec);
my $build = BUILD_get_build( $dir);
my $version = BUILD_build_2_version( $build);
$version .= ' Cur'
if ($build eq $build_now);
push @build_refs, [ $version, $build ];
}
if ($build_start_cur ne '' && !grep( $build_start_cur eq $_->[1], @build_refs))
{
my $version = BUILD_build_2_version( $build_start_cur) . ' Gone';
push @build_refs, [ $version , $build_start_cur ];
}
if ($build_end_cur ne '' && !grep( $build_end_cur eq $_->[1], @build_refs))
{
my $version = BUILD_build_2_version( $build_end_cur) . ' Gone';
push @build_refs, [ $version , $build_end_cur ];
}
@build_refs = sort( { $a->[1] cmp $b->[1] } @build_refs);




my $ans;
do
{
my $s_index = ASK_index_from_menu( 'Select Build Start', \$build_start_new, @build_refs);
my $e_index = ASK_index_from_menu( 'Select Build End', \$build_end_new, @build_refs);
$build_start_new = $build_refs[$s_index]->[1];
$build_end_new   = $build_refs[$e_index]->[1];

ENV_say( 1, "Limits:");
ENV_say( 2, sprintf $format2, 'Start', scalar BUILD_build_2_version( $build_start_new),
scalar BUILD_build_2_version( $build_start_cur));
ENV_say( 2, sprintf $format2, 'End',   scalar BUILD_build_2_version( $build_end_new),
scalar BUILD_build_2_version( $build_end_cur));
$ans = ASK_YNQ( 'OK?', 'N', 0);
} while ($ans eq 'N');




if ($ans eq 'Y' &&
($build_start_new ne $build_start_cur ||
$build_end_new   ne $build_end_cur))
{
ENV_say( 1, "Updating system.gbs...");
SYSTEM_write( $GBS_ROOT_PATH, build_start => $build_start_new,
build_end   => $build_end_new);
}
}


return 1;
