#=================================================
#
#  maint.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package maint;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
MAINT_audits_handling
MAINT_map_network_drives
);
use strict;
use integer;

use glo::env;
use glo::ask;
use glo::format;
use gbsdb;
use audit;




sub MAINT_audits_handling($$$);
sub MAINT_map_network_drives($$$);

sub run_audit($$);

sub map_add($$$);
sub map_del($$$);
sub map_show($);




my $IS_WIN32 = ENV_is_win32();




sub MAINT_audits_handling($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Audits Handling");

my @audit_items;
foreach my $audit (ENV_getenv( 'GBS_AUDITS'))
{
my $command_ref = AUDIT_get_maint_command_ref( $audit);
push @audit_items, [ $audit, \&run_audit, [ $audit, $command_ref ] ]
if (defined $command_ref);
}

if (@audit_items > 0)
{
ASK_menu( 'Select Audit', \@audit_items, $entries_ref);
} else
{
ENV_warn( "No Audit Maintenance scripts specified");
}
}




sub run_audit($$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

my ($audit, $command_ref) = @{$data_ref};

my $saved_gbs_audit = ENV_getenv( 'GBS_AUDIT');

ENV_setenv( GBS_AUDIT => $audit);
ENV_say( 1, "Executing $audit Maint...");
ENV_system( $command_ref, undef, undef);		# no RC check, no logfile

ENV_setenv( GBS_AUDIT => $saved_gbs_audit);
}




sub MAINT_map_network_drives($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref
) = @_;

ENV_say( 1, "Map Network Drives (Windows -> Unix)");

my @map_refs = GBSDB_maps_get();
map_show( \@map_refs);

my @menu_refs = (
[ 'Add    Entry', \&map_add, \@map_refs ],
[ 'Delete Entry', \&map_del, \@map_refs ],
);
ASK_menu( 'Select MAPs action', \@menu_refs, $entries_ref);

}




sub map_add($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref
) = @_;

my @map_refs = @{$data_ref};

my $win_path;
my $lunix_path;
if ($IS_WIN32)
{
$win_path = ASK_path( "Enter Windows Path (X:/... or //...)", '', 1, 0);
} else
{
$win_path = ASK_other_path( "Enter Windows Path (X:/... or //...)", '', 0);
}
if ($win_path ne '')
{
if ($IS_WIN32)
{
$lunix_path = ASK_other_path( "Enter Lunix Path (~/... or /...)", '', 0);
} else
{
$lunix_path = ASK_path( "Enter Lunix Path (~/... or /...)", '', 1, 0);
}
if ($lunix_path ne '')
{
ENV_say( 2, "$win_path -> $lunix_path");
if (ASK_YNQ( "OK?", 'Y', 1) eq 'Y')
{
push @map_refs, [ $win_path, $lunix_path ];
GBSDB_maps_write( \@map_refs);
@{$data_ref} = @map_refs;
ENV_say( 1, "Added");
}
}
}
map_show( \@map_refs);
}




sub map_del($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref
) = @_;

my @map_refs = @{$data_ref};

my @wins = map { "$_->[0] -> $_->[1]" } @map_refs;
my $index = ASK_index_from_menu( "Windows Path to be deleted", -1, ('', @wins));
if ($index != 0)
{
my ($win_path, $lunix_path) = @{$map_refs[$index - 1]};
ENV_say( 2, "$win_path -> $lunix_path");
if (ASK_YNQ( "OK?", 'Y', 1) eq 'Y')
{
@map_refs = grep $win_path ne $_->[0], @map_refs;
GBSDB_maps_write( \@map_refs);
@{$data_ref} = @map_refs;
ENV_say( 1, "Deleted");
}
}
map_show( \@map_refs);
}




sub map_show($)
{
my ($map_refs_ref) = @_;

if (@{$map_refs_ref})
{
ENV_say( 1, "Current Maps:");
ENV_say( 0, FORMAT_table( 0, 2, ' -> ', undef, @{$map_refs_ref}));
} else
{
ENV_say( 1, "No current Maps");
}
}

return 1;
