#===============================================================
#
#   doxygen.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Call:
#	arg_1	= $selection = 'full' (default) or 'api'
#============================================================================

print "*DOXYGEN\n" if ($ENV{GBSDEBUG_FILE});

use strict;
use integer;

use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::file;
use glo::spit;
use gbsenv;
use gbsglo;
use tool;




sub run_doxygen();
sub get_command_arguments();




my $GBS_SCRIPTS_PATH = GBSENV_perl_gbs_scripts();
my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_DOXYGEN_PATH = "$GBS_ROOT_PATH/silo/doxygen";
my $GBSEXT_DOXYGEN_PATH = ENV_getenv( 'GBSEXT_DOXYGEN_PATH');
my $HTML_DIR;
my $GBS_DOXYGEN_HTML_PATH;




my $SELECTION = 'full';	    # 'full' or 'api'












$| = 1;
my $RC = 0;
GBSENV_init();

get_command_arguments();

ENV_error_exit( "No GBS_ROOT_PATH defined") if (!$GBS_ROOT_PATH);
$HTML_DIR = "html_$SELECTION";
$GBS_DOXYGEN_HTML_PATH = "$GBS_DOXYGEN_PATH/$HTML_DIR";




run_doxygen();




ENV_exit( $RC);




sub END
{
ENV_print_end_msg(1);
}




sub run_doxygen()
{
ENV_say( 1, "SELECTION=$SELECTION");

my $inputs_file_spec = "$GBS_DOXYGEN_PATH/doxygen_inputs_${SELECTION}_$$.txt";




ENV_mkdir( $GBS_DOXYGEN_PATH);
ENV_mkdir( $GBS_DOXYGEN_HTML_PATH);




ENV_say( 1, "Cleanup directories...");
FILE_del_tree( $GBS_DOXYGEN_HTML_PATH, undef, 0);




ENV_say( 1, "Collecting directories...");
ENV_chdir( "$GBS_ROOT_PATH/dev");

my @dirs;
foreach my $subsys ( GBSGLO_subsystems_for_tool( 'doxygen'))
{
push @dirs, glob( "./$subsys/comp/*/src")
unless ($SELECTION eq 'api');
push @dirs, glob( "./$subsys/comp/*/inc");
push @dirs, glob( "./$subsys/comp/*/loc")
unless ($SELECTION eq 'api');
}
ENV_error_exit( "No valid directories found. <SubSystem>tools/doxygen directory missing?")
if (!@dirs);




{
my @lines;
push @lines, "INPUT = \\";
push @lines, map { "\t$_ \\" } @dirs;
push @lines, '';
SPIT_file_nl( $inputs_file_spec, \@lines);
}




my $config_file = "$GBS_ROOT_PATH/systools/doxygen/doxygen.conf";
$config_file = "$GBS_SCRIPTS_PATH/plugins/tools/doxygen/doxygen.conf"
if (! -e $config_file);
ENV_fatal( "doxygen.conf ($config_file) not found")
if (! -e $config_file);





ENV_say( 1, "Running doxygen...");

TOOL_set_envs( 'doxygen');

$ENV{GBS_DOXYGEN_INCLUDE} = $inputs_file_spec;
$ENV{GBS_DOXYGEN_PATH} = $GBS_DOXYGEN_PATH;
$ENV{GBS_HTML_OUTPUT} = $HTML_DIR;
my $command = ($GBSEXT_DOXYGEN_PATH) ? "$GBSEXT_DOXYGEN_PATH/bin/doxygen" : 'doxygen';
my $command_ref = ENV_command_ref( $command, [ $config_file ]);
$RC = ENV_system( $command_ref, undef, undef);
if ($RC == 0)
{
unlink( $inputs_file_spec);
}
}




sub get_command_arguments()
{

( $SELECTION,
) = ARGS_get( [0,1], 'perl doxygen.pl [ full | api ]', undef);

if (defined $SELECTION)
{
if ($SELECTION ne 'full' && $SELECTION ne 'api')
{
ARGS_error_exit( "Selector must be 'full' or 'api' ($SELECTION)");
}
}
}



