#===============================================================
#
#   roots.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package roots;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
ROOTS_get_nr
ROOTS_get_all
ROOTS_exists
ROOTS_add
ROOTS_delete
ROOTS_write
ROOTS_close
);

use strict;
use integer;

use glo::env;
use gbsdb;




sub ROOTS_get_nr();
sub ROOTS_get_all($);
sub ROOTS_exists($);
sub ROOTS_add($);
sub ROOTS_delete($);
sub ROOTS_write();
sub ROOTS_close($);

sub verify_file_open();












my $FILE_IS_OPEN = 0;
my $FILE_MODIFIED = 0;




my @OS_ROOTS;





sub ROOTS_get_nr()
{


verify_file_open();

return scalar @OS_ROOTS;
}




sub ROOTS_get_all($)
{
my ($perl_format,
) = @_;


verify_file_open();

if ($perl_format)
{
return map { ENV_perl_paths( $_) } @OS_ROOTS;
} else
{
return @OS_ROOTS;
}
}




sub ROOTS_exists($)
{
my ($root,
) = @_;


verify_file_open();

my $os_root = ENV_os_path( $root);

return (grep $_ eq $os_root, @OS_ROOTS) ? 1 : 0;
}




sub ROOTS_add($)
{
my ($root,
) = @_;
my $is_added = 0;

verify_file_open();

my $os_root = ENV_os_path( $root);

if (!grep $_ eq $os_root, @OS_ROOTS)
{
@OS_ROOTS = sort (@OS_ROOTS, $os_root);
$is_added = 1;
$FILE_MODIFIED = 1;
}

return $is_added;
}




sub ROOTS_delete($)
{
my ($root,
) = @_;
my $is_deleted = 0;

verify_file_open();

my $os_root = ENV_os_path( $root);

my $old_nr_roots = @OS_ROOTS;
@OS_ROOTS = grep $_ ne $os_root, @OS_ROOTS;
if (@OS_ROOTS != $old_nr_roots)
{
$is_deleted = 1;
$FILE_MODIFIED = 1;
}

return $is_deleted;
}




sub ROOTS_write()
{
my $was_modified = $FILE_MODIFIED;

if ($FILE_MODIFIED)
{
GBSDB_roots_write( \@OS_ROOTS);
$FILE_MODIFIED = 0;
}

return $was_modified;
}




sub ROOTS_close($)
{
my ($must_write) = @_;
my $was_modified = $FILE_MODIFIED;

if ($FILE_IS_OPEN)
{
if ($must_write)
{
ROOTS_write();
}
@OS_ROOTS = ();
$FILE_IS_OPEN = 0;
}

return $was_modified;
}




sub verify_file_open()
{
if (!$FILE_IS_OPEN)
{
@OS_ROOTS = GBSDB_roots_get();
$FILE_IS_OPEN = 1;
$FILE_MODIFIED = 0;
}
}

return 1;
