#===============================================================
#
#   scadef.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scadef;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SCADEF_get_severity_text
SCADEF_set_envs
SCADEF_metric_is_enabled
SCADEF_get_derived_items
SCADEF_get_section_items
SCADEF_get_levels
SCADEF_get_levels_by_level
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbssfile;
use gbsfileglo;
use scamet;




sub SCADEF_get_severity_text($);
sub SCADEF_set_envs($);
sub SCADEF_metric_is_enabled($$);
sub SCADEF_get_derived_items($@);
sub SCADEF_get_section_items($$@);
sub SCADEF_get_levels($);
sub SCADEF_get_levels_by_level($);

sub read_scadef($);
sub do_format($$$$$);
sub do_level($$$$$);
sub do_special($$$$$);
sub do_metric($$$$$);




my $GBS_SCRIPTS_PATH = GBSENV_perl_gbs_scripts();
my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SYSAUDIT_PATH = "$GBS_ROOT_PATH/sysaudit";

my $CUR_SCADEF = '';
my $CUR_SCADEF_REF;

my %SCAS = ();























my @SEVERITY_TEXTS = qw( Info Warning Fatal);




my @SEVERITY_VALUES = qw( I W F);
my %SEVERITY_VALUES = map { $_ => 1 } @SEVERITY_VALUES;












my %SETUP_DEFS = (
NICE_NAME	     => [ 'ss' , undef,	0, undef,     1, undef,	    undef ],
HELPFILES_PATH   => [ 'ss' , undef,	0, undef,     1, undef,	    undef ],
);

my %INIT_DEFS = (
SET_W	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET_X	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET		    => [ 'aa' , [],	   0, 2,     1,			 undef,	undef ],
SETPATH	    => [ 'aa' , [],	   0, 3,   [1],			 undef,	undef ],
CALL	    => [ 'aC' , [],	   0, 4, undef,			 undef,	undef ],    # Not implemented
);

my %OPTION_FORMATS_DEFS = (
QUIET_FORMAT    => [ 'ss' , undef,	   0, undef, undef,        \&do_format, undef ],
);

my %LEVELS_DEFS = (
LEVEL	    => [ 'aa' , [],	   0, undef,     2,         \&do_level, undef ],
);

my %MESSAGES_DEFS = (
SPECIAL	    => [ 'aa' , [],	   0, undef,     1,       \&do_special, undef ],
);
my %METRICS_DEFS = (
ENABLE	    => [ 'as' , [],	   0, undef,   [1],        \&do_metric, undef ],
IGNORE	    => [ 'as' , [],	   0, undef,   [1],        \&do_metric, undef ],
);










my %SECTION_DEFS = (
SETUP	    => [ 'N', 0, 1, 0,  \%SETUP_DEFS,		undef,  undef ],
INIT	    => [ 'N', 0, 0, 0,  \%INIT_DEFS,		undef,  undef ],
OPTION_FORMATS  => [ 'N', 0, 1, 0,  \%OPTION_FORMATS_DEFS,	undef,  undef ],
LEVELS	    => [ 'N', 0, 1, 0,  \%LEVELS_DEFS,		undef,  undef ],
MESSAGES	    => [ 'N', 0, 1, 0,  \%MESSAGES_DEFS,	undef,  undef ],
METRICS	    => [ 'N', 1, 0, 0,  \%METRICS_DEFS,		undef,  undef ],
);




sub SCADEF_get_severity_text($)
{
my ($severity_index) = @_;

return $SEVERITY_TEXTS[$severity_index];
}




sub SCADEF_set_envs($)
{
my ($audit,
) = @_;

read_scadef( $audit);




GBSSFILE_set_init_envs( $CUR_SCADEF_REF);
}




sub SCADEF_metric_is_enabled($$)
{
my ($audit,
$metric,
) = @_;
my $is_enabled;	    # 1 = enabled, 0 = not-enabled, undef = ignored

read_scadef( $audit);

$is_enabled = $CUR_SCADEF_REF->{'.'}->{ENABLED_METRICS}->{$metric}; # 1 = enabled 0 = ignored, undef = not-enabled
if (!defined $is_enabled)
{
$is_enabled = 0;
} elsif ($is_enabled == 0)
{
$is_enabled = undef;
}

return $is_enabled;
}




sub SCADEF_get_derived_items($@)
{
my ($audit,
@items,
) = @_;
my @values = ();

read_scadef( $audit);


return GBSSFILE_get_section_items( $CUR_SCADEF_REF, '.', @items);
}




sub SCADEF_get_section_items($$@)
{
my ($audit,
$section,
@items,
) = @_;
my @values = ();


read_scadef( $audit);

return GBSSFILE_get_section_items( $CUR_SCADEF_REF, $section, @items);
}





sub SCADEF_get_levels($)
{
my ($audit,
) = @_;
my @msg_level_refs;


read_scadef( $audit);

@msg_level_refs = @{GBSSFILE_get_section_items( $CUR_SCADEF_REF, 'LEVELS', 'LEVEL')};


return @msg_level_refs;
}





sub SCADEF_get_levels_by_level($)
{
my ($audit,
) = @_;
my %levels_by_level;


read_scadef( $audit);

my @msg_level_refs = @{GBSSFILE_get_section_items( $CUR_SCADEF_REF, 'LEVELS', 'LEVEL')};


%levels_by_level = map { $_->[0] => $_ } @msg_level_refs;		# $level

return %levels_by_level;
}




sub read_scadef($)
{
my ($audit) = @_;	    # undef = use current, '' = use EnvVar GBS_AUDIT

if (defined $audit)
{
$audit = ENV_getenv( 'GBS_AUDIT')
if ($audit eq '');

if ($audit ne $CUR_SCADEF)
{
$CUR_SCADEF = $audit;
$CUR_SCADEF_REF = $SCAS{$CUR_SCADEF};
}
if (!defined $CUR_SCADEF_REF)
{
$SCAS{$CUR_SCADEF} = {};
$CUR_SCADEF_REF = $SCAS{$CUR_SCADEF};




my $gbs_sca_plugin = ENV_getenv( 'GBS_SCA_PLUGIN_NAME');
my $sca_file = "$GBS_SCRIPTS_PATH/plugins/audit/$gbs_sca_plugin/scadef";
my @inc_path = ( "$GBS_SYSAUDIT_PATH/$CUR_SCADEF",
$GBS_SYSAUDIT_PATH);
GBSSFILE_parse( $CUR_SCADEF_REF, $sca_file, $audit, \@inc_path, \%SECTION_DEFS);

}
}
}




sub do_format($$$$$)
{
my ($section,	# OPTION_FORMATS
$subsection,	# empty
$item,		# QUIET
$values_ref,	# ([ $sub_key, @sub_values ], ...)
$func_par,
) = @_;

my $format  = $values_ref->[0];

GBSFILEGLO_error_exit( "Format must contain '%s'")
if ($format !~ /%s/);

return ($format);
}




sub do_level($$$$$)
{
my ($section,	# LEVELS
$subsection,	# empty
$item,		# LEVEL
$values_ref,	# ([ $sub_key, @sub_values ], ...)
$func_par,
) = @_;

my @value_refs = @{$values_ref};
foreach my $ref (@value_refs)
{
my ($level, $severity, $text) = @{$ref};

GBSFILEGLO_error_exit( "Level must be numeric ($level).")
if ($level !~ /^\d+$/);

GBSFILEGLO_error_exit( "Syntax is <level> => I|W|F <text>")
if ($severity !~ /^[IWF]$/);
push @{$ref}, index( 'IWF', $severity);	    # $severity_index

}

return @value_refs;
}




sub do_special($$$$$)
{
my ($section,	# MESSAGES
$subsection,	# empty
$item,		# SPECIAL
$values_ref,	# ([ $sub_key, @sub_values ], ...)
$func_par,
) = @_;

my @value_refs = @{$values_ref};
foreach my $ref (@value_refs)
{
my ($msg_nr, $text) = @{$ref};

GBSFILEGLO_error_exit( "Message number must be numeric ($msg_nr).")
if ($msg_nr !~ /^\d+$/);
}

return @value_refs;
}




sub do_metric($$$$$)
{
my ($section,	# METRICS
$subsection,	# empty
$item,		# ENABLE or IGNORE
$values_ref,	# [ @values ]
$func_par,
) = @_;

my @values = @{$values_ref};
foreach my $value (@values)
{

GBSFILEGLO_error_exit( "Duplicate Metric mnemonic '$value'")
if (exists $CUR_SCADEF_REF->{'.'}->{ENABLED_METRICS}->{$value});

my $enabled = 0;
if ($item eq 'ENABLE')
{
GBSFILEGLO_error_exit( "Invalid Metric mnemonic '$value'")
if (!SCAMET_name_is_valid( $value));
$enabled = 1;
}
$CUR_SCADEF_REF->{'.'}->{ENABLED_METRICS}->{$value} = $enabled;
}

return @values;
}

return 1;


