#=================================================
#
#  scmsmaint.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scmsmaint;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
SCMSMAINT_main
);
use strict;
use integer;

use glo::env;
use glo::ask;
use glo::slurp;
use glo::scm;
use gbsenv;
use gbsscm;
use system;
use files;




sub SCMSMAINT_main($$$);

sub scm_modify($$$);

sub scm_fix_ignore($$$);
sub fix_ignore_level($);
sub fix_ignore_level_recurse($);
sub fix_ignore_wild();
sub fix_ignore($);

sub scm_disable_enable($$$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $SAVED_SCMS;




sub SCMSMAINT_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

my $scms_state = (defined $SAVED_SCMS) ? 'Disabled' : 'Enabled';
my @scm_menu_items = (
[ "Modify SCMS settings",
\&scm_modify ],
[ "Fix SCM 'ignore' settings. (svn only)",
\&scm_fix_ignore ],
[ "Temporarily disable/enable SCM   ($scms_state)",
\&scm_disable_enable ],
);

ENV_say( 1, "GBS SCMS Tools");

ASK_menu( 'Select function to perform', \@scm_menu_items, $entries_ref);
}




sub scm_modify($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Modify SCMS settings");




my ($gbs_scms, $gbs_scms_repository, $gbs_scms_data) = GBSSCM_preset( 0);
($gbs_scms, $gbs_scms_repository, $gbs_scms_data) =
SCM_select( $gbs_scms, $GBS_ROOT_PATH, $gbs_scms_repository, $gbs_scms_data);

ENV_say( 1, "$gbs_scms: $gbs_scms_repository ($gbs_scms_data)");
if (ASK_YNQ( 'OK?', 'Y', 1) eq 'Y')
{



SCM_preset( $GBS_ROOT_PATH, [$gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_setenv( [ $gbs_scms, $gbs_scms_repository, $gbs_scms_data ]);
GBSSCM_connect();

SYSTEM_write( $GBS_ROOT_PATH,
scms	    => $gbs_scms,
scms_repository => $gbs_scms_repository,
scms_data	    => $gbs_scms_data,
);
}
}




sub scm_fix_ignore($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

GBSSCM_preset( 0);
if (SCM_get_current_id() eq 'svn')
{
ENV_say( 1, "Fix SCM 'ignore' (svn only)");
ENV_say( 1, "GBS_ROOT_PATH: $GBS_ROOT_PATH");
my @item_refs;
push @item_refs, [ 'Whole System. (may take a while!)',
sub { fix_ignore_level( $GBS_ROOT_PATH) } ];
push @item_refs, [ "Current SubSys ($GBS_SUBSYS)",
sub { fix_ignore_level( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS") } ]
if ($GBS_SUBSYS);
push @item_refs, [ "Current Component ($GBS_COMPONENT)",
sub { fix_ignore_level( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT") } ]
if ($GBS_COMPONENT);
push @item_refs, [ 'Wildcard',
\&fix_ignore_wild ];

ASK_menu( 'Select function',
\@item_refs,
$entries_ref);
} else
{
ENV_warn( 'Not SubVersioN');
}
}




sub fix_ignore_level($)
{
my ($spec) = @_;	# SYSTEM, SUBSYS, COMPONENT

GBSSCM_preset( 0);

ENV_say( 1, "Caching states...");
SCM_cache_states( $spec);
fix_ignore_level_recurse( $spec);
}




sub fix_ignore_level_recurse($)
{
my ($spec) = @_;

fix_ignore( $spec);
my @dirs = SLURP_dir_dirs( $spec, 0);
foreach my $dir (@dirs)
{
fix_ignore_level_recurse( "$spec/$dir");    #recurse
}
}




sub fix_ignore_wild()
{
ENV_say( 1, "WildCards relative to GBS_ROOT_PATH");
my $wild_spec;
do
{
$wild_spec = ASK_text( 'Enter Wildcard', '', 0, undef);
if ($wild_spec)
{
my $abs_wild_spec = "$GBS_ROOT_PATH/$wild_spec";
my @specs = glob( $abs_wild_spec);
if (@specs)
{
foreach my $spec (@specs)
{
if (-e $spec)
{
fix_ignore( $spec) if (-d _);
} else
{
ENV_say( 1, "No such file/dir '$spec'");
}
}
} else
{
ENV_say( 1, "No files/directories found!");
}
}
} while ($wild_spec);
}




sub fix_ignore($)
{
my ($spec) = @_;


my ($ignores_ref, $scm) = FILES_attrib( $spec);
if (defined $ignores_ref && $scm != 0)		# scm: 0 = no, 1 = yes, -1 = don't know
{
if (SCM_get_states( $spec) >= 0)	# state: 0 = checked in, 1 = checked_out
{
ENV_say( 1, "  $spec (@{$ignores_ref})");
SCM_set_ignore( $spec, $ignores_ref);
}
}
}




sub scm_disable_enable($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Temporarily disable/enable SCMS");
if (defined $SAVED_SCMS)
{



ENV_say( 1, "SCM is currently disabled");
if ( ASK_YN( "Enable SCM '$SAVED_SCMS'?", 'Y') eq 'Y')
{
GBSSCM_resume();
$SAVED_SCMS = undef;
}
} else
{



my $current_scms = ENV_getenv( 'GBS_SCMS');
if ($current_scms eq 'No_SCMS')
{
ENV_warn( "No current SCMS");
} else
{
ENV_say( 1, "SCM ('$current_scms') is currently enabled");
if ( ASK_YN( "Disable SCMS?", 'Y') eq 'Y')
{
GBSSCM_suspend();
$SAVED_SCMS = $current_scms;
my $root = GBSENV_perl_gbs_root();
SCM_preset( $root, [ 'no_scms', undef, undef ]);
}
}
}
}

return 1;
