#===============================================
#
#   scope.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package scope;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SCOPE_get_components
SCOPE_get_includes
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbsfile;




sub SCOPE_get_components($);
sub SCOPE_get_includes($@);

sub read_scope($);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my @GBS_ALL_TARGETS = ENV_getenv('GBS_ALL_TARGETS');
my %GBS_ALL_TARGETS = map { $_ => 1 } ENV_getenv('GBS_ALL_TARGETS');

my %TARGET_TOKENS = (
'='     => '=',
'=='    => '=',
'!'     => '!',
'!='    => '!',
);

my %SCOPE_CACHE;





sub SCOPE_get_components($)
{
my ($component) = @_;

return read_scope( $component);
}




sub SCOPE_get_includes($@)
{
my ($component,
@sub_dirs
) = @_;
my @scope_dirs;

foreach my $component (read_scope( $component))
{
foreach my $sub_dir (@sub_dirs)
{
push @scope_dirs, "../../$component$sub_dir";
}
}

return @scope_dirs;
}




sub read_scope($)
{
my ($component) = @_;
my @components;


if (exists( $SCOPE_CACHE{$component}))
{
@components = @{$SCOPE_CACHE{$component}};
} else
{
my $gbs_comp_root = "$GBS_ROOT_PATH/dev/$ENV{GBS_SUBSYS}/comp";
my $scope_file = "$gbs_comp_root/$component/scope";
if (GBSFILE_open( $scope_file, 0, undef, ENV_getenv( 'GBS_TARGET')))
{
my $line = GBSFILE_get_line();
while ($line)
{

if (-d "$gbs_comp_root/$line")
{
push @components, $line;
}  else
{
GBSFILE_error( "No such Component '$line'");
}
$line = GBSFILE_get_line();
}
$SCOPE_CACHE{$component} = [ @components ];
}
}

return @components;
}

return 1;

