#==============================================================
#
#   swc.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package swc;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWC_validate
SWC_set
SWC_entry
SWC_exit
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbs;
use gbsrc;
use gbsglo;




sub SWC_validate($$$);
sub SWC_set($$$$);
sub SWC_entry($$$);
sub SWC_exit($$$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();




sub SWC_validate($$$)
{
my ($root_path,
$subsys,
$component,  # '.' = current;
) = @_;
my $new_component = '';



my $full_gbs = GBSGLO_subsystem_is_full_gbs( $subsys, $root_path);
if ($component eq '.')
{
if ($full_gbs)
{
my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');

if ($root_path eq $GBS_ROOT_PATH && $subsys eq $gbs_subsys)
{
$component = ENV_getenv( 'GBS_COMPONENT');
}
if ($component eq '' || $component eq '.')
{
$component = GBSRC_get_subsys( $root_path, $subsys, 'component');
if ($component ne '' && ! -d "$root_path/dev/$subsys/comp/$component/src")
{
ENV_error_exit( "Component <$component> does not exist (anymore)");
}
}
} else
{
$component = '';
}
}

if ($component ne '')
{
if ($full_gbs)
{
my $comp_dir = "$root_path/dev/$subsys/comp";
if (GBSGLO_component_exists( $root_path, $subsys, $component) &&
-d "$comp_dir/$component/src" && -d "$comp_dir/$component/loc")
{
$new_component = $component;
} else
{
ENV_error_exit( "'$component' is not a Component in this SubSystem");
}
} else
{
ENV_error_exit( "Cannot specify a component ($component) for Non GBS SubSystem");
}
}

return $new_component;
}




sub SWC_set($$$$)
{
my ($root_path,
$subsys,
$component,
$must_write,
) = @_;

ENV_debug( "Set $root_path: $subsys: $component");

my $gbs_component = ENV_getenv( 'GBS_COMPONENT');
if ($component ne $gbs_component)
{
my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');

if (GBSENV_mode_is_interactive())
{
if ($gbs_subsys ne '' && $must_write && GBSGLO_subsystem_is_full_gbs( $gbs_subsys, $root_path))
{

GBSRC_write_subsys( $root_path, $gbs_subsys, component => $component);
}
}




if ($component eq '')
{
GBSENV_setenv( GBS_COMPONENT	=> '');
GBSENV_setenv( GBS_COMPONENT_PATH	=> '');
} else
{
my $component_path = "$root_path/dev/$gbs_subsys/comp/$component";

GBSENV_setenv( GBS_COMPONENT	=> $component);
GBSENV_setenv( GBS_COMPONENT_PATH	=> $component_path);
}
}
}




sub SWC_entry($$$)
{
my ($root_path,
$subsys,
$component,
) = @_;
my @lines;

push @lines, GBS_set_title( $component, $subsys, $root_path);

return @lines;
}




sub SWC_exit($$$)
{
my ($root_path,
$subsys,
$component,
) = @_;
my @lines;

push @lines, GBS_set_title( $subsys, $root_path);

return @lines;
}

return 1;


