#==============================================================
#
#   swsglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package swglo;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWGLO_validate
SWGLO_new_cwd
);

use strict;
use integer;

use glo::env;
use swr;
use sws;
use swc;
use swt;
use gbsenv;
use gbsrc;




sub SWGLO_validate($;$$$);
sub SWGLO_new_cwd($$$);




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();




sub SWGLO_validate($;$$$)
{
my ($target,	    # '', '-', '.', <target>
$component,	    # '', '-', '.', <component>
$subsys,	    # '', '-', '.', <subsys>
$root,		    # '', '-', '.', '!!', '!<os_spec>', <number>, <os_spec>
) = @_;
my ($new_target, $new_component, $new_subsys, $new_root, $new_audit);

if (0)
{
print "** ROOT   =$root\n"
if (defined $root);
print "** SUBSYS =$subsys\n"
if (defined $subsys);
print "** COMP   =$component\n"
if (defined $component);
print "** TARGET =$target\n"
if (defined $target);
print "**\n";
}




if (defined $root)
{

if ($root eq '-')
{
$new_root = '';
$subsys = '-';
$component = '-';
$target = '-';	# only for swr!
} elsif ($root eq '!!')
{
$new_root = SWR_validate( $CWD);
} elsif (substr( $root, 0, 1) eq '!')
{
my $this_dir = ($root eq '!') ? $CWD : substr( $root, 1);
($new_root, my $found_subsys, my $found_component) = SWR_find_root( $this_dir);
$subsys = $found_subsys
if ($subsys eq '.' || $subsys eq '');
$component = $found_component
if ($component eq '.' || $component eq '');
} else	# $root eq '.', <number> or <spec>
{
$new_root = SWR_validate( $root);
SWR_validate_base( $new_root);
}
if ($new_root ne '')
{
$new_audit = GBSRC_get_root( $new_root, 'audit');
} else
{
$new_audit = '';
}
} else
{
$new_audit = ENV_getenv( 'GBS_AUDIT');
$new_root = GBSENV_perl_gbs_root();
}




if (defined $subsys)
{

if ($subsys eq '-')
{
$new_subsys = '';
$component = '-';
} else # $subsys eq '.' or <name>
{
$subsys = '.'
if ($subsys eq '');
$new_subsys = SWS_validate( $new_root, $subsys);
}
} else
{
$new_subsys = ENV_getenv( 'GBS_SUBSYS');
}




if (defined $component)
{

if ($component eq '-')
{
$new_component = '';
} else # $component eq '.' or <name>
{
$component = '.'
if ($component eq '');
$new_component = SWC_validate( $new_root, $new_subsys, $component);
}
} else
{
$new_component = ENV_getenv( 'GBS_COMPONENT');
}




if (defined $target)
{

if ($target eq '-')
{
$new_target = '';
} else # $target eq '.' or <name>
{
$target = '.'
if ($target eq '');
$new_target = SWT_validate( $new_root, $new_subsys, $new_component, $target);
SWT_validate_dirs( $new_root, $new_subsys, $new_component, $new_target);
}
} else
{
$new_target = ENV_getenv( 'GBS_TARGET');
}

if (0)
{
print "** NEW_AUDIT  =$new_audit\n";
print "** NEW_ROOT   =$new_root\n";
print "** NEW_SUBSYS =$new_subsys\n";
print "** NEW_COMP   =$new_component\n";
print "** NEW_TARGET =$new_target\n";
}
return (wantarray) ? ($new_target, $new_component, $new_subsys, $new_root, $new_audit) : $new_target;
}




sub SWGLO_new_cwd($$$)
{
my ($root,
$subsys,
$component,
) = @_;
my $new_cwd;

if ($component ne '')
{
$new_cwd = "$root/dev/$subsys/comp/$component/src";
} elsif ($subsys ne '')
{
$new_cwd = "$root/dev/$subsys";
} elsif ($root ne '')
{
$new_cwd = $root;
} else  # $root eq ''
{





my $cwd = ENV_cwd();
if ($GBS_ROOT_PATH ne '' &&
substr( $cwd, 0, length($GBS_ROOT_PATH)) . '/' eq $GBS_ROOT_PATH . '/')
{
$new_cwd = ENV_parent_path( $GBS_ROOT_PATH);
ENV_say( 1, "Setting CWD to $new_cwd...");
} else
{
$new_cwd = $cwd;
}
}

return $new_cwd;
}

return 1;


