#==============================================================
#
#   sws.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package sws;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWS_validate
SWS_set
SWS_entry
SWS_exit
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbs;
use gbsrc;
use files;
use gbsglo;




sub SWS_validate($$);
sub SWS_set($$$);
sub SWS_entry($$);
sub SWS_exit($$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();




sub SWS_validate($$)
{
my ($root_path,
$subsys,	# '.' = current;
) = @_;
my $new_subsys = '';



if ($subsys eq '.')
{
$subsys = ENV_getenv( 'GBS_SUBSYS')
if ($root_path eq $GBS_ROOT_PATH);
if ($subsys eq '' || $subsys eq '.')
{
$subsys = GBSRC_get_root( $root_path, 'subsys');
ENV_warn( "No current SubSystem") if ($subsys eq '');
}
}

if ($subsys ne '')
{
if (GBSGLO_subsystem_exists( $root_path, $subsys))
{
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
my $subsys_dir = "$root_path/dev/$subsys";
if (FILES_check_gbs_subsys( $subsys_dir))
{
$new_subsys = $subsys;
} else
{
ENV_error_exit( "SubSys '$subsys' is not a valid Full-GBS SubSystem",
"One or more GBS directories and/or files are missing");
}
} else
{
$new_subsys = $subsys;
}
} else
{
ENV_error_exit( "SubSys '$subsys' does not exist (anymore)");
}
}

return $new_subsys;
}




sub SWS_set($$$)
{
my ($root_path,
$subsys,
$must_write,
) = @_;

ENV_debug( "Set $root_path: $subsys");

my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');
if ($subsys ne $gbs_subsys)
{
if (GBSENV_mode_is_interactive())
{
if ($must_write)
{

GBSRC_write_root( $root_path, subsys => $subsys);
}
}




my $is_full_gbs;
my $subsys_path;
my @env_refs = (



[ SUBSYS	    => 1, 1, 'f', sub { return $subsys } ],
[ SUBSYS_PATH   => 1, 1, 'f', sub { return $subsys_path } ],
[ EXPORT_PATH   => 1, 1, 'p', undef ],
[ FULL	    => 1, 1, 'f', sub { return $is_full_gbs } ],


[ GEN_PATH	    => 1, 0, 'p', undef ],
[ AUDIT_PATH    => 1, 0, 'p', undef ],
[ TOOLS_PATH    => 1, 0, 'p', undef ],
[ COMP_PATH	    => 1, 0, 'p', undef ],
[ IMPORT_PATH   => 1, 0, 'p', undef ],


[ APP_PATH	    => 0, 1, 'p', undef ],
);

if ($subsys eq '')
{



map { GBSENV_setenv( "GBS_$_->[0]", '') } @env_refs;
} else
{



$subsys_path = "$root_path/dev/$subsys";
$is_full_gbs = GBSGLO_subsystem_is_full_gbs( $subsys, $root_path);
foreach my $ref (@env_refs)
{
my ($name, $for_full, $for_not_full, $type, $arg) = @{$ref};

my $env_name = "GBS_$name";
my $value;
if ($is_full_gbs && $for_full ||
!$is_full_gbs && $for_not_full)
{
if ($type eq 'p')
{
my $path_part = lc( substr( $name, 0, -5));	    # minus "_PATH"
$value = "$root_path/$path_part";
} elsif ($type eq 'c')
{
$value = '';
} elsif ($type eq 'f')
{
$value = $arg->();
} else
{
ENV_fatal( "Impossible type ($type)");
}
} else
{
$value = '';
}

GBSENV_setenv( $env_name => $value );
}
}
}
}




sub SWS_entry($$)
{
my ($root_path,
$subsys,
) = @_;
my @lines;

push @lines, GBS_set_title( $subsys, $root_path);

return @lines;
}




sub SWS_exit($$)
{
my ($root_path,
$subsys,
) = @_;
my @lines;

push @lines, GBS_set_title( $root_path);

return @lines;
}

return 1;


