#==============================================================
#
#   swt.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package swt;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
SWT_validate
SWT_validate_dirs
SWT_set
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbsglo;
use gbsrc;
use system;




sub SWT_validate($$$$);
sub SWT_validate_dirs($$$$);
sub SWT_set($$$);

sub assume_target($$$$);




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();




sub SWT_validate($$$$)
{
my ($root_path,
$subsys,
$component,
$target,	# '.' = current;
) = @_;
my $new_target;


my $original_target = $target;




my @all_targets;
if ($root_path eq $GBS_ROOT_PATH)
{
@all_targets = ENV_getenv( 'GBS_TARGETS');
} else
{
@all_targets = SYSTEM_get( $root_path, 'targets');
}
if (@all_targets == 0)
{
ENV_error_exit( "No Targets defined for System '$root_path'");
} else
{



if ($target eq '.')
{
$target = ENV_getenv( 'GBS_TARGET')
if ($root_path eq $GBS_ROOT_PATH);
if ($target eq '.' || $target eq '')
{

$target = GBSRC_get_root( $root_path, 'target');

}
}

if ($target eq '')
{
$new_target = '';
} else
{



if (grep $target eq $_, @all_targets)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
$new_target = SWT_validate_dirs( $root_path, $subsys, $component, $target);
} else
{
$new_target = $target;
}
if ($new_target eq '' && $original_target eq '.')
{
$new_target = assume_target( $root_path, $subsys, $component, [@all_targets]);
}
} else
{
ENV_error_exit( "No such Target '$target' for System '$root_path'",
"Valid Targets are: @all_targets");
}
}
}

return $new_target;
}




sub SWT_validate_dirs($$$$)
{
my ($root_path,
$subsys,
$component,
$target,
) = @_;
my $new_target = $target;


if ($subsys ne '' && GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{



my $gen_dir = "$root_path/dev/$subsys/gen";
if (-d "$gen_dir/$target")
{



if ($component ne '')
{
my $bld_dir = "$root_path/dev/$subsys/comp/$component/bld";
if (!-d "$bld_dir/$target")
{
ENV_warn( "Component '$component' does not generate for Target '$target'");
$new_target = '';
}
}
} else
{
ENV_warn( "SubSystem '$subsys' does not generate for Target '$target'");
$new_target = '';
}
}

return $new_target;
}




sub SWT_set($$$)
{
my ($root_path,
$target,
$must_write,
) = @_;

ENV_debug( "Set $root_path ($target)");

if ($target ne ENV_getenv( 'GBS_TARGET'))
{
if (GBSENV_mode_is_interactive())
{
if ($must_write)
{

GBSRC_write_root( $root_path, target => $target);
}
}




ENV_setenv( GBS_TARGET => $target);
}
}




sub assume_target($$$$)
{
my ($root_path,
$subsys,
$component,
$all_targets_ref,
) = @_;
my $new_target = '';

if (scalar @{$all_targets_ref} == 1)
{
$new_target = $all_targets_ref->[0];
ENV_say( 1, "Single Target '$new_target' taken as default");
} else
{
if ($subsys eq '')
{
$new_target = $all_targets_ref->[0];
ENV_say( 1, "Target '$new_target' tentatively taken as default");
} else
{
if ($component eq '')
{
my $bld_root = "$root_path/dev/$subsys/comp/$component/bld";
foreach my $target (@{$all_targets_ref})
{
if (-d "$bld_root/$target")
{
$new_target = $target;
last;
}
}
}
if ($new_target eq '')
{
my $gen_root = "$root_path/dev/$subsys/gen";
foreach my $target (@{$all_targets_ref})
{
if (-d "$gen_root/$target")
{
$new_target = $target;
last;
}
}
}
if ($new_target eq '')
{
ENV_say( 1, "No Target directories found");
} else
{
ENV_say( 1, "Target '$new_target' tentatively assumed");
}
}
}

return $new_target;
}

return 1;


