#===============================================================
#
#   tool.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package tool;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
TOOL_get_all_tool_dirs
TOOL_set_envs
TOOL_get_derived_items
TOOL_get_section_items
);

use strict;
use integer;

use glo::env;
use gbsenv;
use gbssfile;





sub TOOL_get_all_tool_dirs($);
sub TOOL_set_envs($);
sub TOOL_get_derived_items($@);
sub TOOL_get_section_items($$@);

sub read_tool($);





my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_SYSTOOLS_PATH = "$GBS_ROOT_PATH/systools";

my $CUR_TOOL = '';
my $CUR_TOOL_REF;

my %TOOLS = ();



























my %INIT_DEFS = (
SET_W	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET_X	    => [ 'aa' , [],	   0, 1,     1,			 undef,	undef ],
SET		    => [ 'aa' , [],	   0, 2,     1,			 undef,	undef ],
SETPATH	    => [ 'aa' , [],	   0, 3,   [1],			 undef,	undef ],
CALL	    => [ 'aC' , [],	   0, 4, undef,			 undef,	undef ],    # Not implemented
);










my %SECTION_DEFS = (
INIT     => [ 'N', 0, 0, 0,   \%INIT_DEFS,  undef, undef ],
);





sub TOOL_get_all_tool_dirs($)
{
my ($gbs_root) = @_;


return GBSSFILE_get_all_structs( $gbs_root, 'tool');
}




sub TOOL_set_envs($)
{
my ($tool) = @_;

read_tool( $tool);




GBSSFILE_set_init_envs( $CUR_TOOL_REF);
}




sub TOOL_get_derived_items($@)
{
my ($tool,
@items,
) = @_;
my @values = ();


read_tool( $tool);

return GBSSFILE_get_section_items( $CUR_TOOL_REF, '.', @items);
}




sub TOOL_get_section_items($$@)
{
my ($tool,
$section,
@items,
) = @_;
my @values = ();


read_tool( $tool);

return GBSSFILE_get_section_items( $CUR_TOOL_REF, $section, @items);
}




sub read_tool($)
{
my ($tool) = @_;	    # undef = use current, '' = use EnvVar GBS_TOOL

if (defined $tool)
{
$tool = ENV_getenv( 'GBS_TOOL')
if ($tool eq '');

if ($tool ne $CUR_TOOL)
{

$CUR_TOOL = $tool;
$CUR_TOOL_REF = $TOOLS{$CUR_TOOL};
}
if (!defined $CUR_TOOL_REF)
{

$TOOLS{$CUR_TOOL} = {};
$CUR_TOOL_REF = $TOOLS{$CUR_TOOL};




my $tool_file = "$GBS_SYSTOOLS_PATH/$CUR_TOOL/tool";
my @inc_path = ( "$GBS_SYSTOOLS_PATH/$CUR_TOOL",
$GBS_SYSTOOLS_PATH);
GBSSFILE_parse( $CUR_TOOL_REF, $tool_file, $tool, \@inc_path, \%SECTION_DEFS);

}
}
}

return 1;

