#===============================================================
#
#   upgrade.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================

package upgrade;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw( UPGRADE_main
);

use strict;
use integer;

use glo::env;
use glo::ask;
use glo::scm;
use gbsenv;
use system;
use gbssfile;
use gbsscm;
use files;
use fix;




sub UPGRADE_main($$$);

sub upgrade_200_300();
sub update_system();




my $GBS_ROOT_PATH = GBSENV_perl_gbs_root();
my $GBS_VERSION = ENV_getenv( 'GBS_VERSION');
my $GBS_ROOT_VERSION = ENV_getenv( 'GBS_ROOT_VERSION');




sub UPGRADE_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref) = @_;

ENV_say( 1, "Upgrade");

ENV_say( 1, "Current GBS  version is $GBS_VERSION",
"Current ROOT version is $GBS_ROOT_VERSION");
if ($GBS_VERSION gt $GBS_ROOT_VERSION)  # use integer, so we ignore decimals!!
{
ENV_say( 1, "Version mismatch".
"Please read the release notes for any actions to be performed.",
"You may wish to exit this function in order to fix things first.",
"If you proceed with the Upgrade, this function will:",
"- Make the changes in the directory/file structure",
"- Check and repair the system.gbs file",
"- Check and repair the directory structure (as a final check)",
"- Set the ROOT-version to the current GBS version.",
"Make sure you have an UNMODIFIED work-area.",
"Immediately execute 'swr .' after the upgrade!",
"Immediately Checkin/Commit/Consolidate the work-area after the upgrade!");
if (ASK_YN( 'Upgrade?', 'Y') eq 'Y')
{
GBSSCM_preset( 0);
GBSSCM_connect();
upgrade_200_300();
my $ok = FIX_structure();
if ($ok)
{
if (ASK_YN( 'Set ROOT-version?', 'Y') eq 'Y')
{
update_system();
}
} else
{
ENV_warn( 'Cannot update Root-Version: Missing/Incomplete files/directories');
}
}
ENV_say( 1, "*** You MUST run 'swr .' before continuing with other GBS commands! ***");
} else
{
ENV_say( 1, "Root-Version is Up-to-date");
}

return;
}




sub upgrade_200_300()
{
ENV_say( 1, "Upgrading 2.xx to 3.xx...");




if (SYSTEM_fix( $GBS_ROOT_PATH))
{
ENV_say( 1, "system.gbs updated");
}




{



my @new_dirs = ("$GBS_ROOT_PATH/sysaudit", "$GBS_ROOT_PATH/sys");
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 1);	    # $pre_validated, $verbose




my @move_pair_refs;						    # [$old_spec, $new_spec ]

{



my @gbs_all_audits = GBSSFILE_get_all_structs( $GBS_ROOT_PATH, 'audit');
my @audit_specs = glob "$GBS_ROOT_PATH/sysgen/*/audit.gbs";
foreach my $audit_spec (@audit_specs)
{
my $dir = ENV_split_spec_d( $audit_spec);
my ($audit) = $dir =~ m!.*/(.*)!;
push @gbs_all_audits, $audit;
push @move_pair_refs, [ "$GBS_ROOT_PATH/sysgen/$audit", "$GBS_ROOT_PATH/sysaudit/$audit" ];
}
GBSENV_setenv( GBS_AUDITS => [ @gbs_all_audits ]);
ENV_say( 1, "GBS_AUDITS set to (@gbs_all_audits)");
}

{



push @move_pair_refs, ["$GBS_ROOT_PATH/sysgen/steps.gbs", "$GBS_ROOT_PATH/sys/steps.gbs"]
if (-e "$GBS_ROOT_PATH/sysgen/steps.gbs");
push @move_pair_refs, ["$GBS_ROOT_PATH/owners.gbs", "$GBS_ROOT_PATH/sys/owners.gbs"]
if (-e "$GBS_ROOT_PATH/owners.gbs");
push @move_pair_refs, ["$GBS_ROOT_PATH/sysgen/sysgen.gbs", "$GBS_ROOT_PATH/sys/sys.gbs"]
if (-e "$GBS_ROOT_PATH/sysgen/sysgen.gbs");
foreach my $spec (glob( "$GBS_ROOT_PATH/sysgen/*.bat"), glob("$GBS_ROOT_PATH/sysgen/*.sh"))
{
my $file = ENV_split_spec_f($spec);
push @move_pair_refs, [$spec, "$GBS_ROOT_PATH/sys/$file"];
}
}

SCM_move( \@move_pair_refs, 0, 0);				    # $pre_validated, $verbose


ENV_rename( "$GBS_ROOT_PATH/sysgen/steps.usr", "$GBS_ROOT_PATH/sys/steps.usr")
if (-e "$GBS_ROOT_PATH/sysgen/steps.usr");
ENV_rename( "$GBS_ROOT_PATH/owners.usr", "$GBS_ROOT_PATH/sys/owners.usr")
if (-e "$GBS_ROOT_PATH/owners.usr");
ENV_rename( "$GBS_ROOT_PATH/sysgen/sysgen.usr", "$GBS_ROOT_PATH/sys/sys.usr")
if (-e "$GBS_ROOT_PATH/sysgen/sysgen.usr");
}




{



my @delete_dirs;
push @delete_dirs, "$GBS_ROOT_PATH/silo/.gbs/html";
map { push @delete_dirs, "$GBS_ROOT_PATH/silo/.gbs/$_" } ENV_getenv( 'GBS_AUDITS');
@delete_dirs = grep ( -e $_, @delete_dirs);
SCM_remove_names( \@delete_dirs);




my @new_dirs;
push @new_dirs, "$GBS_ROOT_PATH/silo/.gbs/cache";
push @new_dirs, "$GBS_ROOT_PATH/silo/.gbs/cache/html";
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 0);	    # $pre_validated, $verbose
}
}




sub update_system()
{
ENV_say( 1, "Updating ROOT version...");
SYSTEM_write( $GBS_ROOT_PATH, root_version => $GBS_VERSION);
$GBS_ROOT_VERSION = $GBS_VERSION;
ENV_setenv( GBS_ROOT_VERSION => $GBS_ROOT_VERSION);
}

return 1;
